%====================================================================================
%                        USER MANUAL -- TGF software
%====================================================================================
%
% Version 2019-10-29
%
% Citation for this work:
% M. Yang, C. Hirt, R. Pail (2019) TGF: A New MATLAB-based Software for Terrain 
% Gravity Field Calculation.  Comput. Geosci., under review.

%-----------------------------------------------------------------------------------
% This is the user manual for TGF software which is developed for the calculations 
% of full-scale topographic and RTM gravitational field in the Matlab environment. 
%
% Please consult the Quick-start file for further information, including test data
% sets and simple examples for using the TGF software with its GUI or in batch mode.
%
% The full-scale topographic gravitational field calculated via the TGF software 
% is generated by the topographic masses boundied by the mean sea level and the 
% Earth surface, while the high-frequency RTM gravity field is caused by the 
% attraction of residual masses between the Earth's surafce and a reference 
% surface. 
%
% In the TGF software, the topographic masses around the calculation point are 
% divided into four zones for RTM gravity field calculation and five zones for 
% full-scale topographic gravity field. The mass elements of nearest zones around 
% calculation points are approximated by polyhedron and prism for precise 
% computation, while tesseroid and point-mass for efficient computation for the
% far-zone masses. Please refer to following papers for detailed information about 
% combination  of various mass elements, and numerical evaluation of gravity field 
% generated by polyhedron, tesseroid, and point mass:

% D. Tsoulis (2012) Analytical computation of the full gravity tensor of a 
% homogeneous arbitrarily shaped polyhedral source using line integral. 
% Geophysics 77(2), F1-F11.

% D. Nagy, G. Papp, J. Benedek (2000) The gravitational potential and its 
% derivatives for prism.  Journal of Geodesy 74, 552-560.

% D. Nagy, G. Papp, J. Benedek (2002) Corrections to "The gravitational potential 
% and its derivatives for prism". Journal of Geodesy 76, 475-475.

% T. Grombein, K. Seitz, B. Heck (2013) Optimized formulas for the gravitational
%  field of a tesseroid. Journal of Geodesy 87(7), 645-660.

% X.L. Deng, T. Grombein, W.B. Shen, B. Heck, K. Seitz (2016) Corrections to 
% “A comparison of the tesseroid, prism and point-mass approaches for mass 
% reductions in gravity field modelling? (Heck and Seitz, 2007) and “Optimized 
% formulas for the gravitational field of a tesseroid? (Grombein et al., 2013). 
% Journal of Geodesy 90, 585-587.

% M. Yang, C. Hirt, R. Tenzer, R. Pail (2018) Experiences with the use of 
% mass-density maps in residual gravity forward modelling. Stud. Geophys. Geod., 62.

% M. Yang, C. Hirt, M. Rexer, R. Pail, D.  Yamazaki (2019) The tree canopy effect 
% in gravity forward modelling. Geophys. J. Int., 219 (1), 
% https://doi.org/10.1093/gji/ggz264 . 

% M. Yang, C. Hirt, R. Pail (2019) TGF: A New MATLAB-based Software for Terrain 
% Gravity Field Calculation. Comput. Geosci., under review.

%% ----------------- Structures of the TGF software --------------------------------
% The TGF software works in two modes: in interactive mode with GUI interface and 
% in batch mode without the GUI interface. In interative mode, the TGF interface is 
% divided into four functional components: Computation Point, Forward Masses, 
% Gravitational Field and Output:

% The 'Computation Point' component specifies and located the file contains 
% computation point information. 

% The ’Forward Mass’ module defines the input file to define the topographic masses, 
% via the geometric upper and lower boundaries and density values. 

% Parameters required for gravitational field calculation are defined in the module 
% ‘Gravitational Field’.

% The outputs are defined in the 'Output' panel.

%% ---------------------- Input parameters -----------------------------------------
% ComPoints      --- in binary format, list of number of points and triplet of the 
%                    spherical/ellipsoidal coordinates (lat[-90, 90], 
%                    lon [-180, 180], height [in meters])
% DetailedDEM    --- Detailed DEM, in binary format
%                    --- [minlat maxlat reslat minlon maxlon reslon elevation] 
% DetailedREF    --- RTM reference surface with same resolution as DetailedDEM.bin
% MassDensity    --- Map of mass density values, same resolution as DetailedDEM.bin
%                    --- (from nearest neigb.)
%                    --- Only used when not a constant value is selected
%                    --- Unit g/cm^3
% TessDEM        --- DEM for tessroid zone
% TessREF        --- REF for tessroid zone
% TessDensity    --- mass density model for tessroid zone
% CoarseDEM      --- lower resolution DEM for far-zone 
% CoarseREF      --- lower resolution REF for far-zone 
% CoarseDensity  --- lower resolution mass density model for far-zone 
% GlobalDEM      --- Global DEM for full-scale topographic gravity field calculations   
% GlobalREF      --- Global reference DEM for full-scale topographic gravity field 
% calculations (mean sea level) In full-scale topographic gravity field calculations, 
% & constant density assumption is made in TGF software for GlobalDEM covred area.


% idensity  --- flag for mass density with values of 0, 1
%           --- 0, constant value is used
%           --- 1, density map is used

% ikind  --- flag for the type of modelling with values of 1, 2
%        --- 1, Topographic gravitational field
%        --- 2, RTM gravitational field

% itype  --- Specification of field functionals with values of 0, 1, 2, 4, 10, 103, 104
%        --- 0, height anomaly / geoid height (N) in cm  
%        --- 1, Dovs (arc-sec) and gravity disturbances (mGal)
%        --- 4, Dovs (arc-sec) and gravity anomaly (mGal)
%        --- 2, all gradients (6 elements) in E
%        --- 10, all functionals (geoid, Dovs, gravity disturbances, gradient tensor)
%        --- 103, geoid height, Dovs and gravity disturbances
%        --- 104, geoid height, Dovs and gravity anomaly

% rzones --- integral radius specifying the computation zones rzones = [r1 r2 r3 r4]
%        --- r1, radius for polyhedron (detailed DEM)
%        --- r2, radius for prism (detailed DEM)
%        --- r3, radius for tesseroid (Tesseroid DEM)
%        --- r4, radius for point-mass (coarse DEM)

% flag_earth --- flag of earth approximation with values of 0, 1
%            --- 0, spherical approximation
%            --- 1, ellipsoidal approximation

% e      --- mass density in g/cm^3 when constant density assumption is used

%% ------------------------ Output parameters --------------------------------------
% height anomaly in unit of centi-meter, Dov in unit of arc-sec, dg in unit of
% mGal, gradients in unit of E

%% ------------------------ Run TGF in batch mode ----------------------------------

% TGF_GUI (vstpar,ComPoints, DetailedDEM, DetailedREF, MassDensity, TessDEM, TessREF, ...
% TessDensity, CoarseDEM, CoarseREF, CoarseDensity,GlobalDEM,GlobalREF, outname, ...
% ikind, itype,idensity, flag_earth,rzones, e);

%% ---------------------- Notes on data formats ------------------------------------

% The input files (DEM grids) must be in binary format. You can generate your own 
% DEM input data in binary format using the Matlab-script Write_DEM.m. Please 
% consult this file for the file structure.

%% ---------------------- Assistance needed ? --------------------------------------

% Write an e-mail to Meng Yang via meng.yang@tum.de or meng.yang@vip.126.com. 

%% --------------------------------  DISCLAIMER ------------------------------------

% The TGF is research-only software. Neither TU Munich nor any of its staff accept 
% any liability in connection with the use of the software, data and models provided 
% here. Neither TU Munich nor any of its staff make any warranty of correctness, 
% fitness, completeness, usefulness and accuracy of the the software, data and 
% models for any intented or unintended purpose.

%-----------------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function TGF_GUI (vstpar,ComPoints, DetailedDEM, DetailedREF, MassDensity, TessDEM, TessREF, TessDensity, CoarseDEM, CoarseREF, CoarseDensity,GlobalDEM,GlobalREF, outname,ikind, itype,idensity, flag_earth,rzones, e)
path=pwd;
R1=0.8; G1=0.8; B1=0.8;
R2=0.95; G2=0.95; B2=0.95;

if nargin==0
%Main window 
    figure('units','pixels','numbertitle','off','name','TGF Software',...
        'color',[R1 G1 B1],'position',[300 100 600 700],...
        'tag','okno','menubar','none','tag','TC_main'); 
% Panel for computation points
    uipanel('Title','Computation Points','FontSize',12,'FontWeight','bold','BackgroundColor',[R1 G1 B1],...
        'HighlightColor',[R2 G2 B2],'Position',[0.05 0.88 0.88 0.10]);           
    uicontrol('Units','normalized','position',[0.08 0.90 0.30 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','Computation Point',...
        'Callback','TGF_GUI import_COM','tag','COM');               % computation points button command
    uicontrol('Units','normalized','position',[0.40 0.90 0.30 0.04],...
        'style','edit','tag','nameCOM',...
        'backgroundcolor',[R2 G2 B2]);                              %name of computation points   
    uicontrol('Units','normalized','position',[0.72 0.90 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','Display',...
        'Callback','TGF_GUI display_COM','tag','dis_COM');               % display computation points
    
% Panel for forward masses
    uipanel('Title','Forward Masses','FontSize',12,'FontWeight','bold','BackgroundColor',[R1 G1 B1],...
        'HighlightColor',[R2 G2 B2],'Position',[0.06 0.30 0.88 0.58]);  % panel for Forward masses
    
% Panel for forward masses -- detailed DEM  
    uicontrol('Units','normalized','position',[0.08 0.81 0.30 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','text','string','DetailedMasses','HorizontalAlignment','left'); 
    
    uicontrol('Units','normalized','position',[0.08 0.78 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','DetailedDEM',...
        'Callback','TGF_GUI import_DDEM');               % Detailed DEM button command  
    uicontrol('Units','normalized','position',[0.30 0.78 0.20 0.04],...
        'style','edit','tag','nameDDEM',...
        'backgroundcolor',[R2 G2 B2]);                   %name of detailed DEM
    uicontrol('Units','normalized','position',[0.52 0.78 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','displayDDEM',...
        'Callback','TGF_GUI display_DDEM');               % display detailed DEM  
                 %name of detailed DEM
    uicontrol('Units','normalized','position',[0.08 0.73 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','DetailedREF',...
        'Callback','TGF_GUI import_DREF');               % Detailed REF 
    uicontrol('Units','normalized','position',[0.30 0.73 0.20 0.04],...
        'style','edit','tag','nameDREF',...
        'backgroundcolor',[R2 G2 B2]);                  % name and path of detailed REF
    uicontrol('Units','normalized','position',[0.52 0.73 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','displayDREF',...
        'Callback','TGF_GUI display_DREF');               % display detailed REF    
    uicontrol('Units','normalized','position',[0.74 0.76 0.18 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','displayRTM',...
        'Callback','TGF_GUI display_DdH');               % display detailed RTM height
    
% Panel for tess masses  
    uicontrol('Units','normalized','position',[0.08 0.68 0.30 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','text','string','TessMasses','HorizontalAlignment','left'); 
 
    
% panel for tess masses -- Tess DEM
    uicontrol('Units','normalized','position',[0.08 0.65 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','TessDEM',...
        'Callback','TGF_GUI import_TDEM');               % tesseroid DEM button command  
    uicontrol('Units','normalized','position',[0.30 0.65 0.20 0.04],...
        'style','edit','tag','nameTDEM',...
        'backgroundcolor',[R2 G2 B2]);                   %name of tesseroid DEM
    uicontrol('Units','normalized','position',[0.52 0.65 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','displayTDEM',...
        'Callback','TGF_GUI display_TDEM');               % display tesseroid DEM  
    uicontrol('Units','normalized','position',[0.08 0.60 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','TessREF',...
        'Callback','TGF_GUI import_TREF');               % tesseroid REF
    uicontrol('Units','normalized','position',[0.30 0.60 0.20 0.04],...
        'style','edit','tag','nameTREF',...
        'backgroundcolor',[R2 G2 B2]);                  % name and path of tesseroid REF
    uicontrol('Units','normalized','position',[0.52 0.60 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','displayTREF',...
        'Callback','TGF_GUI display_TREF');             % display tesseroid REF      
    uicontrol('Units','normalized','position',[0.74 0.63 0.18 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','displayTRTM',...
        'Callback','TGF_GUI display_TdH');               % display tesseroid RTM height

% Panel for coarse masses 
    uicontrol('Units','normalized','position',[0.08 0.55 0.30 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','text','string','CoarseMasses','HorizontalAlignment','left');     
    
% panel for coarse masses -- coarse DEM
    uicontrol('Units','normalized','position',[0.08 0.52 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','CoarseDEM',...
        'Callback','TGF_GUI import_CDEM');               % coarse DEM button command  
    uicontrol('Units','normalized','position',[0.30 0.52 0.20 0.04],...
        'style','edit','tag','nameCDEM',...
        'backgroundcolor',[R2 G2 B2]);                   %name of coarse DEM
    uicontrol('Units','normalized','position',[0.52 0.52 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','displayCDEM',...
        'Callback','TGF_GUI display_CDEM');               % display coarse DEM  
    uicontrol('Units','normalized','position',[0.08 0.47 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','CoarseREF',...
        'Callback','TGF_GUI import_CREF');               % coarse REF
    uicontrol('Units','normalized','position',[0.30 0.47 0.20 0.04],...
        'style','edit','tag','nameCREF',...
        'backgroundcolor',[R2 G2 B2]);                  % name and path of coarse REF
    uicontrol('Units','normalized','position',[0.52 0.47 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','displayCREF',...
        'Callback','TGF_GUI display_CREF');             % display coarse REF      
    uicontrol('Units','normalized','position',[0.74 0.50 0.18 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','displayCRTM',...
        'Callback','TGF_GUI display_CdH');               % display coarse RTM height
    
 % panel for forward masses --- density model 
    uicontrol('Units','normalized','position',[0.08 0.42 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','Checkbox','string','idensity',...
        'tag','flag_density','value',0); %flag for density model       
    uicontrol('Units','normalized','position',[0.30 0.41 0.30 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','text','string','const-e [g/cm^3]'); 
    uicontrol('Units','normalized','position',[0.62 0.42 0.30 0.04],'FontSize',12,...
        'BackgroundColor',[R2 G2 B2],'style','edit','string','2.67','tag','const-e');
                                          % %constant density value
    uicontrol('Units','normalized','position',[0.08 0.37 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','MassDensity',...
        'tag','command_de','Callback','TGF_GUI import_D_e');  %  Detailed density          
    uicontrol('Units','normalized','position',[0.30 0.37 0.20 0.04],...
        'style','edit','tag','nameD_e',...
        'backgroundcolor',[R2 G2 B2]);  % path and name of detailed density % 
    uicontrol('Units','normalized','position',[0.52 0.37 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','CoarseDensity',...
        'tag','command_ce','Callback','TGF_GUI import_C_e');       % coarse density 
    uicontrol('Units','normalized','position',[0.74 0.37 0.18 0.04],...
        'style','edit','tag','nameC_e',...
        'backgroundcolor',[R2 G2 B2]);    % path and name of coarse density
    
    uicontrol('Units','normalized','position',[0.08 0.32 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','TessDensity',...
        'tag','command_de','Callback','TGF_GUI import_T_e');  %  tess density          
    uicontrol('Units','normalized','position',[0.30 0.32 0.20 0.04],...
        'style','edit','tag','nameT_e',...
        'backgroundcolor',[R2 G2 B2]);  % path and name of tess density % 
% gravity field definition
    uipanel('Title','Gravitational Field','FontSize',12,'FontWeight','bold','BackgroundColor',[R1 G1 B1],...
        'HighlightColor',[R2 G2 B2],'Position',[0.06 0.04 0.40 0.25]);  % panel for Forward masses
% gravity field definition -- components
    uicontrol('Units','normalized','position',[0.08 0.21 0.10 0.04],'FontSize',12,... 
        'BackgroundColor',[R1 G1 B1],'style','text','string','ikind'); 
    uicontrol('Units','normalized','position',[0.20 0.21 0.24 0.04],'FontSize',12,...
        'BackgroundColor',[R2 G2 B2],'style','popup','string','|1 -- topographic gravitational field|2 -- RTM gravitational fiel','tag','ikind','value',3);
                                          % ikind 
    uicontrol('Units','normalized','position',[0.08 0.16 0.10 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','text','string','itype'); 
    uicontrol('Units','normalized','position',[0.20 0.16 0.24 0.04],...
        'style','popup','string',...
        '|0 -- geoid height|1 -- xi, eta and gravity disturbance|4 -- xi, eta and gravity anomaly|2 -- tensors|10 -- geoid height, xi, eta, gravity disturbance and tensors|103 -- geoid height, xi, eta, and gravity disturbance|104 -- geoid height, xi, eta, and gravity anomaly',...
        'backgroundcolor',[R2 G2 B2],'tag','itype','value',6);  % itype 
    uicontrol('Units','normalized','position',[0.08 0.11 0.10 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','text','string','rzones'); 
    uicontrol('Units','normalized','position',[0.20 0.11 0.24 0.04],'FontSize',12,...
        'BackgroundColor',[R2 G2 B2],'style','edit','string',' 0, 0, 0,0 ','tag','rzones');
    uicontrol('Units','normalized','position',[0.08 0.06 0.10 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','text','string','iflag'); 
    uicontrol('Units','normalized','position',[0.20 0.06 0.24 0.04],...
        'style','popup','string',...
        '|0 -- spherical approximation|1 -- ellipsoidal approximation',...
        'backgroundcolor',[R2 G2 B2],'tag','iflag','value',2);  % iflag   
                                                                            
% output and grid2bin                                 
    uipanel('Title','Output','FontSize',12,'FontWeight','bold','BackgroundColor',[R1 G1 B1],...
        'HighlightColor',[R2 G2 B2],'Position',[0.51 0.15 0.43 0.14]);  % panel for Forward masses    
    uicontrol('Units','normalized','position',[0.53 0.21 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','Output file','Callback','TGF_GUI output_name','tag','output'); 
    uicontrol('Units','normalized','position',[0.53 0.16 0.20 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','grid2bin','Callback','TGF_GUI grid2bin','tag','grid2bin');    
% Process  
    uicontrol('Units','normalized','position',[0.51 0.11 0.20 0.04],'FontSize',12,'FontWeight','bold',...
         'BackgroundColor',[R1 G1 B1],'style','text','string','The TGF');
    uicontrol('Units','normalized','position',[0.51 0.045 0.43 0.065],'FontSize',8,'Foreground','blue',...
         'BackgroundColor',[R2 G2 B2],'style','text','string','  ','tag','process');           
% computation 
    uicontrol('Units','normalized','position',[0.45 0.00 0.10 0.04],'FontSize',12,...
        'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','OK',...
        'Callback','TGF_GUI RUN','tag','run');  %  Detailed density             
    set(findobj('tag','rzones'),'string','[0.01,0.02, 0.1, 0.5]');
    ComPoints=strcat(path,'\COM\ComPoints_5_points.bin');
    set(findobj('tag','nameCOM'),'string',ComPoints);
    DetailedDEM=strcat(path,'\DEM_MASS\DetailedDEM.bin');
    set(findobj('tag','nameDDEM'),'string',DetailedDEM);
    DetailedREF=strcat(path,'\DEM_MASS\DetailedREF.bin');
    set(findobj('tag','nameDREF'),'string',DetailedREF);
    TessDEM=strcat(path,'\DEM_MASS\TessDEM.bin');
    set(findobj('tag','nameTDEM'),'string',TessDEM);
    TessREF=strcat(path,'\DEM_MASS\TessREF.bin');
    set(findobj('tag','nameTREF'),'string',TessREF);
    CoarseDEM=strcat(path,'\DEM_MASS\CoarseDEM.bin');
    set(findobj('tag','nameCDEM'),'string',CoarseDEM);
    CoarseREF=strcat(path,'\DEM_MASS\CoarseREF.bin');
    set(findobj('tag','nameCREF'),'string',CoarseREF);
else
    switch (vstpar)
        case('import_COM')
            [ComPoints, Comdir]=uigetfile('*.*','Select Computation File');
            if ComPoints==0
            else
                ComPoints=strcat(Comdir,ComPoints);
                set(findobj('tag','nameCOM'),'string',ComPoints);
            end
        case('display_COM')
            ComPoints=get(findobj('tag','nameCOM'),'string');
            if isempty(ComPoints)
                error('Computation points file should not be empty.')
            else
                fid = fopen(ComPoints,'r');
                [data, cnt] = fread(fid, inf, 'double','native');
                fclose(fid);
                nrow=cnt/4;
                data=reshape(data,nrow,4);
                com_lat=data(:,2);
                com_lon=data(:,3);
                com_h=data(:,4);
                figure('units','pixels','numbertitle','off','name','Computation Points',...
                    'color',[R2 G2 B2],'position',[300 100 600 600]); 
                scatter(com_lon,com_lat,8,com_h,'filled')
                colorbar
                set(gca,'XDir','normal','YDir','normal')
                grid on
                xlabel('Longitude [deg]')
                ylabel('Latitude [deg]')
                title('Computation Points Elevation (m)')
                axis image
            end
        case('import_DDEM')
            [DetailedDEM, Demdir]=uigetfile('*.*','Select Computation File');
            if DetailedDEM==0
            else
                DetailedDEM=strcat(Demdir,DetailedDEM);
                set(findobj('tag','nameDDEM'),'string',DetailedDEM);
            end   
        case('import_DREF')
            [DetailedREF, Demdir]=uigetfile('*.*','Select Computation File');
            if DetailedREF==0
            else
                DetailedREF=strcat(Demdir,DetailedREF);
                set(findobj('tag','nameDREF'),'string',DetailedREF);
            end  
        case('display_DDEM')
            DetailedDEM=get(findobj('tag','nameDDEM'),'string');
            if isempty(DetailedDEM)
                error('DetailedDEM file should not be empty in such case.')
            else
                [minlat1, maxlat1, minlon1, maxlon1, reslat1, reslon1, z1] = read_DEM(DetailedDEM);
                x1=minlon1:reslon1:maxlon1;
                y1=minlat1:reslat1:maxlat1;
                [x1,y1]=meshgrid(x1,y1);
                figure('units','pixels','numbertitle','off','name','Detailed DEM',...
                    'color',[R2 G2 B2],'position',[300 100 600 600]); 
                imagesc(x1(1,:),y1(:,1),z1)
                colorbar
                set(gca,'XDir','normal','YDir','normal')
                grid on
                xlabel('Longitude [deg]')
                ylabel('Latitude [deg]')
                title('Detailed DEM (m)')
                axis image
            end
            
        case('display_DREF')
            DetailedREF=get(findobj('tag','nameDREF'),'string');
            if isempty(DetailedREF)
                error('DetailedREF file should not be empty in such case.')
            else
                [minlat1, maxlat1, minlon1, maxlon1, reslat1, reslon1, z1_ref] = read_DEM(DetailedREF);
                x1=minlon1:reslon1:maxlon1;
                y1=minlat1:reslat1:maxlat1;
                [x1,y1]=meshgrid(x1,y1);
                figure('units','pixels','numbertitle','off','name','Detailed REF',...
                    'color',[R2 G2 B2],'position',[300 100 600 600]); 
                imagesc(x1(1,:),y1(:,1),z1_ref)
                colorbar
                set(gca,'XDir','normal','YDir','normal')
                grid on
                xlabel('Longitude [deg]')
                ylabel('Latitude [deg]')
                title('Detailed REF (m)')
                axis image
            end
            
        case('display_DdH')
            DetailedREF=get(findobj('tag','nameDREF'),'string');
            DetailedDEM=get(findobj('tag','nameDDEM'),'string');
            if isempty(DetailedREF) | isempty(DetailedDEM)
                error('DetailedREF or DetailedDEM file should not be empty in such case.')
            else
                [minlat1, maxlat1, minlon1, maxlon1, reslat1, reslon1, z1_ref] = read_DEM(DetailedREF);
                [minlat1, maxlat1, minlon1, maxlon1, reslat1, reslon1, z1] = read_DEM(DetailedDEM);
                x1=minlon1:reslon1:maxlon1;
                y1=minlat1:reslat1:maxlat1;
                [x1,y1]=meshgrid(x1,y1);
                figure('units','pixels','numbertitle','off','name','Detailed RTM height',...
                    'color',[R2 G2 B2],'position',[300 100 600 600]); 
                imagesc(x1(1,:),y1(:,1),z1-z1_ref)
                colorbar
                set(gca,'XDir','normal','YDir','normal')
                grid on
                xlabel('Longitude [deg]')
                ylabel('Latitude [deg]')
                title('Detailed RTM height (m)')
                axis image
            end  
 % tess REF           
         case('import_TDEM')
            [TessDEM, Demdir]=uigetfile('*.*','Select Computation File');
            if TessDEM==0
            else
                TessDEM=strcat(Demdir,TessDEM);
                set(findobj('tag','nameTDEM'),'string',TessDEM);
            end   
        case('import_TREF')
            [TessREF, Demdir]=uigetfile('*.*','Select Computation File');
            if TessREF==0
            else
                TessREF=strcat(Demdir,TessREF);
                set(findobj('tag','nameTREF'),'string',TessREF);
            end  
        case('display_TDEM')
            TessDEM=get(findobj('tag','nameTDEM'),'string');
            if isempty(TessDEM)
                error('TessDEM file should not be empty in such case.')
            else
                [minlatt, maxlatt, minlont, maxlont, reslatt, reslont, zt] = read_DEM(TessDEM);
                xt=minlont:reslont:maxlont;
                yt=minlatt:reslatt:maxlatt;
                [xt,yt]=meshgrid(xt,yt);
                figure('units','pixels','numbertitle','off','name','Tess DEM',...
                    'color',[R2 G2 B2],'position',[300 100 600 600]); 
                imagesc(xt(1,:),yt(:,1),zt)
                colorbar
                set(gca,'XDir','normal','YDir','normal')
                grid on
                xlabel('Longitude [deg]')
                ylabel('Latitude [deg]')
                title('Tess DEM (m)')
                axis image
            end
            
        case('display_TREF')
            TessREF=get(findobj('tag','nameTREF'),'string');
            if isempty(TessREF)
                error('TessREF file should not be empty in such case.')
            else
                [minlatt, maxlatt, minlont, maxlont, reslatt, reslont, zt_ref] = read_DEM(TessREF);
                xt=minlont:reslont:maxlont;
                yt=minlatt:reslatt:maxlatt;
                [xt,yt]=meshgrid(xt,yt);
                figure('units','pixels','numbertitle','off','name','Tess REF',...
                    'color',[R2 G2 B2],'position',[300 100 600 600]); 
                imagesc(xt(1,:),yt(:,1),zt_ref)
                colorbar
                set(gca,'XDir','normal','YDir','normal')
                grid on
                xlabel('Longitude [deg]')
                ylabel('Latitude [deg]')
                title('Tess REF (m)')
                axis image
            end
            
        case('display_TdH')
            TessREF=get(findobj('tag','nameTREF'),'string');
            TessDEM=get(findobj('tag','nameTDEM'),'string');
            if isempty(TessREF) | isempty(TessDEM)
                error('TessREF or TessDEM file should not be empty in such case.')
            else
                [minlatt, maxlatt, minlont, maxlont, reslatt, reslont, zt_ref] = read_DEM(TessREF);
                [minlatt, maxlatt, minlont, maxlont, reslatt, reslont, zt] = read_DEM(TessDEM);
                xt=minlont:reslont:maxlont;
                yt=minlatt:reslatt:maxlatt;
                [xt,yt]=meshgrid(xt,yt);
                figure('units','pixels','numbertitle','off','name','Tess RTM height',...
                    'color',[R2 G2 B2],'position',[300 100 600 600]); 
                imagesc(xt(1,:),yt(:,1),zt-zt_ref)
                colorbar
                set(gca,'XDir','normal','YDir','normal')
                grid on
                xlabel('Longitude [deg]')
                ylabel('Latitude [deg]')
                title('Tess RTM height (m)')
                axis image
            end     
                 
        case('import_CDEM')
            [CoarseDEM, Demdir]=uigetfile('*.*','Select Computation File');
            if CoarseDEM==0
            else
                CoarseDEM=strcat(Demdir,CoarseDEM);
                set(findobj('tag','nameCDEM'),'string',CoarseDEM);
            end   
        case('import_CREF')
            [CoarseREF, Demdir]=uigetfile('*.*','Select Computation File');
            if CoarseREF==0
            else
                CoarseREF=strcat(Demdir,CoarseREF);
                set(findobj('tag','nameCREF'),'string',CoarseREF);
            end  
        case('display_CDEM')
            CoarseDEM=get(findobj('tag','nameCDEM'),'string');
            if isempty(CoarseDEM)
                error('CoarseDEM file should not be empty in such case.')
            else
                [minlat2, maxlat2, minlon2, maxlon2, reslat2, reslon2, z2] = read_DEM(CoarseDEM);
                x2=minlon2:reslon2:maxlon2;
                y2=minlat2:reslat2:maxlat2;
                [x2,y2]=meshgrid(x2,y2);
                figure('units','pixels','numbertitle','off','name','Coarse DEM',...
                    'color',[R2 G2 B2],'position',[300 100 600 600]); 
                imagesc(x2(1,:),y2(:,1),z2)
                colorbar
                set(gca,'XDir','normal','YDir','normal')
                grid on
                xlabel('Longitude [deg]')
                ylabel('Latitude [deg]')
                title('Coarse DEM (m)')
                axis image
            end
            
        case('display_CREF')
            CoarseREF=get(findobj('tag','nameCREF'),'string');
            if isempty(CoarseREF)
                error('DetailedREF file should not be empty in such case.')
            else
                [minlat2, maxlat2, minlon2, maxlon2, reslat2, reslon2, z2_ref] = read_DEM(CoarseREF);
                x2=minlon2:reslon2:maxlon2;
                y2=minlat2:reslat2:maxlat2;
                [x2,y2]=meshgrid(x2,y2);                
                figure('units','pixels','numbertitle','off','name','Coarse REF',...
                    'color',[R2 G2 B2],'position',[300 100 600 600]); 
                imagesc(x2(1,:),y2(:,1),z2_ref)
                colorbar
                set(gca,'XDir','normal','YDir','normal')
                grid on
                xlabel('Longitude [deg]')
                ylabel('Latitude [deg]')
                title('Coarse REF (m)')
                axis image
            end
            
        case('display_CdH')
            CoarseREF=get(findobj('tag','nameCREF'),'string');
            CoarseDEM=get(findobj('tag','nameCDEM'),'string');
            if isempty(CoarseREF) | isempty(CoarseDEM)
                error('CoarseREF or CoarseDEM file should not be empty in such case.')
            else
                [minlat2, maxlat2, minlon2, maxlon2, reslat2, reslon2, z2_ref] = read_DEM(CoarseREF);
                [minlat2, maxlat2, minlon2, maxlon2, reslat2, reslon2, z2] = read_DEM(CoarseDEM);
                x2=minlon2:reslon2:maxlon2;
                y2=minlat2:reslat2:maxlat2;
                [x2,y2]=meshgrid(x2,y2);  
                figure('units','pixels','numbertitle','off','name','Coarse RTM Height',...
                    'color',[R2 G2 B2],'position',[300 100 600 600]); 
                imagesc(x2(1,:),y2(:,1),z2-z2_ref)
                colorbar
                set(gca,'XDir','normal','YDir','normal')
                grid on
                xlabel('Longitude [deg]')
                ylabel('Latitude [deg]')
                title('Coarse RTM height (m)')
                axis image
            end   
            
        case('import_D_e')   
            [Massdensity, Demdir]=uigetfile('*.*','Select Computation File');
            if Massdensity==0
            else
                Massdensity=strcat(Demdir,Massdensity);
                set(findobj('tag','nameD_e'),'string',Massdensity);
            end  
            
        case('import_C_e')   
            [Coarsedensity, Demdir]=uigetfile('*.*','Select Computation File');
            if Coarsedensity==0
            else
                Coarsedensity=strcat(Demdir,Coarsedensity);
                set(findobj('tag','nameC_e'),'string',Coarsedensity);
            end  
            
        case('import_T_e')   
            [Tessdensity, Demdir]=uigetfile('*.*','Select Computation File');
            if Tessdensity==0
            else
                Tessdensity=strcat(Demdir,Tessdensity);
                set(findobj('tag','nameT_e'),'string',Tessdensity);
            end 
            
        case('output_name')
            [outname, outdir]=uiputfile('*.*');
            if outname==0
            else
                outname=strcat(outdir,outname);
                set(findobj('tag','output'),'userdata',outname);
            end
        case('grid2bin')
            figure('units','pixels','numbertitle','off','name',...
                'grid2bin','color',[R1 G1 B1],'position',...
                [320 150 400 250],'tag','grid2binfun','menubar','none');
            uicontrol('Units','normalized','position',[0.08 0.84 0.26 0.12],'FontSize',12,...
                'BackgroundColor',[R1 G1 B1],'style','text','string','minlat(deg)');
            uicontrol('Units','normalized','position',[0.08 0.76 0.26 0.12],'FontSize',12,...
                'BackgroundColor',[R2 G2 B2],'style','edit','string','  ','tag','grid2bin_minlat');      
            uicontrol('Units','normalized','position',[0.38 0.84 0.26 0.12],'FontSize',12,...
                'BackgroundColor',[R1 G1 B1],'style','text','string','reslat(deg)');
            uicontrol('Units','normalized','position',[0.38 0.76 0.26 0.12],'FontSize',12,...
                'BackgroundColor',[R2 G2 B2],'style','edit','string','  ','tag','grid2bin_reslat');
            uicontrol('Units','normalized','position',[0.68 0.84 0.26 0.12],'FontSize',12,...
                'BackgroundColor',[R1 G1 B1],'style','text','string','maxlat(deg)');
            uicontrol('Units','normalized','position',[0.68 0.76 0.26 0.12],'FontSize',12,...
                'BackgroundColor',[R2 G2 B2],'style','edit','string','  ','tag','grid2bin_maxlat');
            
            uicontrol('Units','normalized','position',[0.08 0.60 0.26 0.12],'FontSize',12,...
                'BackgroundColor',[R1 G1 B1],'style','text','string','minlon(deg)');
            uicontrol('Units','normalized','position',[0.08 0.50 0.26 0.12],'FontSize',12,...
                'BackgroundColor',[R2 G2 B2],'style','edit','string','  ','tag','grid2bin_minlon');      
            uicontrol('Units','normalized','position',[0.38 0.60 0.26 0.12],'FontSize',12,...
                'BackgroundColor',[R1 G1 B1],'style','text','string','reslon(deg)');
            uicontrol('Units','normalized','position',[0.38 0.50 0.26 0.12],'FontSize',12,...
                'BackgroundColor',[R2 G2 B2],'style','edit','string','  ','tag','grid2bin_reslon');
            uicontrol('Units','normalized','position',[0.68 0.60 0.26 0.12],'FontSize',12,...
                'BackgroundColor',[R1 G1 B1],'style','text','string','maxlon(deg)');
            uicontrol('Units','normalized','position',[0.68 0.50 0.26 0.12],'FontSize',12,...
                'BackgroundColor',[R2 G2 B2],'style','edit','string','  ','tag','grid2bin_maxlon');
            
            
            uicontrol('Units','normalized','position',[0.22 0.34 0.26 0.12],'FontSize',12,...
                'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','Grid_file',...
                'Callback','TGF_GUI grid2bin_grid');  %  grid file 
            uicontrol('Units','normalized','position',[0.54 0.34 0.26 0.12],'FontSize',10,...
                'BackgroundColor',[R2 G2 B2],'style','edit','string','  ',...
                'tag','grid2bin_grid');  %  grid file
 
            uicontrol('Units','normalized','position',[0.45 0.10 0.12 0.12],'FontSize',12,...
                'BackgroundColor',[R1 G1 B1],'style','pushbutton','string','OK',...
                'Callback','TGF_GUI grid2bin_RUN','tag','grid2bin_run');  %  Detailed density  
                       
        case('grid2bin_grid')
            [gridname, griddir]=uigetfile('*.*','Select Computation File');
            if gridname==0
            else
                gridname=strcat(griddir,gridname);
                set(findobj('tag','grid2bin_grid'),'string',gridname);
            end 
        case('grid2bin_RUN')
            miny=str2num(get(findobj('tag','grid2bin_minlat'),'string'));
            res_lat=str2num(get(findobj('tag','grid2bin_reslat'),'string'));
            maxy=str2num(get(findobj('tag','grid2bin_maxlat'),'string'));
            minx=str2num(get(findobj('tag','grid2bin_minlon'),'string'));
            res_lon=str2num(get(findobj('tag','grid2bin_reslon'),'string'));
            maxx=str2num(get(findobj('tag','grid2bin_maxlon'),'string'));
            gridname=get(findobj('tag','grid2bin_grid'),'string');
            % read grid file
            z=load(gridname);
            data(1)=miny;
            data(2)=maxy;
            data(3)=minx;
            data(4)=maxx;
            data(5)=res_lat;
            data(6)=res_lon; 
            z=z';
            [m, n]=size(z);
            mn=m*n;
            data(7:mn+6)=z(:);
            
            if find(gridname=='.')>0
                    gridname=gridname(1:(find(gridname=='.')-1));
                    gridout=strcat(gridname,'.bin');
                    fid = fopen(gridout,'wb');
                    fwrite(fid, data, 'double','native');
                    fclose(fid);
            end
            

        case('RUN')
           
            if nargin==1    %working with the GUI
                GUI=1;
                
            else           % working without the GUI
                GUI=0;
            end

            if GUI==1      %if Working with the GUI
                set(findobj('tag','process'),'string','is reading parameters.'); drawnow;
                idensity=get(findobj('tag','flag_density'),'value'); % density mode
                if idensity==0
                    e=str2num(get(findobj('tag','const-e'),'string'));
                else
                    
                    MassDensity=get(findobj('tag','nameD_e'),'string');
                    CoarseDensity=get(findobj('tag','nameC_e'),'string');
                    TessDensity=get(findobj('tag','nameT_e'),'string');
                    
                    if isempty(MassDensity) | isempty(CoarseDensity)| isempty(TessDensity)
                        error('please input a density model while not a constant value is selected');
                    end
                end                                                    
                ComPoints=get(findobj('tag','nameCOM'),'string'); % computation point
                DetailedDEM=get(findobj('tag','nameDDEM'),'string'); % detailed DEM
                DetailedREF=get(findobj('tag','nameDREF'),'string'); % detailed REF
                TessDEM=get(findobj('tag','nameTDEM'),'string'); % tess DEM
                TessREF=get(findobj('tag','nameTREF'),'string'); % tess REF
                CoarseDEM=get(findobj('tag','nameCDEM'),'string');   % coarse DEM
                CoarseREF=get(findobj('tag','nameCREF'),'string');   % coarse REF
                
                ikind=get(findobj('tag','ikind'),'value')-1;         
                % 1 -- Topographical gravitational % 2 --RTM gravitational 
                itype1=get(findobj('tag','itype'),'value')-1;
                if itype1==1
                    itype=0;              %geoid height
                else if itype1==2 
                        itype=1;           % xi, eta, gravity disturbance
                    else if itype1==3
                            itype=4;      % xi eta gravity anomaly
                        else if itype1==4
                                itype=2;   %tensors;
                            else if itype1==5
                                    itype=10; % all components
                                else if itype1==6
                                        itype=103;  % geoid, xi,eta,gravity disturbance
                                    else if itype1==7
                                            itype=104; %geoid xi eta gravity anomaly
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
                clear itype1
                
                rzones=str2num(get(findobj('tag','rzones'),'string'));
                flag_earth=get(findobj('tag','iflag'),'value')-2;      
                outname=get(findobj('tag','output'),'userdata');
            else if GUI==0
                   if idensity==1 
                       if isempty(MassDensity) | isempty(CoarseDensity)
                           error('please input a density model while not a constant value is selected');
                       end
                   end
                   
                   if idensity==0
                       if isempty(e)
                           error('please input constant density value while a constant value is selected');
                       end
                   end  
                   
                   if ikind~=1 & ikind~=2
                       error('Please check ikind value: ikind can only be either 1 (for topographical gravitational field) or 2 (for RTM gravitational field)!');
                   end
                   
                   if itype~=0 & itype~=1 & itype~=4 & itype~=2 & itype~=10 & itype~=103 & itype~=104
                       error('Please check itype value: 0 for geoid; 1 for xi, eta and gravity disturbance; 4 for xi, eta and gravity anomaly; 2 for tensors; 10 for all components; 103 for geoid, xi,eta, gravity disturbance; 104 for geoid, xi, eta, gravity anomaly!');
                   end
                   
                end 
            end
            if isempty(outname)
               c=clock; 
               outname=strcat(num2str(c(1)),num2str(c(2),'%02d'),num2str(c(3),'%02d'),'_',num2str(c(4),'%02d'),num2str(c(5),'%02d'),num2str(floor(c(4)),'%02d'),'.dat');
            end
            
            if find(outname=='.')>0
                output_report=outname(1:(find(outname=='.')-1));
            else
                output_report=outname;
                outname=strcat(outname,'.dat');
            end
            output_report=strcat(output_report,'_report.txt');   

            %constant parameter definition
            Pi = 3.141592654;
            G = 6.67384e-3;   % gravitational constant
            
            %Load computation point
            if GUI==1
                 set(findobj('tag','process'),'string','is loading the computation file.'); drawnow;
            end
            fid = fopen(ComPoints,'r');
            if  isempty(ComPoints)                   % evaluate ComPoints and make a statement
                error('Undefined computation file.')
            end
            if fid<0
                error('The defined computation file is non-existent.')
            end
            
            [data, cnt] = fread(fid, inf, 'double','native'); % format [N lat lon H]
            fclose(fid);
            nrow=cnt/4;
            data=reshape(data,nrow,4);
            com_lat=data(:,2);
            com_lon=data(:,3);
            com_h=data(:,4);   
            num=data(end,1);	
            
            com_h(com_h<0)=1; % negative heights are corrected with positive constant height.
            H_com=com_h; 
            % integration radius
            r1 = rzones(1);            % radius for polyhedron
            r2 = rzones(2);            % radius for prism
            r3 = rzones(3);            % radius for tesseroid
            r4 = rzones(4);            % radius for point-mass
            clear data cnt
            
            if r1>r2                   % r2>=r1
                r2=r1;
            end
            
            
            if r2>r3                   % r3>=r2
                r3=r2;
            end

            if r3>r4                   % r4>=r3
                r4=r3;
            end
            %% write output report --- parameter definition and computation point
            fid=fopen(output_report,'w');
            p='=========================    TGF_GUI project report   =========================';
            formatSpec='%s\n';
            fprintf(fid,formatSpec,p);
            fprintf(fid,'\n');
            fprintf(fid,'%30s','Generating_data:');
            fprintf(fid,'%14s\n',datestr(now,1));
            fprintf(fid,'%30s','Product_type:');
            if ikind==1   
                fprintf(fid,'%35s\n','topographic gravitational field');
            else if ikind==2
                   fprintf(fid,'%26s\n','RTM gravitational field');
                end
            end
            fprintf(fid,'%30s','Functional:');
            fprintf(fid,'%15s %d %s\n','itype=',itype,'(please refer to handbook)');
            fprintf(fid,'%30s','Gravity field type:');
            fprintf(fid,'%15s %d %s\n','ikind=',ikind,'(please refer to handbook)');
            fprintf(fid,'%30s','Earth approximation:');
            fprintf(fid,'%15s %d %s\n','iflag=',flag_earth,'(please refer to handbook)');
            fprintf(fid,'%30s','Four zones radius [deg]:');
            fprintf(fid,'%7.2f, %7.2f, %7.2f, %7.2f\n',rzones);
            fprintf(fid,'\n');
            p='----------------------  computation point information  -----------------------';
            formatSpec='%s\n';
            fprintf(fid,formatSpec,p);
            fprintf(fid,'\n');
            fprintf(fid,'%30s%10.4f\n','comlat_min:',min(com_lat)); 
            fprintf(fid,'%30s%10.4f\n','comlat_max:',max(com_lat)); 
            fprintf(fid,'%30s%10.4f\n','comlon_min:',min(com_lon)); 
            fprintf(fid,'%30s%10.4f\n','comlon_max:',max(com_lon)); 
			fprintf(fid,'%30s%10d\n','number of computation points:',num); 
            fprintf(fid,'\n');
            fclose(fid);            
            
            if GUI==1
               set(findobj('tag','process'),'string','is reading DEM and density information.'); drawnow;
            end
            %read DEM-- Detailed DEM 
            [minlat1, maxlat1, minlon1, maxlon1, reslat1, reslon1, z1] = read_DEM(DetailedDEM);
            if minlat1>min(com_lat)-r2 | maxlat1<max(com_lat)+r2 | minlon1>min(com_lon)-r2 | maxlon1<max(com_lon)+r2
                error('Region of detailed DEM should be larger than integral area of innerzones!')
            end
            
            % read DEM -- tess DEM
            [minlatt, maxlatt, minlont, maxlont, reslatt, reslont, zt] = read_DEM(TessDEM);
            if minlatt>min(com_lat)-r3 | maxlatt<max(com_lat)+r3 | minlont>min(com_lon)-r3 | maxlont<max(com_lon)+r3
                error('Region of detailed DEM should be larger than integral area of innerzones!')
            end
           
            %read DEM -- Coarse DEM
            [minlat2, maxlat2, minlon2, maxlon2, reslat2, reslon2, z2] = read_DEM(CoarseDEM);     
            if minlat2>min(com_lat)-r4 | maxlat2<max(com_lat)+r4 | minlon2>min(com_lon)-r4 | maxlon2<max(com_lon)+r4
                error('Region of coarse DEM should be larger than integral area of outerzone!')
            end
            %DEMs' coordinates
            x1=minlon1:reslon1:maxlon1+1e-7;
            y1=minlat1:reslat1:maxlat1+1e-7;
            [x1,y1]=meshgrid(x1,y1);
            
            xt=minlont:reslont:maxlont+1e-7;
            yt=minlatt:reslatt:maxlatt+1e-7;
            [xt,yt]=meshgrid(xt,yt);

            x2=minlon2:reslon2:maxlon2+1e-7;
            y2=minlat2:reslat2:maxlat2+1e-7;
            [x2,y2]=meshgrid(x2,y2);
            
            if ikind==1         %Reference surface for topographic gravitational field, mean sea level
                [m, n]=size(x1);
                z1_ref(1:m,1:n)=0; 
    
                [m, n]=size(xt);
                zt_ref(1:m,1:n)=0; 
                
                [m, n]=size(x2);
                z2_ref(1:m,1:n)=0; 
            else if ikind==2    %Reference surface for RTM gravitational field
                    CoarseREF=strcat(CoarseREF);
                    DetailedREF=strcat(DetailedREF);
                    TessREF=strcat(TessREF);
                    [minlat1, maxlat1, minlon1, maxlon1, reslat1, reslon1, z1_ref] = read_DEM(DetailedREF);
                    [minlat2, maxlat2, minlon2, maxlon2, reslat2, reslon2, z2_ref] = read_DEM(CoarseREF);
                    [minlatt, maxlatt, minlont, maxlont, reslatt, reslont, zt_ref] = read_DEM(TessREF);
                 end
            end
            
            if ikind==1  %Global DEM and global REF for full-scale gravitational field calculation
                if ~isempty(GlobalDEM)
                    [minyg, maxyg, minxg, maxxg, resolution_g, resolution_g, zg] = read_DEM(GlobalDEM); 
                    [minyg, maxyg, minxg, maxxg, resolution_g, resolution_g, zg_ref] = read_DEM(GlobalREF);  
                    xg=minxg:resolution_g:maxxg+1e-7;
                    yg=minyg:resolution_g:maxyg+1e-7;
                    [xg,yg]=meshgrid(xg,yg);
                end
            end
                    
            % read mass-density model if idensity=1
            if idensity==1
                if ~isempty(MassDensity)  %mass density fit with DetailedDEM and DetailedREF
                    [minlat1, maxlat1, minlon1, maxlon1, reslat1, reslon1, e_d] = read_DEM(MassDensity);
                else
                    [m, n]=size(z1);
                    e_d(1:m,1:n)=e; 
                end
                
                if ~isempty(TessDensity)  %mass density fit with TessDEM and TessREF
                    [minlatt, maxlatt, minlont, maxlont, reslatt, reslont, e_t] = read_DEM(TessDensity);
                else
                    [m, n]=size(zt);
                    e_t(1:m,1:n)=e; 
                end
                
                if ~isempty(CoarseDensity)  %mass density fit with CoarseDEM and CoarseREF
                    [minlat2, maxlat2, minlon2, maxlon2, reslat2, reslon2, e_c] = read_DEM(CoarseDensity);
                else
                    [m, n]=size(z2);
                    e_c(1:m,1:n)=e; 
                end
            else
                [m, n]=size(z1);   % consatnt denisty mode
                e_d(1:m,1:n)=e;
                [m, n]=size(z2);
                e_c(1:m,1:n)=e;
                [m, n]=size(zt);
                e_t(1:m,1:n)=e; 
            end   
            
            %% write output report --- DEM and density information
            fid=fopen(output_report,'a');
            p='-------------------------------  DEM and Mass  -------------------------------';
            formatSpec='%s\n';
            fprintf(fid,formatSpec,p);
            fprintf(fid,'\n');
            fprintf(fid,'%30s %10.4f\n','DetailDEM_latlimit_north',maxlat1);
            fprintf(fid,'%30s %10.4f\n','DetailDEM_latlimit_south',minlat1);
            fprintf(fid,'%30s %10.4f\n','DetailDEM_reslat',reslat1);
            fprintf(fid,'%30s %10.4f\n','DetailDEM_latlimit_west',minlon1);
            fprintf(fid,'%30s %10.4f\n','DetailDEM_latlimit_est',maxlon1);
            fprintf(fid,'%30s %10.4f\n','DetailDEM_reslon',reslon1);
            fprintf(fid,'\n');
            fprintf(fid,'%30s %10.4f\n','CoarseDEM_latlimit_north',maxlat2);
            fprintf(fid,'%30s %10.4f\n','CoarseDEM_latlimit_south',minlat2);
            fprintf(fid,'%30s %10.4f\n','CoarseDEM_reslat',reslat2);
            fprintf(fid,'%30s %10.4f\n','CoarseDEM_latlimit_west',minlon2);
            fprintf(fid,'%30s %10.4f\n','CoarseDEM_latlimit_est',maxlon2);
            fprintf(fid,'%30s %10.4f\n','CoarseDEM_reslon',reslon2);
            fprintf(fid,'\n');
            if idensity==0
                fprintf(fid,'%30s %11s %10.4f %10s\n','Density_model:','const_e=',e,'[g/cm^3]');
            else if idensity==1
                    fprintf(fid,'%30s %50s','Density_model:','density_model shared the same boundary with DEM');
                end
            end
            fprintf(fid,'\n');
            fclose(fid);
                     
            %ellipsoidal parameters of GRS80 reference ellipsoid
            d2r=Pi/180;
            a=6378137;  %major radius
            e_2=0.00673949677548; % eccentricity
            
            %ellipsoidal radius of terrain model
             if flag_earth == 1
                 re1=a./sqrt(1+e_2.*(sin(y1*d2r)).^2);  % ellipsoidal radius on the ellipsoidal surface of fine grid
                 re2=a./sqrt(1+e_2.*(sin(y2*d2r)).^2);  % ellipsoidal radius on the ellipsoidal surface of coarse grid
                 ret=a./sqrt(1+e_2.*(sin(yt*d2r)).^2);
                 re_com=a./sqrt(1+e_2.*(sin(com_lat*d2r)).^2);
             else if flag_earth==0   % with spherical approximation
                     re1=a;
                     re2=a;
                     ret=a;
                     re_com=a;
                 end
             end
            z1=z1+re1;    % ellipsoidal radius of upper boundary
            z2=z2+re2;    
            zt=zt+ret;
            z1_ref=z1_ref+re1;  % ellipsoidal radius of lower boundary
            z2_ref=z2_ref+re2;
            zt_ref=zt_ref+ret;
            
            if ikind==1   % upper and lower boundaries of outest zone in full-scale gravitational field calculations
                if flag_earth==1
                    reg=a./sqrt(1+e_2.*(sin(yg*d2r)).^2);
                else 
                    reg=a;
                end
                zg=zg+reg;
                zg_ref=zg_ref+reg;
            end
 
            com_h=com_h+re_com;   % computation height

            % define the resolution of each zone, resolution of oute zone must be
            % integer times of resolution of the closest inner zone
            resolution_y1=reslat1;
            resolution_x1=reslon1;

            resolution_y2=resolution_y1;
            resolution_x2=resolution_x1;

            resolution_y3=reslatt;
            resolution_x3=reslont;

            resolution_y4=reslat2;
            resolution_x4=reslon2;


            extend_x1=((floor(r1/resolution_x1)))*resolution_x1;
            extend_y1=((floor(r1/resolution_y1)))*resolution_y1;

            extend_x2=((floor(r2/resolution_x2)))*resolution_x2;
            extend_y2=((floor(r2/resolution_y2)))*resolution_y2;

            n1=resolution_x4/resolution_x3;
            n2=resolution_y4/resolution_y3;
            extend_x3=(n1*(floor(r3/resolution_x3/n1)))*resolution_x3;
            extend_y3=(n2*(floor(r3/resolution_y3/n2)))*resolution_y3;
      
            extend_x4=(4*(floor(r4/resolution_x4/4)))*resolution_x4;
            extend_y4=(4*(floor(r4/resolution_y4/4)))*resolution_y4;
            
            
			if ikind==2
  				ref_station=interp2(x1,y1,z1_ref,com_lon,com_lat,'cubic');         % height of computation point on reference surface
            end
            
            [m, n]=size(com_lon);
            size_com=m*n;
            % preallocating variables
              [V11, Vx11, Vy11, Vz11, Vxx11, Vxy11, Vxz11, Vyy11, Vyz11, Vzz11] = matrix_preallocating(1,size_com);
              [Vm, Vxm, Vym, Vzm, Vxxm, Vxym, Vxzm, Vyym, Vyzm, Vzzm] = matrix_preallocating(1,size_com);
              [V2, Vx2, Vy2, Vz2, Vxx2, Vxy2, Vxz2, Vyy2, Vyz2, Vzz2] = matrix_preallocating(1,size_com);
              [V3, Vx3, Vy3, Vz3, Vxx3, Vxy3, Vxz3, Vyy3, Vyz3, Vzz3] = matrix_preallocating(1,size_com); 
            % compute all ten gravity field elements
            if itype==10
                 t1=clock; 
                if GUI==1
                    text_process=strcat('is computing ten gravitational field elements for ',' ',num2str(size_com),' points');
                    set(findobj('tag','process'),'string',text_process); drawnow;
                end
            % polyhedron method for inner zone
                [V1, Vx1, Vy1, Vz1, Vxx1, Vxy1, Vxz1, Vyy1, Vyz1, Vzz1]=Polyhedron_full( x1,y1,z1,z1_ref,com_lat,com_lon,com_h,extend_x1,extend_y1,e_d,resolution_x1,resolution_y1);
                for i=1:size_com
                    lon=com_lon(i);
                    lat=com_lat(i);
                    r0=com_h(i);
            % build grid---first zone  
                    n=find(x1<=lon+extend_x2 & x1>=lon-extend_x2 & y1<=lat+extend_y2 & y1>=lat-extend_y2);
                    xm=x1(n);
                    ym=y1(n);
                    zm=z1(n);
                    ref_m=z1_ref(n);
                    e_m=e_d(n);
            % leakage zone between zones of polyhedron and prism     
                    if extend_x1>0 & extend_y1>0
                        n=find(xm<=lon+extend_x1& xm>=lon-extend_x1& ym<=lat+extend_y1& ym>=lat-extend_y1);
                        lon1=xm(n);
                        lat1=ym(n);
                        h1=zm(n);
                        ref1=ref_m(n);
                        e_zone1=e_m(n);
                    
                        xm(n)=[];
                        ym(n)=[];
                        zm(n)=[];
                        ref_m(n)=[];
                        e_m(n)=[];      
                    
                    
                        lon_r=max(lon1);
                        lon_l=min(lon1);
                        lat_b=min(lat1);
                        lat_t=max(lat1);
         
                        n=find(lat1==lat_t | lat1==lat_b |lon1==lon_r |lon1==lon_l);
                        lon1=lon1(n);
                        lat1=lat1(n);
                        h1=h1(n);
                        ref1=ref1(n);
                        e_zone1=e_zone1(n);
            
                        xx1=lon1-resolution_x1/2;       
                        xx2=lon1+resolution_x1/2;
                        yy1=lat1-resolution_y1/2;
                        yy2=lat1+resolution_y1/2;
                        zz1=h1;
                        zz2=ref1;
                        [g, gx, gy, gz, gxx, gxy, gxz, gyy, gyz, gzz]=Prism(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e_zone1,resolution_x1,resolution_y1);  % prism field evaluation
                        Vz11(i)=gz/2;
                        Vx11(i)=gx/2;
                        Vy11(i)=gy/2; 
                        V11(i)=g/2;
                        Vxx11(i)=gxx/2;
                        Vxy11(i)=gxy/2;
                        Vxz11(i)=gxz/2;
                        Vyy11(i)=gyy/2;
                        Vyz11(i)=gyz/2;
                        Vzz11(i)=gzz/2;
                    else
                        n=find(xm<=lon+extend_x2& xm>=lon-extend_x2& ym<=lat+extend_y2& ym>=lat-extend_y2);
                        lon1=xm(n);
                        lat1=ym(n);
                    end
            % second zone        
                    if(r2>r1)
                        xx1=xm-resolution_x2/2;
                        xx2=xm+resolution_x2/2;
                        yy1=ym-resolution_y2/2;
                        yy2=ym+resolution_y2/2;
                        zz1=zm;
                        zz2=ref_m;
                        [g, gx, gy, gz, gxx, gxy, gxz, gyy, gyz, gzz]=Prism(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e_m,resolution_x2,resolution_y2);
                        Vzm(i)=gz;
                        Vm(i)=g;
                        Vxm(i)=gx;
                        Vym(i)=gy;
                        Vxxm(i)=gxx;
                        Vxym(i)=gxy;
                        Vxzm(i)=gxz;
                        Vyym(i)=gyy;
                        Vyzm(i)=gyz;
                        Vzzm(i)=gzz;
                   end
             
            % third zone 
                   if(r3>r2)
                       [lon2,lat2]=Build_grid(mean(lon1),mean(lat1),extend_x2,extend_y2,extend_x3,extend_y3,resolution_x3,resolution_y3);
                       F=griddedInterpolant(xt',yt',zt','cubic');
                       h2=F(lon2,lat2);
                       F=griddedInterpolant(xt',yt',zt_ref','cubic');
                       ref_2=F(lon2,lat2);
                       F=griddedInterpolant(xt',yt',e_t','cubic');
                       e_2=F(lon2,lat2);
                       [g, gx, gy, gz, gxx, gxy, gxz, gyy, gyz, gzz]=Tesseroid(lon2,lat2,h2,ref_2,lon,lat,r0,resolution_x3,resolution_y3,e_2);
                       V2(i)=g;
                       Vx2(i)=gx;
                       Vy2(i)=gy;
                       Vz2(i)=gz;
                       Vxx2(i)=gxx;
                       Vxy2(i)=gxy;
                       Vxz2(i)=gxz;
                       Vyy2(i)=gyy;
                       Vyz2(i)=gyz;
                       Vzz2(i)=gzz;  
                   end
            % Fouth zone --- point mass
                    if((extend_x4>extend_x3 & extend_y4>extend_y3))
                       [lon3,lat3]=Build_grid(mean(lon1),mean(lat1),extend_x3,extend_y3,extend_x4,extend_y4,resolution_x4,resolution_y4);
                        F=griddedInterpolant(x2',y2',z2','cubic');
                        h3=F(lon3,lat3);
                        F=griddedInterpolant(x2',y2',z2_ref','cubic');
                        ref3=F(lon3,lat3);
                        F=griddedInterpolant(x2',y2',e_c','cubic');
                        e3=F(lon3,lat3);
   
                       xx1=lon3-resolution_x4/2;
                       xx2=lon3+resolution_x4/2;
                       yy1=lat3-resolution_y4/2;
                       yy2=lat3+resolution_y4/2;
                       zz1=h3;
                       zz2=ref3;
                       [g, gx, gy, gz, gxx, gxy, gxz, gyy, gyz, gzz] = Point_mass(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e3);
                       
                       V3(i)=g;
                       Vx3(i)=gx;
                       Vy3(i)=gy;
                       Vz3(i)=gz;
                       Vxx3(i)=gxx;
                       Vxy3(i)=gxy;
                       Vxz3(i)=gxz;
                       Vyy3(i)=gyy;
                       Vyz3(i)=gyz;
                       Vzz3(i)=gzz;  
                    end  
                    if ikind==1 
					%outest zone for global integration
                       [lon4,lat4]=Build_grid(mean(lon1),mean(lat1),extend_x4,extend_y4,360,180,resolution_g,resolution_g);
                       n=find(lon4<maxxg & lon4>minxg& lat4<maxyg& lat4>minyg);
                       lon4=lon4(n);
                       lat4=lat4(n); 
                       F=griddedInterpolant(xg',yg',zg','cubic');
                       h4=F(lon4,lat4);
                       F=griddedInterpolant(xg',yg',zg_ref','cubic');
                       ref4=F(lon4,lat4);
                       
                       e4=2.67;
                       xx1=lon4-resolution_g/2;
                       xx2=lon4+resolution_g/2;
                       yy1=lat4-resolution_g/2;
                       yy2=lat4+resolution_g/2;
                       zz1=h4;
                       zz2=ref4;
                       [g gx gy gz gxx gxy gxz gyy gyz gzz] = Point_mass(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e4);
                       V4(i)=g;
                       Vx4(i)=gx;
                       Vy4(i)=gy;
                       Vz4(i)=gz;  
                       Vxx4(i)=gxx;
                       Vxy4(i)=gxy;
                       Vxz4(i)=gxz;
                       Vyy4(i)=gyy;
                       Vyz4(i)=gyz;
                       Vzz4(i)=gzz;
                   else
                       V4(i)=0;
                       Vx4(i)=0;
                       Vy4(i)=0;
                       Vz4(i)=0; 
                       Vxx4(i)=0;
                       Vxy4(i)=0;
                       Vxz4(i)=0;
                       Vyy4(i)=0;
                       Vyz4(i)=0;
                       Vzz4(i)=0;
                    end
                    if GUI==1
                       text_process_step=strcat(text_process,num2str(i),'/',num2str(size_com),'were completed');
                       set(findobj('tag','process'),'string',text_process_step); drawnow;
                    end
                end
                 V=V1+V2+V3+Vm+V11+V4;
                 Vx=Vx1+Vx2+Vx3+Vxm+Vx11+Vx4;
                 Vy=Vy1+Vy2+Vy3+Vym+Vy11+Vy4;
                 Vz=Vz1+Vz2+Vz3+Vzm+Vz11+Vz4;
                 if ikind==2  % harmonic correction
                     harmonic(1:size_com)=0;
                     n=find(ref_station-com_h>0);
                     harmonic(n)=4*Pi*G*2.67.*(ref_station(n)-com_h(n));
                     Vz=Vz+harmonic;
                 end
                 Vxx=Vxx1+Vxx2+Vxx3+Vxxm+Vxx11+Vxx4;
                 Vxy=Vxy1+Vxy2+Vxy3+Vxym+Vxy11+Vxy4;
                 Vxz=Vxz1+Vxz2+Vxz3+Vxzm+Vxz11+Vxz4;
                 Vyy=Vyy1+Vyy2+Vyy3+Vyym+Vyy11+Vyy4;
                 Vyz=Vyz1+Vyz2+Vyz3+Vyzm+Vyz11+Vyz4;
                 Vzz=Vzz1+Vzz2+Vzz3+Vzzm+Vzz11+Vzz4; 
            % normal gravity computation
                 gama=normal_gravity_generic(com_lat,H_com);
                 Dov_const=3600*180/Pi./gama*10^-5;    
                 xi_RTM=-Vx'.*Dov_const;      % Dovs in arc-sec
                 eta_RTM=-Vy'.*Dov_const;
                 dg_RTM=-Vz';                             % gravity disturbance in mGal
                 N=V'./gama;                               % geoid height in m
                 tensor=[com_lon(:) com_lat(:) H_com(:) N xi_RTM eta_RTM dg_RTM Vxx' Vxy' Vxz' Vyy' Vyz' Vzz'];
                 t2=clock;
                 t=etime(t2,t1)/3600;
                 fid=fopen(output_report,'a');
                 p='-------------------------  integration information  --------------------------';
                 fprintf(fid,'%s\n',p);
                 fprintf(fid,'\n');
                 fprintf(fid,'%30s%10.5f\n','Computation time[hour]:',t);
                 fprintf(fid,'\n');
                 p='--------------------------  Output information  -------------------------------';
                 fprintf(fid,'%30s','Output file name:');
                 fprintf(fid,'%15s %s \n','Output file:',outname);
                 fprintf(fid,'\n');
                 p='------------------------------  Results Format  ------------------------------';
                 fprintf(fid,'%s\n',p);
                 fprintf(fid,'\n');
                 p={'lon'  'lat'  'H'  'N'  'xi'  'eta'  'dg'  'Vxx'  'Vxy'  'Vxz'  'Vyy'  'Vyz'  'Vzz'};
                 formatSpec='%7s%7s%7s%7s%7s%7s%7s%7s%7s%7s%7s%7s%7s\n';
                 fprintf(fid,formatSpec,p{1,:});
                 p={'[deg]'  '[deg]'  '[m]'  '[m]'  '["]'  '["]'  '[mGal]'  '[E]'  '[E]'  '[E]'  '[E]'  '[E]'  '[E]'};
                 formatSpec='%7s%7s%7s%7s%7s%7s%7s%7s%7s%7s%7s%7s%7s\n';
                 fprintf(fid,formatSpec,p{1,:});
                 fclose(fid);
                 save(outname,'tensor','-ASCII')
            end
			
            
			% compute geoid height
            if itype==0
                t1=clock;
                if GUI==1
                    text_process=strcat('is computing geoid height for',' ',num2str(size_com),' points');
                    set(findobj('tag','process'),'string',text_process);  drawnow;
                end
            % polyhedron method for inner zone
                [V1, Vx1, Vy1, Vz1, Vxx1, Vxy1, Vxz1, Vyy1, Vyz1, Vzz1]=Polyhedron_g( x1,y1,z1,z1_ref,com_lat,com_lon,com_h,extend_x1,extend_y1,e_d,resolution_x1,resolution_y1);
                for i=1:size_com  
                    lon=com_lon(i);
                    lat=com_lat(i);
                    r0=com_h(i);
            % build grid---first zone  
                    n=find(x1<=lon+extend_x2 & x1>=lon-extend_x2 & y1<=lat+extend_y2 & y1>=lat-extend_y2);
                    xm=x1(n);
                    ym=y1(n);
                    zm=z1(n);
                    ref_m=z1_ref(n);
                    e_m=e_d(n);
            % leakage zone between zones of polyhedron and prism           
                    if extend_x1>0 & extend_y1>0
                        n=find(xm<=lon+extend_x1 & xm>=lon-extend_x1 & ym<=lat+extend_y1 & ym>=lat-extend_y1);
                        lon1=xm(n);
                        lat1=ym(n);
                        h1=zm(n);
                        ref1=ref_m(n);
                        e_zone1=e_m(n);
                    
                        xm(n)=[];
                        ym(n)=[];
                        zm(n)=[];
                        ref_m(n)=[];
                        e_m(n)=[];      
                    
                    
                        lon_r=max(lon1);
                        lon_l=min(lon1);
                        lat_b=min(lat1);
                        lat_t=max(lat1);
         
                        n=find(lat1==lat_t | lat1==lat_b |lon1==lon_r |lon1==lon_l);
                        lon1=lon1(n);
                        lat1=lat1(n);
                        h1=h1(n);
                        ref1=ref1(n);
                        e_zone1=e_zone1(n);
            
                        xx1=lon1-resolution_x1/2;
                        xx2=lon1+resolution_x1/2;
                        yy1=lat1-resolution_y1/2;
                        yy2=lat1+resolution_y1/2;
                        zz1=h1;
                        zz2=ref1;
                        [g ]=Prism_g(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e_zone1,resolution_x1,resolution_y1);
                        V11(i)=g/2;
                    else
                        n=find(xm<=lon+extend_x2 & xm>=lon-extend_x2 &ym<=lat+extend_y2 &ym>=lat-extend_y2);
                        lon1=xm(n);
                        lat1=ym(n);
                    end
            % second zone        
                    if(r2>r1)
                        xx1=xm-resolution_x2/2;
                        xx2=xm+resolution_x2/2;
                        yy1=ym-resolution_y2/2;
                        yy2=ym+resolution_y2/2;
                        zz1=zm;
                        zz2=ref_m;
                        [g]=Prism_g(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e_m,resolution_x2,resolution_y2);
                        Vm(i)=g;  
                   end
             
            % third zone 
                   if(r3>r2)
                       [lon2,lat2]=Build_grid(mean(lon1),mean(lat1),extend_x2,extend_y2,extend_x3,extend_y3,resolution_x3,resolution_y3);
                       F=griddedInterpolant(xt',yt',zt','cubic');
                       h2=F(lon2,lat2);
                       F=griddedInterpolant(xt',yt',zt_ref','cubic');
                       ref_2=F(lon2,lat2);
                       F=griddedInterpolant(xt',yt',e_t','cubic');
                       e_2=F(lon2,lat2);
                       [g]=Tesseroid_g(lon2,lat2,h2,ref_2,lon,lat,r0,resolution_x3,resolution_y3,e_2);
                       V2(i)=g;
                    else
                       V2(i)=0;
                   end
            % Fouth zone --- point mass
                    if(extend_x4>extend_x3  &extend_y4>extend_y3)
                       [lon3,lat3]=Build_grid(mean(lon1),mean(lat1),extend_x3,extend_y3,extend_x4,extend_y4,resolution_x4,resolution_y4);
                        F=griddedInterpolant(x2',y2',z2','cubic');
                        h3=F(lon3,lat3);
                        F=griddedInterpolant(x2',y2',z2_ref','cubic');
                        ref3=F(lon3,lat3);
                        F=griddedInterpolant(x2',y2',e_c','cubic');
                        e3=F(lon3,lat3);
                        
                       xx1=lon3-resolution_x4/2;
                       xx2=lon3+resolution_x4/2;
                       yy1=lat3-resolution_y4/2;
                       yy2=lat3+resolution_y4/2;
                       zz1=h3;
                       zz2=ref3;
                       [g] = Point_g(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e3);
                       V3(i)=g;
                    end   
                    if ikind==1 
					% outest zone for global integration
                       [lon4,lat4]=Build_grid(mean(lon1),mean(lat1),extend_x4,extend_y4,360,180,resolution_g,resolution_g);
                       n=find(lon4<maxxg & lon4>minxg& lat4<maxyg& lat4>minyg);
                       lon4=lon4(n);
                       lat4=lat4(n); 
                       F=griddedInterpolant(xg',yg',zg','cubic');
                       h4=F(lon4,lat4);
                       F=griddedInterpolant(xg',yg',zg_ref','cubic');
                       ref4=F(lon4,lat4);
                       
                       e4=2.67;
                       xx1=lon4-resolution_g/2;
                       xx2=lon4+resolution_g/2;
                       yy1=lat4-resolution_g/2;
                       yy2=lat4+resolution_g/2;
                       zz1=h4;
                       zz2=ref4;
                       [g ] = Point_g(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e4);
                       V4(i)=g;
                      
                   else
                       V4(i)=0;
                    end 
                    if GUI==1
                       text_process_step=strcat(text_process,num2str(i),'/',num2str(size_com),'were completed');
                       set(findobj('tag','process'),'string',text_process_step); drawnow;
                    end 
                end
                 V=V1+V2+V3+Vm+V11+V4; 
            % normal gravity computation
                 gama=normal_gravity_generic(com_lat,H_com);
                 N=V'./gama;                               % geoid height in m
                 tensor=[com_lon(:) com_lat(:) H_com(:) N];
                 
                 t2=clock;
                 t=etime(t2,t1)/3600;
                 fid=fopen(output_report,'a');
                 p='-------------------------  integration information  --------------------------';
                 fprintf(fid,'%s\n',p);
                 fprintf(fid,'\n');
                 fprintf(fid,'%30s%10.5f\n','Computation time[hour]:',t);
                 fprintf(fid,'\n');
                 p='--------------------------  Output information  -------------------------------';
                 fprintf(fid,'%30s','Output file name:');
                 fprintf(fid,'%15s %s \n','Output file:',outname);
                 fprintf(fid,'\n');
                 p='------------------------------  Results Format  ------------------------------'; 
                 fprintf(fid,'%s\n\n',p);
                 p={'lon'  'lat'  'H'  'N'};
                 formatSpec='%7s%7s%7s%7s\n';
                 fprintf(fid,formatSpec,p{1,:});
                 p={'[deg]'  '[deg]'  '[m]'  '[m]'};
                 formatSpec='%7s%7s%7s%7s\n';
                 fprintf(fid,formatSpec,p{1,:});
                 fclose(fid);
                 save(outname,'tensor','-ASCII')
                 
            end
			
			% compute geoid height, gravity components related to first-derivatives of potential
            if itype==103 | itype==104 | itype==4
                t1=clock;
                if GUI ==1
                     if itype==103
                         text_process=strcat('is computing geoid height, vertical deflection, and gravity disturbance for ',num2str(size_com),' points');
                         set(findobj('tag','process'),'string',text_process); drawnow;
                     elseif itype==104
                         text_process=strcat('computing geoid height, vertical deflection, and gravity anomaly for ',num2str(size_com),' points');
                         set(findobj('tag','process'),'string',text_process); drawnow;
                     elseif itype==4
                         text_process=strcat('computing geoid height, gravity anomaly for ',num2str(size_com),' points');
                         set(findobj('tag','process'),'string',text_process); drawnow;
                     end
                end
 
            % polyhedron method for inner zone
                [V1, Vx1, Vy1, Vz1, Vxx1, Vxy1, Vxz1, Vyy1, Vyz1, Vzz1]=Polyhedron_g_gx( x1,y1,z1,z1_ref,com_lat,com_lon,com_h,extend_x1,extend_y1,e_d,resolution_x1,resolution_y1);
                for i=1:size_com
                    lon=com_lon(i);
                    lat=com_lat(i);
                    r0=com_h(i);
            % build grid---first zone  
                    n=find(x1<=lon+extend_x2 & x1>=lon-extend_x2 & y1<=lat+extend_y2 & y1>=lat-extend_y2);
                    xm=x1(n);
                    ym=y1(n);
                    zm=z1(n);
                    ref_m=z1_ref(n);
                    e_m=e_d(n);
            % leakage zone between zones of polyhedron and prism           
                     if extend_x1>0  & extend_y1>0
                        n=find(xm<=lon+extend_x1 & xm>=lon-extend_x1 & ym<=lat+extend_y1 & ym>=lat-extend_y1);
                        lon1=xm(n);
                        lat1=ym(n);
                        h1=zm(n);
                        ref1=ref_m(n);
                        e_zone1=e_m(n);
                    
                        xm(n)=[];
                        ym(n)=[];
                        zm(n)=[];
                        ref_m(n)=[];
                        e_m(n)=[];      
                    
                    
                        lon_r=max(lon1);
                        lon_l=min(lon1);
                        lat_b=min(lat1);
                        lat_t=max(lat1);
         
                        n=find(lat1==lat_t | lat1==lat_b |lon1==lon_r |lon1==lon_l);
                        lon1=lon1(n);
                        lat1=lat1(n);
                        h1=h1(n);
                        ref1=ref1(n);
                        e_zone1=e_zone1(n);
            
                        xx1=lon1-resolution_x1/2;
                        xx2=lon1+resolution_x1/2;
                        yy1=lat1-resolution_y1/2;
                        yy2=lat1+resolution_y1/2;
                        zz1=h1;
                        zz2=ref1;
                        [g, gx, gy, gz]=Prism_g_gx(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e_zone1,resolution_x1,resolution_y1);
                        Vz11(i)=gz/2;
                        Vx11(i)=gx/2;
                        Vy11(i)=gy/2; 
                        V11(i)=g/2;
                     else
                        n=find(xm<=lon+extend_x2 & xm>=lon-extend_x2 & ym<=lat+extend_y2 & ym>=lat-extend_y2);
                        lon1=xm(n);
                        lat1=ym(n);
                    end
            % second zone        
                    if(r2>r1)

                        xx1=xm-resolution_x2/2;
                        xx2=xm+resolution_x2/2;
                        yy1=ym-resolution_y2/2;
                        yy2=ym+resolution_y2/2;
                        zz1=zm;
                        zz2=ref_m;
                        [g, gx, gy, gz]=Prism_g_gx(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e_m,resolution_x2,resolution_y2);
                        Vzm(i)=gz;
                        Vm(i)=g;
                        Vxm(i)=gx;
                        Vym(i)=gy;
                   end
             
            % third zone 
                   if(r3>r2)
                       [lon2,lat2]=Build_grid(mean(lon1),mean(lat1),extend_x2,extend_y2,extend_x3,extend_y3,resolution_x3,resolution_y3);
                       F=griddedInterpolant(xt',yt',zt','cubic');
                       h2=F(lon2,lat2);
                       F=griddedInterpolant(xt',yt',zt_ref','cubic');
                       ref_2=F(lon2,lat2);
                       F=griddedInterpolant(xt',yt',e_t','cubic');
                       e_2=F(lon2,lat2);
                       [g, gx, gy, gz]=Tesseroid_g_gx(lon2,lat2,h2,ref_2,lon,lat,r0,resolution_x3,resolution_y3,e_2);
                       V2(i)=g;
                       Vx2(i)=gx;
                       Vy2(i)=gy;
                       Vz2(i)=gz;
                    else
                       V2(i)=0;
                       Vx2(i)=0;
                       Vy2(i)=0;
                       Vz2(i)=0; 
                   end
            % Fouth zone --- point mass
                    if(extend_x4>extend_x3  & extend_y4>extend_y3)
                       
                       [lon3,lat3]=Build_grid(mean(lon1),mean(lat1),extend_x3,extend_y3,extend_x4,extend_y4,resolution_x4,resolution_y4);
                       F=griddedInterpolant(x2',y2',z2','cubic');
                       h3=F(lon3,lat3);
                       F=griddedInterpolant(x2',y2',z2_ref','cubic');
                       ref3=F(lon3,lat3);
                       F=griddedInterpolant(x2',y2',e_c','cubic');
                       e3=F(lon3,lat3);
                        
                       xx1=lon3-resolution_x4/2;
                       xx2=lon3+resolution_x4/2;
                       yy1=lat3-resolution_y4/2;
                       yy2=lat3+resolution_y4/2;
                       zz1=h3;
                       zz2=ref3;
                       [g, gx, gy, gz] = Point_g_gx(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e3);
                       V3(i)=g;
                       Vx3(i)=gx;
                       Vy3(i)=gy;
                       Vz3(i)=gz; 
                    end  
                    if ikind==1 
					%outest zone for global integration
                       [lon4,lat4]=Build_grid(mean(lon1),mean(lat1),extend_x4,extend_y4,360,180,resolution_g,resolution_g);
                       n=find(lon4<maxxg & lon4>minxg& lat4<maxyg& lat4>minyg);
                       lon4=lon4(n);
                       lat4=lat4(n); 
                       F=griddedInterpolant(xg',yg',zg','cubic');
                       h4=F(lon4,lat4);
                       F=griddedInterpolant(xg',yg',zg_ref','cubic');
                       ref4=F(lon4,lat4);
                       
                       e4=2.67;
                       xx1=lon4-resolution_g/2;
                       xx2=lon4+resolution_g/2;
                       yy1=lat4-resolution_g/2;
                       yy2=lat4+resolution_g/2;
                       zz1=h4;
                       zz2=ref4;
                       [g gx gy gz] = Point_g_gx(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e4);
                       V4(i)=g;
                       Vx4(i)=gx;
                       Vy4(i)=gy;
                       Vz4(i)=gz;  
                   else
                       V4(i)=0;
                       Vx4(i)=0;
                       Vy4(i)=0;
                       Vz4(i)=0; 
                    end
                    if GUI==1
                       text_process_step=strcat(text_process,num2str(i),'/',num2str(size_com),'were completed');
                       set(findobj('tag','process'),'string',text_process_step); drawnow;
                    end
                end
                 V=V1+V2+V3+Vm+V11+V4;
                 Vx=Vx1+Vx2+Vx3+Vxm+Vx11+Vx4;
                 Vy=Vy1+Vy2+Vy3+Vym+Vy11+Vy4;
                 Vz=Vz1+Vz2+Vz3+Vzm+Vz11+Vz4;
                 if ikind==2  % harmonic correction
                     harmonic(1:size_com)=0;
                     n=find(ref_station-com_h>0);
                     harmonic(n)=4*Pi*G*2.67.*(ref_station(n)-com_h(n));
                     Vz=Vz+harmonic;
                 end
            % normal gravity computation
                 gama=normal_gravity_generic(com_lat,H_com);
                 Dov_const=3600*180/Pi./gama*10^-5;
                 xi_RTM=-Vx'.*Dov_const;      % Dovs in arc-sec
                 eta_RTM=-Vy'.*Dov_const;
                 dg_RTM=-Vz';                             % gravity disturbance in mGal
                 N=V'./gama;                               % geoid height in m
                 tensor=[com_lon(:) com_lat(:) H_com(:) N xi_RTM eta_RTM dg_RTM];
				 if itype==104 | itype==4
                    dg_RTM=dg_RTM-0.3086*N;             % gravity anomaly in mGal
                    tensor=[com_lon(:) com_lat(:) H_com(:) N xi_RTM eta_RTM dg_RTM];
                 end 
                 t2=clock;
                 t=etime(t2,t1)/3600;
                 fid=fopen(output_report,'a');
                 p='-------------------------  integration information  --------------------------';
                 fprintf(fid,'%s\n\n',p);
                 fprintf(fid,'%30s%10.5f\n\n','Computation time[hour]:',t);
                 p='--------------------------  Output information  -------------------------------';
                 fprintf(fid,'%30s','Output file name:');
                 fprintf(fid,'%15s %s \n','Output file:',outname);
                 fprintf(fid,'\n');
                 p='------------------------------  Results Format  ------------------------------';
                 fprintf(fid,'%s\n\n',p);
                 p={'lon'  'lat'  'H'  'N'  'xi'  'eta'  'dg'};
                 formatSpec='%7s%7s%7s%7s%7s%7s%7s\n';
                 fprintf(fid,formatSpec,p{1,:});
                 p={'[deg]'  '[deg]'  '[m]' '[m]'  '["]'  '["]'  '[mGal]'};
                 formatSpec='%7s%7s%7s%7s%7s%7s%7s\n';
                 fprintf(fid,formatSpec,p{1,:});
                 fclose(fid);
                 save(outname,'tensor','-ASCII')
            end
			
			% compute all xi, eta and dg
			if itype==1
                tic;
                if GUI ==1
                     text_process=strcat('is computing vertical deflection, and gravity disturbance for ',num2str(size_com),' points');
                     set(findobj('tag','process'),'string',text_process); drawnow;
                end
            % polyhedron method for inner zone
                [V1, Vx1, Vy1, Vz1, Vxx1, Vxy1, Vxz1, Vyy1, Vyz1, Vzz1]=Polyhedron_gx( x1,y1,z1,z1_ref,com_lat,com_lon,com_h,extend_x1,extend_y1,e_d,resolution_x1,resolution_y1);
                for i=1:size_com
                    lon=com_lon(i);
                    lat=com_lat(i);
                    r0=com_h(i);
            % build grid---first zone   
                    n=find(x1<=lon+extend_x2 & x1>=lon-extend_x2 & y1<=lat+extend_y2 & y1>=lat-extend_y2);
                    xm=x1(n);
                    ym=y1(n);
                    zm=z1(n);
                    ref_m=z1_ref(n);
                    e_m=e_d(n);
            % leakage zone between zones of polyhedron and prism                      
                    if extend_x1>0  & extend_y1>0
                        n=find(xm<=lon+extend_x1 & xm>=lon-extend_x1 & ym<=lat+extend_y1 & ym>=lat-extend_y1);
                        lon1=xm(n);
                        lat1=ym(n);
                        h1=zm(n);
                        ref1=ref_m(n);
                        e_zone1=e_m(n);
                    
                        xm(n)=[];
                        ym(n)=[];
                        zm(n)=[];
                        ref_m(n)=[];
                        e_m(n)=[];      
                    
                    
                        lon_r=max(lon1);
                        lon_l=min(lon1);
                        lat_b=min(lat1);
                        lat_t=max(lat1);
         
                        n=find(lat1==lat_t | lat1==lat_b |lon1==lon_r |lon1==lon_l);
                        lon1=lon1(n);
                        lat1=lat1(n);
                        h1=h1(n);
                        ref1=ref1(n);
                        e_zone1=e_zone1(n);
            
                        xx1=lon1-resolution_x1/2;
                        xx2=lon1+resolution_x1/2;
                        yy1=lat1-resolution_y1/2;
                        yy2=lat1+resolution_y1/2;
                        zz1=h1;
                        zz2=ref1;
                        [ gx, gy, gz]=Prism_gx(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e_zone1,resolution_x1,resolution_y1);
                        Vz11(i)=gz/2;
                        Vx11(i)=gx/2;
                        Vy11(i)=gy/2; 
                    else                        
                        n=find(xm<=lon+extend_x2 & xm>=lon-extend_x2 & ym<=lat+extend_y2 & ym>=lat-extend_y2);
                        lon1=xm(n);
                        lat1=ym(n);
                    end
                    
                                                           
            % second zone        
                    if(r2>r1)
                        xx1=xm-resolution_x2/2;
                        xx2=xm+resolution_x2/2;
                        yy1=ym-resolution_y2/2;
                        yy2=ym+resolution_y2/2;
                        zz1=zm;
                        zz2=ref_m;
                        [gx, gy, gz]=Prism_gx(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e_m,resolution_x2,resolution_y2);
                        Vzm(i)=gz;
                        Vxm(i)=gx;
                        Vym(i)=gy;
                   end
             
            % third zone 
                   if(r3>r2)
                       [lon2,lat2]=Build_grid(mean(lon1),mean(lat1),extend_x2,extend_y2,extend_x3,extend_y3,resolution_x3,resolution_y3);
                       F=griddedInterpolant(xt',yt',zt','cubic');
                       h2=F(lon2,lat2);
                       F=griddedInterpolant(xt',yt',zt_ref','cubic');
                       ref_2=F(lon2,lat2);
                       F=griddedInterpolant(xt',yt',e_t','cubic');
                       e_2=F(lon2,lat2);
                               
                       [gx, gy, gz]=Tesseroid_gx(lon2,lat2,h2,ref_2,lon,lat,r0,resolution_x3,resolution_y3,e_2);
                       Vx2(i)=gx;
                       Vy2(i)=gy;
                       Vz2(i)=gz;
                   end
            % Fouth zone --- point mass
                    if(extend_x4>extend_x3  & extend_y4>extend_y3)
                        [lon3,lat3]=Build_grid(mean(lon1),mean(lat1),extend_x3,extend_y3,extend_x4,extend_y4,resolution_x4,resolution_y4);
                        F=griddedInterpolant(x2',y2',z2','cubic');
                        h3=F(lon3,lat3);
                        F=griddedInterpolant(x2',y2',z2_ref','cubic');
                        ref3=F(lon3,lat3);
                        F=griddedInterpolant(x2',y2',e_c','cubic');
                        e3=F(lon3,lat3);
 
                       xx1=lon3-resolution_x4/2;
                       xx2=lon3+resolution_x4/2;
                       yy1=lat3-resolution_y4/2;
                       yy2=lat3+resolution_y4/2;
                       zz1=h3;
                       zz2=ref3;
                       [gx gy gz] = Point_gx(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e3);
                                             
                       Vx3(i)=gx;
                       Vy3(i)=gy;
                       Vz3(i)=gz;
                    end 
                    if ikind==1 
					%outest zone for global integration
                       [lon4,lat4]=Build_grid(mean(lon1),mean(lat1),extend_x4,extend_y4,360,180,resolution_g,resolution_g);
                       n=find(lon4<maxxg & lon4>minxg& lat4<maxyg& lat4>minyg);
                       lon4=lon4(n);
                       lat4=lat4(n); 
                       F=griddedInterpolant(xg',yg',zg','cubic');
                       h4=F(lon4,lat4);
                       F=griddedInterpolant(xg',yg',zg_ref','cubic');
                       ref4=F(lon4,lat4);
                       
                       e4=2.67;
                       xx1=lon4-resolution_g/2;
                       xx2=lon4+resolution_g/2;
                       yy1=lat4-resolution_g/2;
                       yy2=lat4+resolution_g/2;
                       zz1=h4;
                       zz2=ref4;
                       [ gx gy gz ] = Point_gx(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e4);
                       Vx4(i)=gx;
                       Vy4(i)=gy;
                       Vz4(i)=gz;  
                    else 
                       Vx4(i)=0;
                       Vy4(i)=0;
                       Vz4(i)=0; 
                    end
                    if GUI==1
                       text_process_step=strcat(text_process,num2str(i),'/',num2str(size_com),'were completed');
                       set(findobj('tag','process'),'string',text_process_step); drawnow;
                    end
                end
                Vx=Vx1+Vx2+Vxm+Vx3+Vx11+Vx4;
                Vy=Vy1+Vy2+Vym+Vy3+Vy11+Vy4;
                Vz=Vz1+Vz2+Vzm+Vz3+Vz11+Vz4;
                 if ikind==2
                     harmonic(1:size_com)=0;
                     n=find(ref_station-com_h>0);
                     harmonic(n)=4*Pi*G*2.67*(ref_station(n)-com_h(n));
                     Vz=Vz+harmonic;
                 end
            % normal gravity computation
                 gama=normal_gravity_generic(com_lat,H_com);
                 Dov_const=3600*180/Pi./gama*10^-5;
                 xi_RTM=-Vx'.*Dov_const;      % Dovs in arc-sec
                 eta_RTM=-Vy'.*Dov_const;
                 dg_RTM=-Vz';                             % gravity disturbance in mGal

                 t=toc;
                 tensor=[com_lon(:) com_lat(:) H_com(:) xi_RTM eta_RTM dg_RTM];
                 fid=fopen(output_report,'a');
                 p='-------------------------  integration information  --------------------------';
                 fprintf(fid,'%s\n\n',p);
                 fprintf(fid,'%30s%10.5f\n\n','Computation time[hour]:',t);
                 p='--------------------------  Output information  -------------------------------';
                 fprintf(fid,'%30s','Output file name:');
                 fprintf(fid,'%15s %s \n','Output file:',outname);
                 fprintf(fid,'\n');
                 p='------------------------------  Results Format  ------------------------------';
                 fprintf(fid,'%s\n\n',p);
                 p={'lon'  'lat'  'H'  'xi'  'eta'  'dg'};
                 formatSpec='%7s%7s%7s%7s%7s%7s\n';
                 fprintf(fid,formatSpec,p{1,:});
                 p={'[deg]'  '[deg]'  '[m]'  '["]'  '["]'  '[mGal]'};
                 fprintf(fid,formatSpec,p{1,:});
                 fclose(fid);
                 save(outname,'tensor','-ASCII')
            end
			
			% compute all tensors
            if itype==2
                t1=clock;
                if GUI ==1
                     text_process=strcat('is computing tensors for ',num2str(size_com),' points');
                     set(findobj('tag','process'),'string',text_process);drawnow;
                end
            % polyhedron method for inner zone
                [V1, Vx1, Vy1, Vz1, Vxx1, Vxy1, Vxz1, Vyy1, Vyz1, Vzz1]=Polyhedron_gxx( x1,y1,z1,z1_ref,com_lat,com_lon,com_h,extend_x1,extend_y1,e_d,resolution_x1,resolution_y1);
                for i=1:size_com
                    lon=com_lon(i);
                    lat=com_lat(i);
                    r0=com_h(i);
            % build grid---first zone  
                    n=find(x1<=lon+extend_x2 & x1>=lon-extend_x2 & y1<=lat+extend_y2 & y1>=lat-extend_y2);
                    xm=x1(n);
                    ym=y1(n);
                    zm=z1(n);
                    ref_m=z1_ref(n);
                    e_m=e_d(n);
            % leakage zone between zones of polyhedron and prism           
                    if extend_x1>0  & extend_y1>0
                        n=find(xm<=lon+extend_x1 & xm>=lon-extend_x1 & ym<=lat+extend_y1 & ym>=lat-extend_y1);
                        lon1=xm(n);
                        lat1=ym(n);
                        h1=zm(n);
                        ref1=ref_m(n);
                        e_zone1=e_m(n);
                    
                        xm(n)=[];
                        ym(n)=[];
                        zm(n)=[];
                        ref_m(n)=[];
                        e_m(n)=[];      
                    
                    
                        lon_r=max(lon1);
                        lon_l=min(lon1);
                        lat_b=min(lat1);
                        lat_t=max(lat1);
         
                        n=find(lat1==lat_t | lat1==lat_b |lon1==lon_r |lon1==lon_l);
                        lon1=lon1(n);
                        lat1=lat1(n);
                        h1=h1(n);
                        ref1=ref1(n);
                        e_zone1=e_zone1(n);
            
                        xx1=lon1-resolution_x1/2;
                        xx2=lon1+resolution_x1/2;
                        yy1=lat1-resolution_y1/2;
                        yy2=lat1+resolution_y1/2;
                        zz1=h1;
                        zz2=ref1;
                        [gxx, gxy, gxz, gyy, gyz, gzz]=Prism_gxx(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e_zone1,resolution_x1,resolution_y1);
                        Vxx11(i)=gxx/2;
                        Vxy11(i)=gxy/2;
                        Vxz11(i)=gxz/2;
                        Vyy11(i)=gyy/2;
                        Vyz11(i)=gyz/2;
                        Vzz11(i)=gzz/2;
                    else
                        n=find(xm<=lon+extend_x2 & xm>=lon-extend_x2 & ym<=lat+extend_y2 & ym>=lat-extend_y2);
                        lon1=xm(n);
                        lat1=ym(n);
                    end
            % second zone        
                    if(r2>r1)
                        xx1=xm-resolution_x2/2;
                        xx2=xm+resolution_x2/2;
                        yy1=ym-resolution_y2/2;
                        yy2=ym+resolution_y2/2;
                        zz1=zm;
                        zz2=ref_m;
                        [gxx, gxy, gxz, gyy, gyz, gzz]=Prism_gxx(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e_m,resolution_x2,resolution_y2);
                        Vxxm(i)=gxx;
                        Vxym(i)=gxy;
                        Vxzm(i)=gxz;
                        Vyym(i)=gyy;
                        Vyzm(i)=gyz;
                        Vzzm(i)=gzz;
        
                   end
             
            % third zone 
                   if(r3>r2)
                       [lon2,lat2]=Build_grid(mean(lon1),mean(lat1),extend_x2,extend_y2,extend_x3,extend_y3,resolution_x3,resolution_y3);
                       F=griddedInterpolant(xt',yt',zt','cubic');
                       h2=F(lon2,lat2);
                       F=griddedInterpolant(xt',yt',zt_ref','cubic');
                       ref_2=F(lon2,lat2);
                       F=griddedInterpolant(xt',yt',e_t','cubic');
                       e_2=F(lon2,lat2);
                       [gxx, gxy, gxz, gyy, gyz, gzz]=Tesseroid_gxx(lon2,lat2,h2,ref_2,lon,lat,r0,resolution_x3,resolution_y3,e_2);
                       Vxx2(i)=gxx;
                       Vxy2(i)=gxy;
                       Vxz2(i)=gxz;
                       Vyy2(i)=gyy;
                       Vyz2(i)=gyz;
                       Vzz2(i)=gzz; 
                   end
            % Fouth zone --- point mass
                    if(extend_x4>extend_x3  & extend_y4>extend_y3)
                        [lon3,lat3]=Build_grid(mean(lon1),mean(lat1),extend_x3,extend_y3,extend_x4,extend_y4,resolution_x4,resolution_y4);
                        F=griddedInterpolant(x2',y2',z2','cubic');
                        h3=F(lon3,lat3);
                        F=griddedInterpolant(x2',y2',z2_ref','cubic');
                        ref3=F(lon3,lat3);
                        F=griddedInterpolant(x2',y2',e_c','cubic');
                        e3=F(lon3,lat3);
   
                       xx1=lon3-resolution_x4/2;
                       xx2=lon3+resolution_x4/2;
                       yy1=lat3-resolution_y4/2;
                       yy2=lat3+resolution_y4/2;
                       zz1=h3;
                       zz2=ref3;
                       [gxx, gxy, gxz, gyy, gyz, gzz] = Point_gxx(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e3);
                       Vxx3(i)=gxx;
                       Vxy3(i)=gxy;
                       Vxz3(i)=gxz;
                       Vyy3(i)=gyy;
                       Vyz3(i)=gyz;
                       Vzz3(i)=gzz;  
                    end  
                    if ikind==1 
					%outest zone for global integration
                       [lon4,lat4]=Build_grid(mean(lon1),mean(lat1),extend_x4,extend_y4,360,180,resolution_g,resolution_g);
                       n=find(lon4<maxxg & lon4>minxg& lat4<maxyg& lat4>minyg);
                       lon4=lon4(n);
                       lat4=lat4(n); 
                       F=griddedInterpolant(xg',yg',zg','cubic');
                       h4=F(lon4,lat4);
                       F=griddedInterpolant(xg',yg',zg_ref','cubic');
                       ref4=F(lon4,lat4);
                       
                       e4=2.67;
                       xx1=lon4-resolution_g/2;
                       xx2=lon4+resolution_g/2;
                       yy1=lat4-resolution_g/2;
                       yy2=lat4+resolution_g/2;
                       zz1=h4;
                       zz2=ref4;
                       [gxx gxy gxz gyy gyz gzz] = Point_gxx(xx1,xx2,yy1,yy2,zz1,zz2,lon,lat,r0,e4); 
                       Vxx4(i)=gxx;
                       Vxy4(i)=gxy;
                       Vxz4(i)=gxz;
                       Vyy4(i)=gyy;
                       Vyz4(i)=gyz;
                       Vzz4(i)=gzz;
                    else
                       Vxx4(i)=0;
                       Vxy4(i)=0;
                       Vxz4(i)=0;
                       Vyy4(i)=0;
                       Vyz4(i)=0;
                       Vzz4(i)=0;
                    end
                    if GUI==1
                       text_process_step=strcat(text_process,num2str(i),'/',num2str(size_com),'were completed');
                       set(findobj('tag','process'),'string',text_process_step); drawnow;
                    end
                end
                 Vxx=Vxx1+Vxx2+Vxx3+Vxxm+Vxx11+Vxx4;
                 Vxy=Vxy1+Vxy2+Vxy3+Vxym+Vxy11+Vxy4;
                 Vxz=Vxz1+Vxz2+Vxz3+Vxzm+Vxz11+Vxz4;
                 Vyy=Vyy1+Vyy2+Vyy3+Vyym+Vyy11+Vyy4;
                 Vyz=Vyz1+Vyz2+Vyz3+Vyzm+Vyz11+Vyz4;
                 Vzz=Vzz1+Vzz2+Vzz3+Vzzm+Vzz11+Vzz4;  
                 tensor=[com_lon(:) com_lat(:) H_com(:) Vxx' Vxy' Vxz' Vyy' Vyz' Vzz'];
                 t2=clock;
                 t=etime(t2,t1)/3600;
                 fid=fopen(output_report,'a');
                 p='-------------------------  integration information  --------------------------';
                 fprintf(fid,'%s\n\n',p);
                 fprintf(fid,'%30s%10.5f\n\n','Computation time[hour]:',t);
                 p='--------------------------  Output information  -------------------------------';
                 fprintf(fid,'%30s','Output file name:');
                 fprintf(fid,'%15s %s \n','Output file:',outname);
                 fprintf(fid,'\n');
                 p='------------------------------  Results Format  ------------------------------';
                 fprintf(fid,'%s\n\n',p);
                 p={'lon'  'lat'  'H' 'Vxx'  'Vxy'  'Vxz'  'Vyy'  'Vyz'  'Vzz'};
                 formatSpec='%7s%7s%7s%7s%7s%7s%7s%7s%7s\n';
                 fprintf(fid,formatSpec,p{1,:});
                 p={'[deg]'  '[deg]'  '[m]'  '[E]'  '[E]'  '[E]'  '[E]'  '[E]'  '[E]'};
                 formatSpec='%7s%7s%7s%7s%7s%7s%7s%7s%7s\n';
                 fprintf(fid,formatSpec,p{1,:});
                 fclose(fid);
                 save(outname,'tensor','-ASCII')
            end  
            if GUI ==1
                 text_process=strcat('finishes calculation. Results are available in the file ', outname);
                 set(findobj('tag','process'),'string',text_process); drawnow;
            end 
            cd(path)
    end
               
end

end



function [gamma_h]=normal_gravity_generic(phi,h)
%% For calculation of normal gravity at station height, input latitde in deg, and geodetic height in meter
a=6378137;
b=6356752.3141;
gamma_a=9.7803267715;
gamma_b=9.8321863685;
m=0.00344978600308;

pi=3.141592654;
% conversion angle from deg to rad
phi = phi*pi/180;

% auxiliary quantities k,f,e2
k   = b*gamma_b/(a*gamma_a)-1;
f   = (a-b)/a;
a2=a*a;
e2  = (a2-b*b)/(a2);

% Normalschwere nach SOMIGLIANA, streng ------------------------------------	
% Torge 2002, p.95, Eq. 4.41b
sinphi=sin(phi);
gamma_0 = gamma_a * (1+k*sinphi.*sinphi)./sqrt(1-e2.*sinphi.*sinphi);

% Vertikaler Schweregradient, weitgehende Reihenentwicklung bis 2. Ordnung
% Torge 2002, p 98, Eq. 4.63
delta_gamma_h = gamma_0 .*( -2/a * ( 1 + f + m - 2*f*sinphi.*sinphi).* h  + 3/(a2)*h.*h);


gamma_h = gamma_0 + delta_gamma_h;
end

function [minlat, maxlat, minlon, maxlon, reslat, reslon, elev] = read_DEM(fileName)
% Read 16 bit DEM file, and keep it int16 to save memory.
fid = fopen(fileName,'r');
[data, cnt] = fread(fid, inf, 'double','native');

fclose(fid);
minlat=data(1);
maxlat=data(2);
minlon=data(3);
maxlon=data(4);
reslat=data(5);
reslon=data(6);

% number of row and colume
numCols=((maxlon-minlon)/reslon)+1;
numRows = (cnt-6) / numCols;
elev=data(7:end);

% elevation dataset
elev = reshape(elev,round(numCols),round(numRows))';
end

function [lon21,lat21]=Build_grid(lon,lat,extend_x1,extend_y1,extend_x2,extend_y2,resolution_x2,resolution_y2)
% grid building for each zone to avoid overlaping and leakage betwen adjacent zones 
% input: center of zone (lon, lat), inner and outer radius (extend_x1,extend_y1,extend_x2,extend_y2), grid resolution 
% in longitude and latitude direction respectively
lon21=(lon-extend_x2):resolution_x2:(lon+extend_x2-resolution_x2);
lat21=(lat-extend_y2):resolution_y2:(lat+extend_y2-resolution_y2);
[lon21,lat21]=meshgrid(lon21,lat21);
[m n]=size(lat21);

n1=round((extend_y2-extend_y1)/resolution_y2)+1;
n2=m-round((extend_y2-extend_y1)/resolution_y2);
n3=round((extend_x2-extend_x1)/resolution_x2)+1;
n4=n-round((extend_x2-extend_x1)/resolution_x2);

r=n1:n2;
c=n3:n4;
[c r]=meshgrid(c,r);
nn=r(:)+m*(c(:)-1);
clear c r

lon21(nn)=[];
lat21(nn)=[];

lon21=lon21+resolution_x2/2;
lat21=lat21+resolution_y2/2;

 end
 
function [Vx, Vy, Vz]=g_prism2point(gx,gy,gz,lon,lat,prism_lon,prism_lat)
% In order to overcome the effect of Earth's curvature, transform the first derivatives of gravitational in prism-based 
% system to the station-based coordinate system.
Pi=3.141592654;
d2r = Pi/180;

prism_lon=d2r*prism_lon;
prism_lat=d2r*prism_lat;
lon=d2r*lon;
lat=d2r*lat;

cosbeta = cos((prism_lon - lon));
sinbeta = sin((prism_lon - lon));  % 20190623
cosphi = cos(lat);
sinphi = sin(lat);
cosphil = cos(prism_lat);
sinphil = sin(prism_lat);

cosbetasinphi= cosbeta.*sinphi;
cosbetacosphi= cosbeta.*cosphi;
R0 = cosbetasinphi.*sinphil + cosphi.*cosphil;
R1 = sinbeta.*sinphi;
R2 = -cosbetasinphi.*cosphil + cosphi.*sinphil;
R3 = -sinbeta.*sinphil;
R4= cosbeta;
R5 = sinbeta.*cosphil;
R6 = -cosbetacosphi.*sinphil + sinphi.*cosphil;
R7 = -sinbeta.*cosphi;
R8 = cosbetacosphi.*cosphil + sinphi.*sinphil;

Vx=R0.*gx+R1.*gy+R2.*gz;
Vy=R3.*gx+R4.*gy+R5.*gz;
Vz=R6.*gx+R7.*gy+R8.*gz;
end

function [Vxx, Vxy, Vxz, Vyy, Vyz, Vzz]=ggt_prism2point(gxx,gxy,gxz,gyy,gyz,gzz,lon,lat,prism_lon,prism_lat)
% In order to overcome the effect of Earth's curvature, transform the second-order derivatives of gravitational in prism-based 
% system to the station-based coordinate system.
Pi=3.141592654;
d2r = Pi/180;

atprism0=gxx;
atprism1=gxy;
atprism2=gxz;
atprism3=gxy;
atprism4=gyy;
atprism5=gyz;
atprism6=gxz;
atprism7=gyz;
atprism8=-(gxx+gyy);

 
prism_lon=d2r*prism_lon;
prism_lat=d2r*prism_lat;
lon=d2r*lon;
lat=d2r*lat;

cosbeta = cos((prism_lon - lon));
sinbeta = sin((prism_lon - lon)); %20190623
cosphi = cos(lat);
sinphi = sin(lat);
cosphil = cos(prism_lat);
sinphil = sin(prism_lat);


cosbetasinphi=cosbeta.*sinphi;
cosbetacosphi=cosbeta.*cosphi;
R0 = cosbetasinphi.*sinphil + cosphi.*cosphil;
R1 = sinbeta.*sinphi;
R2 = -cosbetasinphi.*cosphil + cosphi.*sinphil;
R3 = -sinbeta.*sinphil;
R4= cosbeta;
R5 = sinbeta.*cosphil;
R6 = -cosbetacosphi.*sinphil + sinphi.*cosphil;
R7 = -sinbeta.*cosphi;
R8 = cosbetacosphi.*cosphil + sinphi.*sinphil;
clear cosbeta sinbeta cosphi sinphi cosphil sinphil cosbetasinphi cosbetacosphi 

t0=R0.*atprism0+R1.*atprism3+R2.*atprism6;
t1=R0.*atprism1+R1.*atprism4+R2.*atprism7;
t2=R0.*atprism2+R1.*atprism5+R2.*atprism8;
t3=R3.*atprism0+R4.*atprism3+R5.*atprism6;
t4=R3.*atprism1+R4.*atprism4+R5.*atprism7;
t5=R3.*atprism2+R4.*atprism5+R5.*atprism8;
t6=R6.*atprism0+R7.*atprism3+R8.*atprism6;
t7=R6.*atprism1+R7.*atprism4+R8.*atprism7;
t8=R6.*atprism2+R7.*atprism5+R8.*atprism8;
clear atprism0 atprism1 atprism2 atprism3 atprism4 atprism5 atprism6 atprism7 atprism8


V0=t0.*R0+t1.*R1+t2.*R2;
V1=t0.*R3+t1.*R4+t2.*R5;
V2=t0.*R6+t1.*R7+t2.*R8;
V4=t3.*R3+t4.*R4+t5.*R5;
V7=t6.*R3+t7.*R4+t8.*R5;
V8=t6.*R6+t7.*R7+t8.*R8;

Vxx=V0;
Vxy=V1;
Vxz=V2;
Vyy=V4;
Vyz=V7;
Vzz=V8;

end

function [x, y, z]=global2local(lon,lat,r,prism_lon,prism_lat,prism_r)
% transform the spherical coordinates of station to prism-based local cartesian coordinates
Pi=3.141592654;
d2r=Pi/180;

prism_lon=d2r*prism_lon;
prism_lat=d2r*prism_lat;
lon=d2r*lon;
lat=d2r*lat;

coslat=cos(lat);
coslon=cos(lon);
sinlat=sin(lat);
sinlon=sin(lon);

cosprism_lon=cos(prism_lon);
cosprism_lat=cos(prism_lat);
sinprism_lon=sin(prism_lon);
sinprism_lat=sin(prism_lat);


g_x = r.*coslat.*coslon -prism_r.*cosprism_lat.*cosprism_lon;
g_y= r.*coslat.*sinlon - prism_r.*cosprism_lat.*sinprism_lon;
g_z= r.*sinlat - prism_r.*sinprism_lat;


x = -g_x.*sinprism_lat.*cosprism_lon - g_y.*sinprism_lat.*sinprism_lon + g_z.*cosprism_lat;
y = -g_x.*sinprism_lon +  g_y.*cosprism_lon;
z = -(g_x.*cosprism_lat.*cosprism_lon + g_y.*cosprism_lat.*sinprism_lon + g_z.*sinprism_lat);

end

function  [g] = Point_g(xx1,xx2,yy1,yy2,r1,r2,lon,lat,r0,e)
% potential calculation of point mass
G = 6.67384e-3;   % Universal Gravitational constant
d2r=3.141592654/180;

point_r=(r1+r2)/2;
point_lon=(xx1+xx2)/2;
point_lat=(yy1+yy2)/2;

% tesseroid equivalent masses
mass=e.*d2r.*(xx2-xx1).*(r1.^3-r2.^3).*(sin(d2r*yy2)-sin(d2r*yy1))/3;

% spherical coordinates to local cartesian coordinates
[dx dy dz]=global2local(lon,lat,r0,point_lon,point_lat,point_r);
l = 1./(sqrt(dx.*dx+dy.*dy+dz.*dz));


c=G*mass;
res = c.*l;
g=-sum(res)*1e-5;

end

function    [Vx, Vy, Vz] = Point_gx(xx1,xx2,yy1,yy2,r1,r2,lon,lat,r0,e)
% calculation of first derivatives of potential generated by point mass
G = 6.67384e-3;   % Universal Gravitational constant
d2r=3.141592654/180;

point_r=(r1+r2)/2;
point_lon=(xx1+xx2)/2;
point_lat=(yy1+yy2)/2;

% tesseroid equivalent masses
mass=d2r*e.*(xx2-xx1).*(r1.^3-r2.^3).*(sin(d2r*yy2)-sin(d2r*yy1))/3;

% spherical coordinates to local cartesian coordinates
[dx dy dz]=global2local(point_lon,point_lat,point_r,lon,lat,r0);
l = 1./(sqrt(dx.*dx+dy.*dy+dz.*dz));
l3 = l.*l.*l;    

c=G*mass;

res=c.*(dx.*l3);
gx=res;
   
res=c.*(dy.*l3);
gy=res;   
   
res=c.*(dz.*l3);
gz=-res;

Vx=sum(gx);
Vy=sum(gy);
Vz=sum(gz);
end

function    [g, Vx, Vy, Vz] = Point_g_gx(xx1,xx2,yy1,yy2,r1,r2,lon,lat,r0,e)
% calculation of potential and its first derivatives of point mass

G = 6.67384e-3;   % Universal Gravitational constant
d2r=3.141592654/180;

point_r=(r1+r2)/2;
point_lon=(xx1+xx2)/2;
point_lat=(yy1+yy2)/2;

% tesseroid equivalent masses
mass=d2r*e.*(xx2-xx1).*(r1.^3-r2.^3).*(sin(d2r*yy2)-sin(d2r*yy1))/3;

% spherical coordinates to local cartesian coordinates
[dx dy dz]=global2local(point_lon,point_lat,point_r,lon,lat,r0);


l = 1./(sqrt(dx.*dx+dy.*dy+dz.*dz));
l3 = l.*l.*l;   

c=G*mass;
res = c.*l;
g=-sum(res(:))*1e-5;

res=c.*(dx.*l3);
gx=res;
   
res=c.*(dy.*l3);
gy=res;   
   
res=c.*(dz.*l3);
gz=res;

Vx=sum(gx);
Vy=sum(gy);
Vz=-sum(gz);

end

function  [Vxx, Vxy, Vxz, Vyy, Vyz, Vzz] = Point_gxx(xx1,xx2,yy1,yy2,r1,r2,lon,lat,r0,e)
% calculation of second-order derivatives of potential generated by point mass
G = 6.67384e-3;   % Universal Gravitational constant
d2r=3.141592654/180;

point_r=(r1+r2)/2;
point_lon=(xx1+xx2)/2;
point_lat=(yy1+yy2)/2;

% tesseroid equivalent masses
mass=d2r*e.*(xx2-xx1).*(r1.^3-r2.^3).*(sin(d2r*yy2)-sin(d2r*yy1))/3;

% spherical coordinates to local cartesian coordinates
[dx, dy, dz]=global2local(point_lon,point_lat,point_r,lon,lat,r0);

l = 1./(sqrt(dx.*dx+dy.*dy+dz.*dz));
l3 = l.*l.*l;    
l5 = l3.*l.*l;  
c=G*mass;

res = c.*(-l3 + 3.*dx.*dx.*l5);
gxx=-res;
res = c.*(3*dx.*dy.*l5);
gxy=-res;
res = c.*(3*dx.*dz.*l5);
gxz=res;
res = c.*(-l3 + 3*dy.*dy.*l5);
gyy=-res;
res = c.*(3*dy.*dz.*l5);
gyz=res;
res = c.*(-l3 + 3*dz.*dz.*l5);   
gzz=-res;


Vxx=sum(gxx)*1e4;
Vxy=sum(gxy)*1e4;
Vxz=sum(gxz)*1e4;
Vyy=sum(gyy)*1e4;
Vyz=sum(gyz)*1e4;
Vzz=sum(gzz)*1e4;
%Vzz=-(Vxx+Vyy);

end

function  [g, Vx, Vy, Vz, Vxx, Vxy, Vxz, Vyy, Vyz, Vzz] = Point_mass(xx1,xx2,yy1,yy2,r1,r2,lon,lat,r0,e)
% calculation of all gravitational functionals generated by point-mass
G = 6.67384e-3;   % Universal Gravitational constant
d2r=3.141592654/180;

point_r=(r1+r2)/2;
point_lon=(xx1+xx2)/2;
point_lat=(yy1+yy2)/2;

% tesseroid equivalent masses
mass=d2r*e.*(xx2-xx1).*(r1.^3-r2.^3).*(sin(d2r*yy2)-sin(d2r*yy1))/3;

% spherical coordinates to local cartesian coordinates
[dx dy dz]=global2local(point_lon,point_lat,point_r,lon,lat,r0);


l = 1./(sqrt(dx.*dx+dy.*dy+dz.*dz));
l3 = l.*l.*l;    
l5 = l3.*l.*l;  
c=G*mass;

res = c.*l;
g=-sum(res)*1e-5;

res=c.*(dx.*l3);
gx=res;
   
res=c.*(dy.*l3);
gy=res;   
   
res=c.*(dz.*l3);
gz=-res;


Vx=sum(gx);
Vy=sum(gy);
Vz=sum(gz);
   
res = c.*(-l3 + 3*dx.*dx.*l5);
gxx=-res;
res = c.*(3*dx.*dy.*l5);
gxy=-res;
res = c.*(3*dx.*dz.*l5);
gxz=res;
res = c.*(-l3 + 3*dy.*dy.*l5);
gyy=-res;
res = c.*(3*dy.*dz.*l5);
gyz=res;
res = c.*(-l3 + 3*dz.*dz.*l5);   
gzz=-res;

Vxx=sum(gxx)*1e4;
Vxy=sum(gxy)*1e4;
Vxz=sum(gxz)*1e4;
Vyy=sum(gyy)*1e4;
Vyz=sum(gyz)*1e4;
Vzz=sum(gzz)*1e4;
% Vzz=-(Vxx+Vyy);

end

function  [V, Vx, Vy, Vz, Vxx, Vxy, Vxz, Vyy, Vyz, Vzz]= Polyhedron_full( x1_grid,y1_grid,z1_grid,z1_ref,Dov_lat,Dov_lon,Dov_h,extend_x1,extend_y1,e1,resolution_x,resolution_y)
%UNTITLED Summary of this function goes here
%   Detailed explanation goes here
R=6371000;
Pi=3.141592654;
d2r=Pi/180;
G=6.67384e-8;


lat_p=y1_grid(1:end,1);
lon_p=x1_grid(1,1:end);
[m n]=size(Dov_lon);
size_com=m*n;  % number of computation point

[V, Vx, Vy, Vz, Vxx, Vxy, Vxz, Vyy, Vyz, Vzz] = matrix_preallocating(1,size_com);
for i=1:size_com
  lon=Dov_lon(i);
  lat=Dov_lat(i);
  r0=Dov_h(i);


  n=find(lon_p<=(Dov_lon(i))+extend_x1 & lon_p>=(Dov_lon(i))-extend_x1);           % coordinates of considered integration masses 
  m=find(lat_p<=(Dov_lat(i))+extend_y1 & lat_p>=(Dov_lat(i))-extend_y1);

  lon1=lon_p(n);
  lat1=lat_p(m);
  zz_botom=z1_ref(m,n);          %radius of lower boundary 
  zz=z1_grid(m,n);               % radius of upper boundary
  e=e1(m,n);

  [lon1, lat1]=meshgrid(lon1,lat1);                
  lonp=1/2*(lon1(1:end-1,1:end-1)+lon1(2:end,2:end));
  latp=1/2*(lat1(1:end-1,1:end-1)+lat1(2:end,2:end));
  zzp=1/4*(zz(1:end-1,1:end-1)+zz(2:end,2:end)+zz(1:end-1,2:end)+zz(2:end,1:end-1));   % define origin of polyhedron-based coordinate system
  [x, y, z]=global2local(lon,lat,r0,lonp,latp,zzp);            % coordinates of calculation point in polyhedron-based Cartesian coordinate system
% coordinates of corners of polyhedron in polyhedron-based Cartesian coordinate system
  dx=zzp*d2r*resolution_y;
  dy=zzp.*cos(d2r*latp).*d2r.*resolution_x;
  dx=0.5*dx;
  dy=0.5*dy; 
  
  
  xx1=-dx-x;
  yy1=-dy-y;
  zz1=zz(1:end-1,1:end-1)-zzp-z;

  xx2=-dx-x;
  yy2=dy-y;
  zz2=zz(1:end-1,2:end)-zzp-z;

  xx3=dx-x;
  yy3=dy-y;
  zz3=zz(2:end,2:end)-zzp-z;

  xx4=dx-x;
  yy4=-dy-y;
  zz4=zz(2:end,1:end-1)-zzp-z;

  zz_botom=1/4*(zz_botom(1:end-1,1:end-1)+zz_botom(2:end,2:end)+zz_botom(1:end-1,2:end)+zz_botom(2:end,1:end-1));
  xx5=xx1;
  yy5=yy1;
  zz5= zz_botom-zzp-z;

  xx6=xx2;
  yy6=yy2;
  zz6=zz_botom-zzp-z;


  xx7=xx3;
  yy7=yy3;
  zz7=zz_botom-zzp-z;


  xx8=xx4;
  yy8=yy4;
  zz8=zz_botom-zzp-z;
  

  e=e(1:end-1,1:end-1);
  e_d=e(:);
  [m, n]=size(xx1);
  size1=m*n;
% irregular polyhedron detecting
  f1=(zz1-zz5).*(zz2-zz6)<0;
  f2=(zz2-zz6).*(zz3-zz7)<0;
  f3=(zz3-zz7).*(zz4-zz8)<0;
  f4=(zz1-zz5).*(zz4-zz8)<0;
  n1=find( f1  & f2 );
  n2=find( f3  & f2);
  n3=find(f3  & f4 );
  n4=find( f1  & f4);
  n5=find( f4  & f2);
  n6=find(f1  & f3);
  n=[n1;n2; n3; n4; n5; n6];
  n=unique(n);
  [p, l]=size(n);
  p=p*l;
  temp=zeros(p,1)-10000;
  x_s=[xx1(n) xx2(n)  xx3(n) xx4(n)  xx5(n) xx6(n) xx7(n) xx8(n)];
  y_s=[yy1(n) yy2(n)  yy3(n) yy4(n)  yy5(n) yy6(n) yy7(n) yy8(n)];
  z_s=[temp temp  temp temp  zz5(n) zz6(n) zz7(n) zz8(n)];
  lon_s=lonp(n);
  lat_s=latp(n);
  
  zz5(n)=-10000;
  zz6(n)=-10000;
  zz7(n)=-10000;
  zz8(n)=-10000;

  x=[xx1(:) xx2(:)  xx3(:) xx4(:)  xx5(:) xx6(:) xx7(:) xx8(:)];
  y=[yy1(:) yy2(:)  yy3(:) yy4(:)  yy5(:) yy6(:) yy7(:) yy8(:)];
  z=[zz1(:) zz2(:)  zz3(:) zz4(:)  zz5(:) zz6(:) zz7(:) zz8(:)];
  
  x=[x; x_s];
  y=[y; y_s];
  z=[z; z_s];
  
  f1=(abs(x)<0.05);
  x(f1)=x(f1)+0.1; 

  f1=(abs(y)<0.05);
  y(f1)=y(f1)+0.1;
  
  lonp=[lonp(:);lon_s];
  latp=[latp(:);lat_s];
  e_d=[e_d;e_d(n)];
  size1=size1+p;

% call function polyhedron_full in fortran
  g=polyhedron_full(size1,x,y,z);
  g(1:size1,4)=-g(1:size1,4);
  [temp_Vx temp_Vy temp_Vz]=g_prism2point(g(1:size1,2),g(1:size1,3),g(1:size1,4),lon,lat,lonp(:),latp(:));
  [temp_Vxx temp_Vxy temp_Vxz temp_Vyy temp_Vyz temp_Vzz]=ggt_prism2point(g(1:size1,5),g(1:size1,6),g(1:size1,7),g(1:size1,8),g(1:size1,9),g(1:size1,10),lon,lat,lonp(:),latp(:)); 
  c=G*e_d;
  
  V(i)=nansum(c.*g(1:size1,1));
  Vx(i)=nansum(c.*temp_Vx);
  Vy(i)=nansum(c.*temp_Vy);
  Vz(i)=nansum(c.*temp_Vz);
  Vxx(i)=nansum(c.*temp_Vxx);
  Vxy(i)=nansum(c.*temp_Vxy);
  Vxz(i)=nansum(c.*temp_Vxz);
  Vyy(i)=nansum(c.*temp_Vyy);
  Vyz(i)=nansum(c.*temp_Vyz);
  Vzz(i)=nansum(c.*temp_Vzz);
  clear x y z 
end
V=V/2;
Vx=Vx*1e5;
Vy=Vy*1e5;
Vz=-Vz*1e5;

Vxx=Vxx*1e9;
Vxy=Vxy*1e9;
Vxz=Vxz*1e9;
Vyy=Vyy*1e9;
Vyz=Vyz*1e9;
Vzz=Vzz*1e9;
end

function  [V, Vx, Vy, Vz, Vxx, Vxy, Vxz, Vyy, Vyz, Vzz]= Polyhedron_g( x1_grid,y1_grid,z1_grid,z1_ref,Dov_lat,Dov_lon,Dov_h,extend_x1,extend_y1,e1,resolution_x,resolution_y)
G= 6.67384e-8; 
Pi=3.141592654;
R=6371000;
d2r=Pi/180;

lat_p=y1_grid(1:end,1);
lon_p=x1_grid(1,1:end);
[m, n]=size(Dov_lon);      % number of computation point
size_com=m*n;    
[V, Vx, Vy, Vz, Vxx, Vxy, Vxz, Vyy, Vyz, Vzz] = matrix_preallocating(1,size_com);

for i=1:size_com
  lon=Dov_lon(i);
  lat=Dov_lat(i);
  r0=Dov_h(i);


  n=find(lon_p<=(Dov_lon(i))+extend_x1 & lon_p>=(Dov_lon(i))-extend_x1);            % coordinates of considered integration masses 
  m=find(lat_p<=(Dov_lat(i))+extend_y1 & lat_p>=(Dov_lat(i))-extend_y1);

  lon1=lon_p(n);
  lat1=lat_p(m);
  zz_botom=z1_ref(m,n); %radius of lower boundary
  zz=z1_grid(m,n);  % radius of upper boundary
  e=e1(m,n);

% define origin of polyhedron-based coordinate system
  [lon1, lat1]=meshgrid(lon1,lat1);    
  lonp=1/2*(lon1(1:end-1,1:end-1)+lon1(2:end,2:end));
  latp=1/2*(lat1(1:end-1,1:end-1)+lat1(2:end,2:end));
  zzp=1/4*(zz(1:end-1,1:end-1)+zz(2:end,2:end)+zz(1:end-1,2:end)+zz(2:end,1:end-1));
  [x, y, z]=global2local(lon,lat,r0,lonp,latp,zzp); % coordinates of calculation point in polyhedron-based Cartesian coordinate system

% coordinates of corners of polyhedron in polyhedron-based Cartesian coordinate system 
  dx=zzp*d2r*resolution_y;
  dy=zzp.*cos(d2r*latp).*d2r.*resolution_x;
  dx=0.5*dx;
  dy=0.5*dy; 
  
  
  xx1=-dx-x;
  yy1=-dy-y;
  zz1=zz(1:end-1,1:end-1)-zzp-z;

  xx2=-dx-x;
  yy2=dy-y;
  zz2=zz(1:end-1,2:end)-zzp-z;

  xx3=dx-x;
  yy3=dy-y;
  zz3=zz(2:end,2:end)-zzp-z;

  xx4=dx-x;
  yy4=-dy-y;
  zz4=zz(2:end,1:end-1)-zzp-z;

  zz_botom=1/4*(zz_botom(1:end-1,1:end-1)+zz_botom(2:end,2:end)+zz_botom(1:end-1,2:end)+zz_botom(2:end,1:end-1));
  xx5=xx1;
  yy5=yy1;
  zz5= zz_botom-zzp-z;

  xx6=xx2;
  yy6=yy2;
  zz6=zz_botom-zzp-z;


  xx7=xx3;
  yy7=yy3;
  zz7=zz_botom-zzp-z;


  xx8=xx4;
  yy8=yy4;
  zz8=zz_botom-zzp-z;
  

  e=e(1:end-1,1:end-1);
  e_d=e(:);
  [m, n]=size(xx1);
  size1=m*n;
 % irregular polyhedron detecting
  f1=(zz1-zz5).*(zz2-zz6)<0;
  f2=(zz2-zz6).*(zz3-zz7)<0;
  f3=(zz3-zz7).*(zz4-zz8)<0;
  f4=(zz1-zz5).*(zz4-zz8)<0;
  n1=find( f1  & f2 );
  n2=find( f3  & f2);
  n3=find(f3  & f4 );
  n4=find( f1  & f4);
  n5=find( f4  & f2);
  n6=find(f1  & f3);
  n=[n1;n2; n3; n4; n5; n6];
  n=unique(n);
  [p, l]=size(n);
  p=p*l;
  temp=zeros(p,1)-10000;
  x_s=[xx1(n) xx2(n)  xx3(n) xx4(n)  xx5(n) xx6(n) xx7(n) xx8(n)];
  y_s=[yy1(n) yy2(n)  yy3(n) yy4(n)  yy5(n) yy6(n) yy7(n) yy8(n)];
  z_s=[temp temp  temp temp  zz5(n) zz6(n) zz7(n) zz8(n)];
  lon_s=lonp(n);
  lat_s=latp(n);
  
  zz5(n)=-10000;
  zz6(n)=-10000;
  zz7(n)=-10000;
  zz8(n)=-10000;

  x=[xx1(:) xx2(:)  xx3(:) xx4(:)  xx5(:) xx6(:) xx7(:) xx8(:)];
  y=[yy1(:) yy2(:)  yy3(:) yy4(:)  yy5(:) yy6(:) yy7(:) yy8(:)];
  z=[zz1(:) zz2(:)  zz3(:) zz4(:)  zz5(:) zz6(:) zz7(:) zz8(:)];
  
  x=[x; x_s];
  y=[y; y_s];
  z=[z; z_s];
  
  f1=(abs(x)<0.05);
  x(f1)=x(f1)+0.1; 

  f1=(abs(y)<0.05);
  y(f1)=y(f1)+0.1;
  
  lonp=[lonp(:);lon_s];
  latp=[latp(:);lat_s];
  e_d=[e_d;e_d(n)];
  size1=size1+p;

% call function polyhedron_g in fortran
  g=polyhedron_g(size1,x,y,z);
  c=G*e_d;
  
  V(i)=nansum(c.*g(1:size1,1));
clear x y z
end
V=V/2;
Vx=0;
Vy=0;
Vz=0;

Vxx=0;
Vxy=0;
Vxz=0;
Vyy=0;
Vyz=0;
Vzz=0;
end

function  [V, Vx, Vy, Vz, Vxx, Vxy, Vxz, Vyy, Vyz, Vzz]= Polyhedron_g_gx( x1_grid,y1_grid,z1_grid,z1_ref,Dov_lat,Dov_lon,Dov_h,extend_x1,extend_y1,e1,resolution_x,resolution_y)
G= 6.67384e-8; 
Pi=3.141592654;
R=6371000;
d2r=Pi/180;

lat_p=y1_grid(1:end,1);
lon_p=x1_grid(1,1:end);
[m n]=size(Dov_lon);      % number of computation point
size_com=m*n;


[V, Vx, Vy, Vz, Vxx, Vxy, Vxz, Vyy, Vyz, Vzz] = matrix_preallocating(1,size_com);
for i=1:size_com
  lon=Dov_lon(i);
  lat=Dov_lat(i);
  r0=Dov_h(i);


  n=find(lon_p<=(Dov_lon(i))+extend_x1 & lon_p>=(Dov_lon(i))-extend_x1);     % coordinates of considered integration masses 
  m=find(lat_p<=(Dov_lat(i))+extend_y1 & lat_p>=(Dov_lat(i))-extend_y1);

  lon1=lon_p(n);
  lat1=lat_p(m);
  zz_botom=z1_ref(m,n);  %radius of lower boundary 
  zz=z1_grid(m,n);         % radius of upper boundary
  e=e1(m,n);

  [lon1, lat1]=meshgrid(lon1,lat1);    % define origin of polyhedron-based coordinate system
  lonp=1/2*(lon1(1:end-1,1:end-1)+lon1(2:end,2:end));
  latp=1/2*(lat1(1:end-1,1:end-1)+lat1(2:end,2:end));
  zzp=1/4*(zz(1:end-1,1:end-1)+zz(2:end,2:end)+zz(1:end-1,2:end)+zz(2:end,1:end-1));
  [x, y, z]=global2local(lon,lat,r0,lonp,latp,zzp);  % coordinates of calculation point in polyhedron-based Cartesian coordinate system
% coordinates of corners of polyhedron in polyhedron-based Cartesian coordinate system
  dx=zzp*d2r*resolution_y;
  dy=zzp.*cos(d2r*latp).*d2r.*resolution_x;
  dx=0.5*dx;
  dy=0.5*dy; 
  
  
  xx1=-dx-x;
  yy1=-dy-y;
  zz1=zz(1:end-1,1:end-1)-zzp-z;

  xx2=-dx-x;
  yy2=dy-y;
  zz2=zz(1:end-1,2:end)-zzp-z;

  xx3=dx-x;
  yy3=dy-y;
  zz3=zz(2:end,2:end)-zzp-z;

  xx4=dx-x;
  yy4=-dy-y;
  zz4=zz(2:end,1:end-1)-zzp-z;

  zz_botom=1/4*(zz_botom(1:end-1,1:end-1)+zz_botom(2:end,2:end)+zz_botom(1:end-1,2:end)+zz_botom(2:end,1:end-1));
  xx5=xx1;
  yy5=yy1;
  zz5= zz_botom-zzp-z;

  xx6=xx2;
  yy6=yy2;
  zz6=zz_botom-zzp-z;


  xx7=xx3;
  yy7=yy3;
  zz7=zz_botom-zzp-z;


  xx8=xx4;
  yy8=yy4;
  zz8=zz_botom-zzp-z;
  

  e=e(1:end-1,1:end-1);
  e_d=e(:);
  [m, n]=size(xx1);
  size1=m*n;
 % irregular polyhedron detecting
  f1=(zz1-zz5).*(zz2-zz6)<0;
  f2=(zz2-zz6).*(zz3-zz7)<0;
  f3=(zz3-zz7).*(zz4-zz8)<0;
  f4=(zz1-zz5).*(zz4-zz8)<0;
  n1=find( f1  & f2 );
  n2=find( f3  & f2);
  n3=find(f3  & f4 );
  n4=find( f1  & f4);
  n5=find( f4  & f2);
  n6=find(f1  & f3);
  n=[n1;n2; n3; n4; n5; n6];
  n=unique(n);
  [p, l]=size(n);
  p=p*l;
  temp=zeros(p,1)-10000;
  x_s=[xx1(n) xx2(n)  xx3(n) xx4(n)  xx5(n) xx6(n) xx7(n) xx8(n)];
  y_s=[yy1(n) yy2(n)  yy3(n) yy4(n)  yy5(n) yy6(n) yy7(n) yy8(n)];
  z_s=[temp temp  temp temp  zz5(n) zz6(n) zz7(n) zz8(n)];
  lon_s=lonp(n);
  lat_s=latp(n);
  
  zz5(n)=-10000;
  zz6(n)=-10000;
  zz7(n)=-10000;
  zz8(n)=-10000;

  x=[xx1(:) xx2(:)  xx3(:) xx4(:)  xx5(:) xx6(:) xx7(:) xx8(:)];
  y=[yy1(:) yy2(:)  yy3(:) yy4(:)  yy5(:) yy6(:) yy7(:) yy8(:)];
  z=[zz1(:) zz2(:)  zz3(:) zz4(:)  zz5(:) zz6(:) zz7(:) zz8(:)];
  
  x=[x; x_s];
  y=[y; y_s];
  z=[z; z_s];
  
  f1=(abs(x)<0.05);
  x(f1)=x(f1)+0.1; 

  f1=(abs(y)<0.05);
  y(f1)=y(f1)+0.1;
  
  lonp=[lonp(:);lon_s];
  latp=[latp(:);lat_s];
  e_d=[e_d;e_d(n)];
  size1=size1+p;
  
  % call function polyhedron_g_gx in fortran
  g=polyhedron_g_gx(size1,x,y,z);
  g(1:size1,4)=-g(1:size1,4);
  [temp_Vx temp_Vy temp_Vz]=g_prism2point(g(1:size1,2),g(1:size1,3),g(1:size1,4),lon,lat,lonp(:),latp(:));
  c=G*e_d;
  
  V(i)=nansum(c.*g(1:size1,1));
  Vx(i)=nansum(c.*temp_Vx);
  Vy(i)=nansum(c.*temp_Vy);
  Vz(i)=-nansum(c.*temp_Vz);
  clear x y z
end
V=V/2;
Vx=Vx*1e5;
Vy=Vy*1e5;
Vz=Vz*1e5;

Vxx=0;
Vxy=0;
Vxz=0;
Vyy=0;
Vyz=0;
Vzz=0;
end

function  [V, Vx, Vy, Vz, Vxx, Vxy, Vxz, Vyy, Vyz, Vzz]= Polyhedron_gx( x1_grid,y1_grid,z1_grid,z1_ref,Dov_lat,Dov_lon,Dov_h,extend_x1,extend_y1,e1,resolution_x,resolution_y)
%UNTITLED Summary of this function goes here
%   Detailed explanation goes here
G = 6.67384e-8;
Pi=3.141592654;
d2r=Pi/180;

lat_p=y1_grid(1:end,1);   
lon_p=x1_grid(1,1:end);
[m n]=size(Dov_lon);              % number of computation point
size_com=m*n;

[V, Vx, Vy, Vz, Vxx, Vxy, Vxz, Vyy, Vyz, Vzz] = matrix_preallocating(1,size_com);
for i=1:size_com
  lon=Dov_lon(i);
  lat=Dov_lat(i);
  r0=Dov_h(i);


  n=find(lon_p<=(Dov_lon(i))+extend_x1 & lon_p>=(Dov_lon(i))-extend_x1);       % coordinates of considered integration masses 
  m=find(lat_p<=(Dov_lat(i))+extend_y1 & lat_p>=(Dov_lat(i))-extend_y1);

  lon1=lon_p(n);
  lat1=lat_p(m);
  zz_botom=z1_ref(m,n);  %radius of lower boundary 
  zz=z1_grid(m,n);  %radius of upper boundary 
  e=e1(m,n);

  [lon1, lat1]=meshgrid(lon1,lat1);    % define origin of polyhedron-based coordinate system
  lonp=1/2*(lon1(1:end-1,1:end-1)+lon1(2:end,2:end));
  latp=1/2*(lat1(1:end-1,1:end-1)+lat1(2:end,2:end));
  zzp=1/4*(zz(1:end-1,1:end-1)+zz(2:end,2:end)+zz(1:end-1,2:end)+zz(2:end,1:end-1));
  [x, y, z]=global2local(lon,lat,r0,lonp,latp,zzp);  % coordinates of calculation point in polyhedron-based Cartesian coordinate system
% coordinates of corners of polyhedron in polyhedron-based Cartesian coordinate system
  dx=zzp*d2r*resolution_y;
  dy=zzp.*cos(d2r*latp).*d2r.*resolution_x;
  dx=0.5*dx;
  dy=0.5*dy; 
  
  
  xx1=-dx-x;
  yy1=-dy-y;
  zz1=zz(1:end-1,1:end-1)-zzp-z;

  xx2=-dx-x;
  yy2=dy-y;
  zz2=zz(1:end-1,2:end)-zzp-z;

  xx3=dx-x;
  yy3=dy-y;
  zz3=zz(2:end,2:end)-zzp-z;

  xx4=dx-x;
  yy4=-dy-y;
  zz4=zz(2:end,1:end-1)-zzp-z;

  zz_botom=1/4*(zz_botom(1:end-1,1:end-1)+zz_botom(2:end,2:end)+zz_botom(1:end-1,2:end)+zz_botom(2:end,1:end-1));
  xx5=xx1;
  yy5=yy1;
  zz5= zz_botom-zzp-z;

  xx6=xx2;
  yy6=yy2;
  zz6=zz_botom-zzp-z;


  xx7=xx3;
  yy7=yy3;
  zz7=zz_botom-zzp-z;


  xx8=xx4;
  yy8=yy4;
  zz8=zz_botom-zzp-z;
  

  e=e(1:end-1,1:end-1);
  e_d=e(:);
  [m, n]=size(xx1);
  size1=m*n;
 % irregular polyhedron detecting
  f1=(zz1-zz5).*(zz2-zz6)<0;
  f2=(zz2-zz6).*(zz3-zz7)<0;
  f3=(zz3-zz7).*(zz4-zz8)<0;
  f4=(zz1-zz5).*(zz4-zz8)<0;
  n1=find( f1  & f2 );
  n2=find( f3  & f2);
  n3=find(f3  & f4 );
  n4=find( f1  & f4);
  n5=find( f4  & f2);
  n6=find(f1  & f3);
  n=[n1;n2; n3; n4; n5; n6];
  n=unique(n);
  [p, l]=size(n);
  p=p*l;
  temp=zeros(p,1)-10000;
  x_s=[xx1(n) xx2(n)  xx3(n) xx4(n)  xx5(n) xx6(n) xx7(n) xx8(n)];
  y_s=[yy1(n) yy2(n)  yy3(n) yy4(n)  yy5(n) yy6(n) yy7(n) yy8(n)];
  z_s=[temp temp  temp temp  zz5(n) zz6(n) zz7(n) zz8(n)];
  lon_s=lonp(n);
  lat_s=latp(n);
  
  zz5(n)=-10000;
  zz6(n)=-10000;
  zz7(n)=-10000;
  zz8(n)=-10000;

  x=[xx1(:) xx2(:)  xx3(:) xx4(:)  xx5(:) xx6(:) xx7(:) xx8(:)];
  y=[yy1(:) yy2(:)  yy3(:) yy4(:)  yy5(:) yy6(:) yy7(:) yy8(:)];
  z=[zz1(:) zz2(:)  zz3(:) zz4(:)  zz5(:) zz6(:) zz7(:) zz8(:)];
  
  x=[x; x_s];
  y=[y; y_s];
  z=[z; z_s];
  
  f1=(abs(x)<0.05);
  x(f1)=x(f1)+0.1; 

  f1=(abs(y)<0.05);
  y(f1)=y(f1)+0.1;
  
  lonp=[lonp(:);lon_s];
  latp=[latp(:);lat_s];
  e_d=[e_d;e_d(n)];
  size1=size1+p; 
% call function polyhedron_gx in fortran  
  g=polyhedron_gx(size1,x,y,z);
  g(1:size1,4)=-g(1:size1,4);
  [temp_Vx, temp_Vy, temp_Vz]=g_prism2point(g(1:size1,2),g(1:size1,3),g(1:size1,4),lon,lat,lonp(:),latp(:));
   c=G*e_d;
  
  Vx(i)=nansum(c.*temp_Vx);
  Vy(i)=nansum(c.*temp_Vy);
  Vz(i)=-nansum(c.*temp_Vz);
  clear x y z 
end
V=0;
Vx=Vx*1e5;
Vy=Vy*1e5;
Vz=Vz*1e5;

Vxx=0;
Vxy=0;
Vxz=0;
Vyy=0;
Vyz=0;
Vzz=0;

end

function  [V, Vx, Vy, Vz, Vxx, Vxy, Vxz, Vyy, Vyz, Vzz]= Polyhedron_gxx( x1_grid,y1_grid,z1_grid,z1_ref,Dov_lat,Dov_lon,Dov_h,extend_x1,extend_y1,e1,resolution_x,resolution_y)
%UNTITLED Summary of this function goes here
%   Detailed explanation goes here
G= 6.67384e-8; 
Pi=3.141592654;
R=6371000;
d2r=Pi/180;

lat_p=y1_grid(1:end,1);
lon_p=x1_grid(1,1:end);
[m, n]=size(Dov_lon);  % number of computation point
size_com=m*n;

[V, Vx, Vy, Vz, Vxx, Vxy, Vxz, Vyy, Vyz, Vzz] = matrix_preallocating(1,size_com);
for i=1:size_com
  lon=Dov_lon(i);
  lat=Dov_lat(i);
  r0=Dov_h(i);


  n=find(lon_p<=(Dov_lon(i))+extend_x1 & lon_p>=(Dov_lon(i))-extend_x1);   % coordinates of considered integration masses 
  m=find(lat_p<=(Dov_lat(i))+extend_y1 & lat_p>=(Dov_lat(i))-extend_y1);

  lon1=lon_p(n);
  lat1=lat_p(m);
  zz_botom=z1_ref(m,n);    %radius of lower boundary 
  zz=z1_grid(m,n);         % radius of upper boundary
  e=e1(m,n);

  [lon1, lat1]=meshgrid(lon1,lat1);    % define origin of polyhedron-based coordinate system
  lonp=1/2*(lon1(1:end-1,1:end-1)+lon1(2:end,2:end));
  latp=1/2*(lat1(1:end-1,1:end-1)+lat1(2:end,2:end));
  zzp=1/4*(zz(1:end-1,1:end-1)+zz(2:end,2:end)+zz(1:end-1,2:end)+zz(2:end,1:end-1));
  [x, y, z]=global2local(lon,lat,r0,lonp,latp,zzp);% coordinates of calculation point in polyhedron-based Cartesian coordinate system
% coordinates of corners of polyhedron in polyhedron-based Cartesian coordinate system
  dx=zzp*d2r*resolution_y;
  dy=zzp.*cos(d2r*latp).*d2r.*resolution_x;
  dx=0.5*dx;
  dy=0.5*dy; 
  
  
  xx1=-dx-x;
  yy1=-dy-y;
  zz1=zz(1:end-1,1:end-1)-zzp-z;

  xx2=-dx-x;
  yy2=dy-y;
  zz2=zz(1:end-1,2:end)-zzp-z;

  xx3=dx-x;
  yy3=dy-y;
  zz3=zz(2:end,2:end)-zzp-z;

  xx4=dx-x;
  yy4=-dy-y;
  zz4=zz(2:end,1:end-1)-zzp-z;

  zz_botom=1/4*(zz_botom(1:end-1,1:end-1)+zz_botom(2:end,2:end)+zz_botom(1:end-1,2:end)+zz_botom(2:end,1:end-1));
  xx5=xx1;
  yy5=yy1;
  zz5= zz_botom-zzp-z;

  xx6=xx2;
  yy6=yy2;
  zz6=zz_botom-zzp-z;


  xx7=xx3;
  yy7=yy3;
  zz7=zz_botom-zzp-z;


  xx8=xx4;
  yy8=yy4;
  zz8=zz_botom-zzp-z;
  

  e=e(1:end-1,1:end-1);
  e_d=e(:);
  [m, n]=size(xx1);
  size1=m*n;
 % irregular polyhedron detecting
  f1=(zz1-zz5).*(zz2-zz6)<0;
  f2=(zz2-zz6).*(zz3-zz7)<0;
  f3=(zz3-zz7).*(zz4-zz8)<0;
  f4=(zz1-zz5).*(zz4-zz8)<0;
  n1=find( f1  & f2 );
  n2=find( f3  & f2);
  n3=find(f3  & f4 );
  n4=find( f1  & f4);
  n5=find( f4  & f2);
  n6=find(f1  & f3);
  n=[n1;n2; n3; n4; n5; n6];
  n=unique(n);
  [p, l]=size(n);
  p=p*l;
  temp=zeros(p,1)-10000;
  x_s=[xx1(n) xx2(n)  xx3(n) xx4(n)  xx5(n) xx6(n) xx7(n) xx8(n)];
  y_s=[yy1(n) yy2(n)  yy3(n) yy4(n)  yy5(n) yy6(n) yy7(n) yy8(n)];
  z_s=[temp temp  temp temp  zz5(n) zz6(n) zz7(n) zz8(n)];
  lon_s=lonp(n);
  lat_s=latp(n);
  
  zz5(n)=-10000;
  zz6(n)=-10000;
  zz7(n)=-10000;
  zz8(n)=-10000;

  x=[xx1(:) xx2(:)  xx3(:) xx4(:)  xx5(:) xx6(:) xx7(:) xx8(:)];
  y=[yy1(:) yy2(:)  yy3(:) yy4(:)  yy5(:) yy6(:) yy7(:) yy8(:)];
  z=[zz1(:) zz2(:)  zz3(:) zz4(:)  zz5(:) zz6(:) zz7(:) zz8(:)];
  
  x=[x; x_s];
  y=[y; y_s];
  z=[z; z_s];
  
  f1=(abs(x)<0.05);
  x(f1)=x(f1)+0.1; 

  f1=(abs(y)<0.05);
  y(f1)=y(f1)+0.1;
  
  lonp=[lonp(:);lon_s];
  latp=[latp(:);lat_s];
  e_d=[e_d;e_d(n)];
  size1=size1+p;

% call function polyhedron_full in fortran
  g=polyhedron_gxx(size1,x,y,z);
  [temp_Vxx temp_Vxy temp_Vxz temp_Vyy temp_Vyz temp_Vzz]=ggt_prism2point(g(1:size1,5),g(1:size1,6),g(1:size1,7),g(1:size1,8),g(1:size1,9),g(1:size1,10),lon,lat,lonp(:),latp(:)); 
  c=G*e_d;
  
  Vxx(i)=nansum(c.*temp_Vxx);
  Vxy(i)=nansum(c.*temp_Vxy);
  Vxz(i)=nansum(c.*temp_Vxz);
  Vyy(i)=nansum(c.*temp_Vyy);
  Vyz(i)=nansum(c.*temp_Vyz);
  Vzz(i)=nansum(c.*temp_Vzz);
clear x y z 
end
V=0;
Vx=0;
Vy=0;
Vz=0;

Vxx=Vxx*1e9;
Vxy=Vxy*1e9;
Vxz=Vxz*1e9;
Vyy=Vyy*1e9;
Vyz=Vyz*1e9;
Vzz=Vzz*1e9;

end



function [g, Vx, Vy, Vz, Vxx, Vxy, Vxz, Vyy, Vyz, Vzz]=Prism(x1,x2,y1,y2,d1,d2,lon,lat,r0,e,resolution_x,resolution_y)
G = 6.67384e-3;   
Pi=3.141592654;
R=6371000;
d2r=Pi/180;


% definitation of prism and prism-based Cartesian coordinate system
[prism_x1 prism_x2 prism_y1 prism_y2 prism_z1 prism_z2 prism_lon prism_lat prism_r,e]=tess2prism(x1,x2,y1,y2,d1,d2,e);


c = G*e;  % multiplication of gravitational constant and density 



[x y z]=global2local(lon,lat,r0,prism_lon,prism_lat,prism_r); %coordinates of calculation point in prism-based Cartesian coordinate system

xpos2=(prism_x2-x);
xpos1=(prism_x1-x);
ypos2=(prism_y2-y);
ypos1=(prism_y1-y);
zpos2=(prism_z1-z);
zpos1=(prism_z2-z);

% detecting irregular points
fx1=abs(xpos1)<=0.05;
nx1=find(fx1);
fx2=abs(xpos2)<=0.05;
nx2=find(fx2);
fy2=abs(ypos2)<=0.05;
ny2=find(fy2);
fy1=abs(ypos1)<=0.05;
ny1=find(fy1);
fz2=abs(zpos2)<=0.05;
nz2=find(fz2);
fz1=abs(zpos1)<=0.05;
nz1=find(fz1);    


r1 = sqrt(xpos1.*xpos1 + ypos1.*ypos1 + (zpos1).*(zpos1)); 
r2 = sqrt(xpos2.*xpos2 + ypos1.*ypos1 + (zpos1).*(zpos1)); 
r3 = sqrt(xpos1.*xpos1 + ypos2.*ypos2 + (zpos1).*(zpos1)); 
r4 = sqrt(xpos2.*xpos2 + ypos2.*ypos2 + (zpos1).*(zpos1)); 
r5 = sqrt(xpos1.*xpos1 + ypos1.*ypos1 + (zpos2).*(zpos2)); 
r6 = sqrt(xpos2.*xpos2 + ypos1.*ypos1 + (zpos2).*(zpos2)); 
r7 = sqrt(xpos1.*xpos1 + ypos2.*ypos2 + (zpos2).*(zpos2)); 
r8 = sqrt(xpos2.*xpos2 + ypos2.*ypos2 + (zpos2).*(zpos2)); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% g	
res = 0;    

      [t1 t2 t3 t4 t5 t6]=g6term(r1,xpos1,ypos1,zpos1,nx1,ny1,nz1);
      value =(t1+t2+t3+t4+t5+t6);
      res=res+value;
      [t1 t2 t3 t4 t5 t6]=g6term(r2,xpos2,ypos1,zpos1,nx2,ny1,nz1);
      value =(t1+t2+t3+t4+t5+t6);
      res=res-value;
      [t1 t2 t3 t4 t5 t6]=g6term(r3,xpos1,ypos2,zpos1,nx1,ny2,nz1);
      value =(t1+t2+t3+t4+t5+t6);
      res=res-value;
      [t1 t2 t3 t4 t5 t6]=g6term(r4,xpos2,ypos2,zpos1,nx2,ny2,nz1);
      value =(t1+t2+t3+t4+t5+t6);
      res=res+value;
      [t1 t2 t3 t4 t5 t6]=g6term(r5,xpos1,ypos1,zpos2,nx1,ny1,nz2);
      value =(t1+t2+t3+t4+t5+t6);
      res=res-value;
      [t1 t2 t3 t4 t5 t6]=g6term(r6,xpos2,ypos1,zpos2,nx2,ny1,nz2);
      value =(t1+t2+t3+t4+t5+t6);
      res=res+value; 
      [t1 t2 t3 t4 t5 t6]=g6term(r7,xpos1,ypos2,zpos2,nx1,ny2,nz2);
      value =(t1+t2+t3+t4+t5+t6);
      res=res+value;
      [t1 t2 t3 t4 t5 t6]=g6term(r8,xpos2,ypos2,zpos2,nx2,ny2,nz2);
      value =(t1+t2+t3+t4+t5+t6);
      res=res-value;
     
      g= -sum(c.*res)*1e-5;				
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gz	
      
      s1=xpos1.*log((ypos1+r1).*(ypos2+r7)./((ypos2+r3).*(ypos1+r5)));
      s1(nx1)=0;
      s2=xpos2.*log((ypos1+r2).*(ypos2+r8)./((ypos2+r4).*(ypos1+r6)));
      s2(nx2)=0;
      s=s1-s2;
      s1=ypos1.*log((xpos1+r1).*(xpos2+r6)./((xpos2+r2).*(xpos1+r5)));
      s1(ny1)=0;
      s2=ypos2.*log((xpos1+r3).*(xpos2+r8)./((xpos2+r4).*(xpos1+r7)));
      s2(ny2)=0;
      value=s+s1-s2;
      [t3]=gz3term(r1,xpos1,ypos1,zpos1,nz1);
      value=value+t3;
      [t3]=gz3term(r2,xpos2,ypos1,zpos1,nz1);
      value=value-t3;    
      [t3]=gz3term(r3,xpos1,ypos2,zpos1,nz1);
      value=value-t3;
      [t3]=gz3term(r4,xpos2,ypos2,zpos1,nz1);
      value=value+t3;
      [t3]=gz3term(r5,xpos1,ypos1,zpos2,nz2);
      value=value-t3;
      [t3]=gz3term(r6,xpos2,ypos1,zpos2,nz2);
      value=value+t3;
      [t3]=gz3term(r7,xpos1,ypos2,zpos2,nz2);
      value=value+t3;
      [t3]=gz3term(r8,xpos2,ypos2,zpos2,nz2);
      value=value-t3;
      gz=c.*value;     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  gy
      s1=xpos1.*log((zpos1+r1).*(zpos2+r7)./((zpos1+r3).*(zpos2+r5)));
      s1(nx1)=0;
      s2=xpos2.*log((zpos1+r2).*(zpos2+r8)./((zpos1+r4).*(zpos2+r6)));
      s2(nx2)=0;
      s=s1-s2;
      s1=zpos1.*log((xpos1+r1).*(xpos2+r4)./((xpos2+r2).*(xpos1+r3)));
      s1(nz1)=0;
      s2=zpos2.*log((xpos1+r5).*(xpos2+r8)./((xpos2+r6).*(xpos1+r7)));
      s2(nz2)=0;
      value=s+s1-s2;
      [t3]=gz3term(r1,xpos1,zpos1,ypos1,ny1);
      value=value+t3;
      [t3]=gz3term(r2,xpos2,zpos1,ypos1,ny1);
      value=value-t3;    
      [t3]=gz3term(r3,xpos1,zpos1,ypos2,ny2);
      value=value-t3;
      [t3]=gz3term(r4,xpos2,zpos1,ypos2,ny2);
      value=value+t3;
      [t3]=gz3term(r5,xpos1,zpos2,ypos1,ny1);
      value=value-t3;
      [t3]=gz3term(r6,xpos2,zpos2,ypos1,ny1);
      value=value+t3;
      [t3]=gz3term(r7,xpos1,zpos2,ypos2,ny2);
      value=value+t3;
      [t3]=gz3term(r8,xpos2,zpos2,ypos2,ny2);
      value=value-t3;
      gy=-c.*value;     
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  gx
      s1=ypos1.*log((zpos1+r1).*(zpos2+r6)./((zpos1+r2).*(zpos2+r5)));
      s1(ny1)=0;
      s2=ypos2.*log((zpos1+r3).*(zpos2+r8)./((zpos1+r4).*(zpos2+r7)));
      s2(ny2)=0;
      s=s1-s2;
      s1=zpos1.*log((ypos1+r1).*(ypos2+r4)./((ypos1+r2).*(ypos2+r3)));
      s1(nz1)=0;
      s2=zpos2.*log((ypos1+r5).*(ypos2+r8)./((ypos2+r7).*(ypos1+r6)));
      s2(nz2)=0;
      value=s+s1-s2;
      [t3]=gz3term(r1,ypos1,zpos1,xpos1,nx1);
      value=value+t3;
      [t3]=gz3term(r2,ypos1,zpos1,xpos2,nx2);
      value=value-t3;    
      [t3]=gz3term(r3,ypos2,zpos1,xpos1,nx1);
      value=value-t3;
      [t3]=gz3term(r4,ypos2,zpos1,xpos2,nx2);
      value=value+t3;
      [t3]=gz3term(r5,ypos1,zpos2,xpos1,nx1);
      value=value-t3;
      [t3]=gz3term(r6,ypos1,zpos2,xpos2,nx2);
      value=value+t3;
      [t3]=gz3term(r7,ypos2,zpos2,xpos1,nx1);
      value=value+t3;
      [t3]=gz3term(r8,ypos2,zpos2,xpos2,nx2);
      value=value-t3;
      gx=-c.*value;    
                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Vxx     
 
      res = 0;    
      valuexx =atan((zpos1).*(ypos1)./((xpos1).*r1));
      valuexx(nx1)=0;
      res=res+valuexx;
      valuexx =atan((zpos1).*(ypos1)./((xpos2).*r2));     
      valuexx(nx2)=0;
       res=res-valuexx;
      valuexx =atan((zpos1).*(ypos2)./((xpos1).*r3));
      valuexx(nx1)=0;
      res=res-valuexx;
      valuexx =atan((zpos1).*(ypos2)./((xpos2).*r4));
      valuexx(nx2)=0; 
      res=res+valuexx;
      valuexx =atan((zpos2).*(ypos1)./((xpos1).*r5));
      valuexx(nx1)=0;
      res=res-valuexx;
      valuexx =atan((zpos2).*(ypos1)./((xpos2).*r6));
      valuexx(nx2)=0;
      res=res+valuexx;
      valuexx =atan((zpos2).*(ypos2)./((xpos1).*r7));
      valuexx(nx1)=0;
      res=res+valuexx;
      valuexx =atan((zpos2).*(ypos2)./((xpos2).*r8));
      valuexx(nx2)=0;
      res=res-valuexx;
      gxx = (c.*res)*1e4;	
      

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Vyy     
 
      
      res = 0;    
      valuexx =atan((zpos1).*(xpos1)./((ypos1).*r1));
      valuexx(ny1)=0;
      res=res+valuexx;
      valuexx =atan((zpos1).*(xpos2)./((ypos1).*r2));
      valuexx(ny1)=0;
      res=res-valuexx;
      valuexx =atan((zpos1).*(xpos1)./((ypos2).*r3));
      valuexx(ny2)=0;
      res=res-valuexx;
      valuexx =atan((zpos1).*(xpos2)./((ypos2).*r4));
      valuexx(ny2)=0;
      res=res+valuexx;
      valuexx =atan((zpos2).*(xpos1)./((ypos1).*r5));
      valuexx(ny1)=0;
      res=res-valuexx;
      valuexx =atan((zpos2).*(xpos2)./((ypos1).*r6));
      valuexx(ny1)=0;
      res=res+valuexx;
      valuexx =atan((zpos2).*(xpos1)./((ypos2).*r7));
      valuexx(ny2)=0;
      res=res+valuexx;
      valuexx =atan((zpos2).*(xpos2)./((ypos2).*r8));
      valuexx(ny2)=0;
      res=res-valuexx;
      gyy = (c.*res)*1e4;	
	  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Vzz      
 
      res = 0;    
      valuezz =atan((xpos1).*(ypos1)./((zpos1).*r1));
      valuezz(nz1)=0;
      res=res+valuezz;
      valuezz =atan((xpos2).*(ypos1)./((zpos1).*r2));
      valuezz(nz1)=0;
      res=res-valuezz;
      valuezz =atan((xpos1).*(ypos2)./((zpos1).*r3));
      valuezz(nz1)=0;
      res=res-valuezz;
      valuezz =atan((xpos2).*(ypos2)./((zpos1).*r4));
      valuezz(nz1)=0; 
      res=res+valuezz;
      valuezz =atan((xpos1).*(ypos1)./((zpos2).*r5));
      valuezz(nz2)=0;
      res=res-valuezz;
      valuezz =atan((xpos2).*(ypos1)./((zpos2).*r6));
      valuezz(nz2)=0;
      res=res+valuezz;
      valuezz =atan((xpos1).*(ypos2)./((zpos2).*r7));
      valuezz(nz2)=0;
      res=res+valuezz;
      valuezz =atan((xpos2).*(ypos2)./((zpos2).*r8));
      valuezz(nz2)=0;
      res=res-valuezz;
      gzz = (c.*res)*1e4;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Vxy      
   
      s1=(r1+zpos1).*(r4+zpos1).*(r6+zpos2).*(r7+zpos2);
      s2=(r2+zpos1).*(r3+zpos1).*(r5+zpos2).*(r8+zpos2);
      res=log(s1./s2);
      n=find(abs(s2)<=0.001 | abs(s1)<=0.001);
      res(n)=0;
      gxy = -c.*(res)*1e4;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Vxz      
 
      s1=(r1+ypos1).*(r4+ypos2).*(r6+ypos1).*(r7+ypos2);
      s2=(r2+ypos1).*(r3+ypos2).*(r5+ypos1).*(r8+ypos2);
      res=log(s1./s2);
      n=find(abs(s2)<=0.001 | abs(s1)<=0.001);
      res(n)=0; 
      gxz = c.*(res)*1e4;	 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Vyz      
      s1=(r1+xpos1).*(r4+xpos2).*(r6+xpos2).*(r7+xpos1);
      s2=(r2+xpos2).*(r3+xpos1).*(r5+xpos1).*(r8+xpos2);
      res=log(s1./s2);
      n=find(abs(s2)<=0.001 | abs(s1)<=0.001);
      res(n)=0;
      gyz = c.*(res)*1e4;	 
 
% gravitaional field transformation from prism-based system to station-based Cartesian coordinate system 
 [temp_Vx temp_Vy temp_Vz]=g_prism2point(gx,gy,gz,lon,lat,prism_lon,prism_lat);
   Vx=sum(temp_Vx(:));
   Vy=sum(temp_Vy(:));
   Vz=sum(temp_Vz(:));
 
 [temp_Vxx temp_Vxy temp_Vxz temp_Vyy temp_Vyz temp_Vzz]=ggt_prism2point(gxx,gxy,gxz,gyy,gyz,gzz,lon,lat,prism_lon,prism_lat);   
 Vxx=sum(temp_Vxx(:));
 Vxy=sum(temp_Vxy(:));
 Vxz=sum(temp_Vxz(:));
 Vyy=sum(temp_Vyy(:));
 Vyz=sum(temp_Vyz(:));
 Vzz=sum(temp_Vzz(:));
%  Vzz=-(Vxx+Vyy);
end

function [t3]=gz3term( r,xpos1,ypos1,zpos1,nz1)
      t3=-(zpos1).*atan(ypos1.*xpos1./((zpos1).*r));
      t3(nz1)=0;
end

function [t1, t2,t3, t4, t5, t6]=g6term(r,xpos1,ypos1,zpos1,nx1,ny1,nz1)
      t1=xpos1.*ypos1.*log(zpos1+r);
      t1(nx1)=0;
      t1(ny1)=0;
      
      t2=ypos1.*(zpos1).*log(xpos1+r);
      t2(ny1)=0;
      t2(nz1)=0;
      
      t3=xpos1.*(zpos1).*log(ypos1+r);
      t3(nx1)=0;
      t3(nz1)=0;
      
      t4=-xpos1.^2/2.*atan(ypos1.*(zpos1)./(xpos1.*r));
      t4(nx1)=0;
      
      t5=-ypos1.^2/2.*atan(xpos1.*(zpos1)./(ypos1.*r));
      t5(ny1)=0;
      
      t6=-(zpos1).^2/2.*atan(ypos1.*xpos1./((zpos1).*r));
      t6(nz1)=0;
end


function  [g]=Prism_g(x1,x2,y1,y2,d1,d2,lon,lat,r0,e,resolution_x,resolution_y)
G = 6.67384e-3;   
Pi=3.141592654;
R=6371000;
d2r=Pi/180;

% definitation of prism and prism-based Cartesian coordinate system
[prism_x1 prism_x2 prism_y1 prism_y2 prism_z1 prism_z2 prism_lon prism_lat prism_r,e]=tess2prism(x1,x2,y1,y2,d1,d2,e);


c = G*e;  % multiplication of gravitational constant and density 


%coordinates of calculation point in prism-based Cartesian coordinate system
[x y z]=global2local(lon,lat,r0,prism_lon,prism_lat,prism_r);

    
xpos2=(prism_x2-x);
xpos1=(prism_x1-x);
ypos2=(prism_y2-y);
ypos1=(prism_y1-y);
zpos2=(prism_z1-z);
zpos1=(prism_z2-z);

% detecting irregular points
fx1=abs(xpos1)<=0.05;
nx1=find(fx1);
fx2=abs(xpos2)<=0.05;
nx2=find(fx2);
fy2=abs(ypos2)<=0.05;
ny2=find(fy2);
fy1=abs(ypos1)<=0.05;
ny1=find(fy1);
fz2=abs(zpos2)<=0.05;
nz2=find(fz2);
fz1=abs(zpos1)<=0.05;
nz1=find(fz1);    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% g	
   res = 0;    
      r = sqrt(xpos1.*xpos1 + ypos1.*ypos1 + (zpos1).*(zpos1)); 
      [t1 t2 t3 t4 t5 t6]=g6term(r,xpos1,ypos1,zpos1,nx1,ny1,nz1);
      value =(t1+t2+t3+t4+t5+t6);
      res=res+value;
      r = sqrt(xpos2.*xpos2 + ypos1.*ypos1 + (zpos1).*(zpos1)); 
      [t1 t2 t3 t4 t5 t6]=g6term(r,xpos2,ypos1,zpos1,nx2,ny1,nz1);
      value =(t1+t2+t3+t4+t5+t6);
      res=res-value;
      r = sqrt(xpos1.*xpos1 + ypos2.*ypos2 + (zpos1).*(zpos1)); 
      [t1 t2 t3 t4 t5 t6]=g6term(r,xpos1,ypos2,zpos1,nx1,ny2,nz1);
      value =(t1+t2+t3+t4+t5+t6);
      res=res-value;
      r = sqrt(xpos2.*xpos2 + ypos2.*ypos2 + (zpos1).*(zpos1)); 
      [t1 t2 t3 t4 t5 t6]=g6term(r,xpos2,ypos2,zpos1,nx2,ny2,nz1);
      value =(t1+t2+t3+t4+t5+t6);
      res=res+value;
      r = sqrt(xpos1.*xpos1 + ypos1.*ypos1 + (zpos2).*(zpos2)); 
      [t1 t2 t3 t4 t5 t6]=g6term(r,xpos1,ypos1,zpos2,nx1,ny1,nz2);
      value =(t1+t2+t3+t4+t5+t6);
      res=res-value;
      r = sqrt(xpos2.*xpos2 + ypos1.*ypos1 + (zpos2).*(zpos2)); 
      [t1 t2 t3 t4 t5 t6]=g6term(r,xpos2,ypos1,zpos2,nx2,ny1,nz2);
      value =(t1+t2+t3+t4+t5+t6);
      res=res+value;
      r = sqrt(xpos1.*xpos1 + ypos2.*ypos2 + (zpos2).*(zpos2)); 
      [t1 t2 t3 t4 t5 t6]=g6term(r,xpos1,ypos2,zpos2,nx1,ny2,nz2);
      value =(t1+t2+t3+t4+t5+t6);
      res=res+value;
      r = sqrt(xpos2.*xpos2 + ypos2.*ypos2 + (zpos2).*(zpos2)); 
      [t1 t2 t3 t4 t5 t6]=g6term(r,xpos2,ypos2,zpos2,nx2,ny2,nz2);
      value =(t1+t2+t3+t4+t5+t6);
      res=res-value;
     
      g = -sum(c.*res)*1e-5;	


end


function [g, Vx, Vy, Vz]=Prism_g_gx(x1,x2,y1,y2,d1,d2,lon,lat,r0,e,resolution_x,resolution_y)
G = 6.67384e-3;   
Pi=3.141592654;
R=6371000;
d2r=Pi/180;

% definitation of prism and prism-based Cartesian coordinate system
[prism_x1 prism_x2 prism_y1 prism_y2 prism_z1 prism_z2 prism_lon prism_lat prism_r,e]=tess2prism(x1,x2,y1,y2,d1,d2,e);


c = G*e;  % multiplication of gravitational constant and density 


%coordinates of calculation point in prism-based Cartesian coordinate system
[x y z]=global2local(lon,lat,r0,prism_lon,prism_lat,prism_r);

xpos2=(prism_x2-x);
xpos1=(prism_x1-x);
ypos2=(prism_y2-y);
ypos1=(prism_y1-y);
zpos2=(prism_z1-z);
zpos1=(prism_z2-z);

% detecting irregular points
fx1=abs(xpos1)<=0.05;
nx1=find(fx1);
fx2=abs(xpos2)<=0.05;
nx2=find(fx2);
fy2=abs(ypos2)<=0.05;
ny2=find(fy2);
fy1=abs(ypos1)<=0.05;
ny1=find(fy1);
fz2=abs(zpos2)<=0.05;
nz2=find(fz2);
fz1=abs(zpos1)<=0.05;
nz1=find(fz1);    


r1 = sqrt(xpos1.*xpos1 + ypos1.*ypos1 + (zpos1).*(zpos1)); 
r2 = sqrt(xpos2.*xpos2 + ypos1.*ypos1 + (zpos1).*(zpos1)); 
r3 = sqrt(xpos1.*xpos1 + ypos2.*ypos2 + (zpos1).*(zpos1)); 
r4 = sqrt(xpos2.*xpos2 + ypos2.*ypos2 + (zpos1).*(zpos1)); 
r5 = sqrt(xpos1.*xpos1 + ypos1.*ypos1 + (zpos2).*(zpos2)); 
r6 = sqrt(xpos2.*xpos2 + ypos1.*ypos1 + (zpos2).*(zpos2)); 
r7 = sqrt(xpos1.*xpos1 + ypos2.*ypos2 + (zpos2).*(zpos2)); 
r8 = sqrt(xpos2.*xpos2 + ypos2.*ypos2 + (zpos2).*(zpos2)); 
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%g 	
res = 0;    

      [t1 t2 t3 t4 t5 t6]=g6term(r1,xpos1,ypos1,zpos1,nx1,ny1,nz1);
      value =(t1+t2+t3+t4+t5+t6);
      res=res+value;
      [t1 t2 t3 t4 t5 t6]=g6term(r2,xpos2,ypos1,zpos1,nx2,ny1,nz1);
      value =(t1+t2+t3+t4+t5+t6);
      res=res-value;
      [t1 t2 t3 t4 t5 t6]=g6term(r3,xpos1,ypos2,zpos1,nx1,ny2,nz1);
      value =(t1+t2+t3+t4+t5+t6);
      res=res-value;
      [t1 t2 t3 t4 t5 t6]=g6term(r4,xpos2,ypos2,zpos1,nx2,ny2,nz1);
      value =(t1+t2+t3+t4+t5+t6);
      res=res+value;
      [t1 t2 t3 t4 t5 t6]=g6term(r5,xpos1,ypos1,zpos2,nx1,ny1,nz2);
      value =(t1+t2+t3+t4+t5+t6);
      res=res-value;
      [t1 t2 t3 t4 t5 t6]=g6term(r6,xpos2,ypos1,zpos2,nx2,ny1,nz2);
      value =(t1+t2+t3+t4+t5+t6);
      res=res+value; 
      [t1 t2 t3 t4 t5 t6]=g6term(r7,xpos1,ypos2,zpos2,nx1,ny2,nz2);
      value =(t1+t2+t3+t4+t5+t6);
      res=res+value;
      [t1 t2 t3 t4 t5 t6]=g6term(r8,xpos2,ypos2,zpos2,nx2,ny2,nz2);
      value =(t1+t2+t3+t4+t5+t6);
      res=res-value;
     
      g= -sum(c.*res)*1e-5;					
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gz	
      
      s1=xpos1.*log((ypos1+r1).*(ypos2+r7)./((ypos2+r3).*(ypos1+r5)));
      s1(nx1)=0;
      s2=xpos2.*log((ypos1+r2).*(ypos2+r8)./((ypos2+r4).*(ypos1+r6)));
      s2(nx2)=0;
      s=s1-s2;
      s1=ypos1.*log((xpos1+r1).*(xpos2+r6)./((xpos2+r2).*(xpos1+r5)));
      s1(ny1)=0;
      s2=ypos2.*log((xpos1+r3).*(xpos2+r8)./((xpos2+r4).*(xpos1+r7)));
      s2(ny2)=0;
      value=s+s1-s2;
      [t3]=gz3term(r1,xpos1,ypos1,zpos1,nz1);
      value=value+t3;
      [t3]=gz3term(r2,xpos2,ypos1,zpos1,nz1);
      value=value-t3;    
      [t3]=gz3term(r3,xpos1,ypos2,zpos1,nz1);
      value=value-t3;
      [t3]=gz3term(r4,xpos2,ypos2,zpos1,nz1);
      value=value+t3;
      [t3]=gz3term(r5,xpos1,ypos1,zpos2,nz2);
      value=value-t3;
      [t3]=gz3term(r6,xpos2,ypos1,zpos2,nz2);
      value=value+t3;
      [t3]=gz3term(r7,xpos1,ypos2,zpos2,nz2);
      value=value+t3;
      [t3]=gz3term(r8,xpos2,ypos2,zpos2,nz2);
      value=value-t3;
      gz=c.*value;     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  gy
      s1=xpos1.*log((zpos1+r1).*(zpos2+r7)./((zpos1+r3).*(zpos2+r5)));
      s1(nx1)=0;
      s2=xpos2.*log((zpos1+r2).*(zpos2+r8)./((zpos1+r4).*(zpos2+r6)));
      s2(nx2)=0;
      s=s1-s2;
      s1=zpos1.*log((xpos1+r1).*(xpos2+r4)./((xpos2+r2).*(xpos1+r3)));
      s1(nz1)=0;
      s2=zpos2.*log((xpos1+r5).*(xpos2+r8)./((xpos2+r6).*(xpos1+r7)));
      s2(nz2)=0;
      value=s+s1-s2;
      [t3]=gz3term(r1,xpos1,zpos1,ypos1,ny1);
      value=value+t3;
      [t3]=gz3term(r2,xpos2,zpos1,ypos1,ny1);
      value=value-t3;    
      [t3]=gz3term(r3,xpos1,zpos1,ypos2,ny2);
      value=value-t3;
      [t3]=gz3term(r4,xpos2,zpos1,ypos2,ny2);
      value=value+t3;
      [t3]=gz3term(r5,xpos1,zpos2,ypos1,ny1);
      value=value-t3;
      [t3]=gz3term(r6,xpos2,zpos2,ypos1,ny1);
      value=value+t3;
      [t3]=gz3term(r7,xpos1,zpos2,ypos2,ny2);
      value=value+t3;
      [t3]=gz3term(r8,xpos2,zpos2,ypos2,ny2);
      value=value-t3;
      gy=-c.*value;     
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  gx
      s1=ypos1.*log((zpos1+r1).*(zpos2+r6)./((zpos1+r2).*(zpos2+r5)));
      s1(ny1)=0;
      s2=ypos2.*log((zpos1+r3).*(zpos2+r8)./((zpos1+r4).*(zpos2+r7)));
      s2(ny2)=0;
      s=s1-s2;
      s1=zpos1.*log((ypos1+r1).*(ypos2+r4)./((ypos1+r2).*(ypos2+r3)));
      s1(nz1)=0;
      s2=zpos2.*log((ypos1+r5).*(ypos2+r8)./((ypos2+r7).*(ypos1+r6)));
      s2(nz2)=0;
      value=s+s1-s2;
      [t3]=gz3term(r1,ypos1,zpos1,xpos1,nx1);
      value=value+t3;
      [t3]=gz3term(r2,ypos1,zpos1,xpos2,nx2);
      value=value-t3;    
      [t3]=gz3term(r3,ypos2,zpos1,xpos1,nx1);
      value=value-t3;
      [t3]=gz3term(r4,ypos2,zpos1,xpos2,nx2);
      value=value+t3;
      [t3]=gz3term(r5,ypos1,zpos2,xpos1,nx1);
      value=value-t3;
      [t3]=gz3term(r6,ypos1,zpos2,xpos2,nx2);
      value=value+t3;
      [t3]=gz3term(r7,ypos2,zpos2,xpos1,nx1);
      value=value+t3;
      [t3]=gz3term(r8,ypos2,zpos2,xpos2,nx2);
      value=value-t3;
      gx=-c.*value;    
 
%gravitaional field transformation from prism-based system to station-based Cartesian coordinate system 
 [temp_Vx temp_Vy temp_Vz]=g_prism2point(gx,gy,gz,lon,lat,prism_lon,prism_lat);
   Vx=sum(temp_Vx(:));
   Vy=sum(temp_Vy(:));
   Vz=sum(temp_Vz(:));
end

function [Vx, Vy, Vz]=Prism_gx(x1,x2,y1,y2,d1,d2,lon,lat,r0,e,resolution_x,resolution_y)
G = 6.67384e-3;
Pi=3.141592654;
% R=6371000;
% d2r=Pi/180;

% definitation of prism and prism-based Cartesian coordinate system
[prism_x1 prism_x2 prism_y1 prism_y2 prism_z1 prism_z2 prism_lon prism_lat prism_r,e]=tess2prism(x1,x2,y1,y2,d1,d2,e);

c = G*e;  % multiplication of gravitational constant and density 
%coordinates of calculation point in prism-based Cartesian coordinate system
[x y z]=global2local(lon,lat,r0,prism_lon,prism_lat,prism_r);


xpos2=(prism_x2-x);
xpos1=(prism_x1-x);
ypos2=(prism_y2-y);
ypos1=(prism_y1-y);
zpos2=(prism_z1-z);
zpos1=(prism_z2-z);

% detecting irregular points
fx1=abs(xpos1)<=0.05;
nx1=find(fx1);
fx2=abs(xpos2)<=0.05;
nx2=find(fx2);
fy2=abs(ypos2)<=0.05;
ny2=find(fy2);
fy1=abs(ypos1)<=0.05;
ny1=find(fy1);
fz2=abs(zpos2)<=0.05;
nz2=find(fz2);
fz1=abs(zpos1)<=0.05;
nz1=find(fz1);    


r1 = sqrt(xpos1.*xpos1 + ypos1.*ypos1 + (zpos1).*(zpos1)); 
r2 = sqrt(xpos2.*xpos2 + ypos1.*ypos1 + (zpos1).*(zpos1)); 
r3 = sqrt(xpos1.*xpos1 + ypos2.*ypos2 + (zpos1).*(zpos1)); 
r4 = sqrt(xpos2.*xpos2 + ypos2.*ypos2 + (zpos1).*(zpos1)); 
r5 = sqrt(xpos1.*xpos1 + ypos1.*ypos1 + (zpos2).*(zpos2)); 
r6 = sqrt(xpos2.*xpos2 + ypos1.*ypos1 + (zpos2).*(zpos2)); 
r7 = sqrt(xpos1.*xpos1 + ypos2.*ypos2 + (zpos2).*(zpos2)); 
r8 = sqrt(xpos2.*xpos2 + ypos2.*ypos2 + (zpos2).*(zpos2));
%clear xpos12 xpos22 ypos12 ypos22 zpos12 zpos22
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% gz	
      
      s1=xpos1.*log((ypos1+r1).*(ypos2+r7)./((ypos2+r3).*(ypos1+r5)));
      s1(nx1)=0;
      s2=xpos2.*log((ypos1+r2).*(ypos2+r8)./((ypos2+r4).*(ypos1+r6)));
      s2(nx2)=0;
      s=s1-s2;
      s1=ypos1.*log((xpos1+r1).*(xpos2+r6)./((xpos2+r2).*(xpos1+r5)));
      s1(ny1)=0;
      s2=ypos2.*log((xpos1+r3).*(xpos2+r8)./((xpos2+r4).*(xpos1+r7)));
      s2(ny2)=0;
      value=s+s1-s2;
      [t3]=gz3term(r1,xpos1,ypos1,zpos1,nz1);
      value=value+t3;
      [t3]=gz3term(r2,xpos2,ypos1,zpos1,nz1);
      value=value-t3;    
      [t3]=gz3term(r3,xpos1,ypos2,zpos1,nz1);
      value=value-t3;
      [t3]=gz3term(r4,xpos2,ypos2,zpos1,nz1);
      value=value+t3;
      [t3]=gz3term(r5,xpos1,ypos1,zpos2,nz2);
      value=value-t3;
      [t3]=gz3term(r6,xpos2,ypos1,zpos2,nz2);
      value=value+t3;
      [t3]=gz3term(r7,xpos1,ypos2,zpos2,nz2);
      value=value+t3;
      [t3]=gz3term(r8,xpos2,ypos2,zpos2,nz2);
      value=value-t3;
      gz=c.*value;     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  gy
      s1=xpos1.*log((zpos1+r1).*(zpos2+r7)./((zpos1+r3).*(zpos2+r5)));
      s1(nx1)=0;
      s2=xpos2.*log((zpos1+r2).*(zpos2+r8)./((zpos1+r4).*(zpos2+r6)));
      s2(nx2)=0;
      s=s1-s2;
      s1=zpos1.*log((xpos1+r1).*(xpos2+r4)./((xpos2+r2).*(xpos1+r3)));
      s1(nz1)=0;
      s2=zpos2.*log((xpos1+r5).*(xpos2+r8)./((xpos2+r6).*(xpos1+r7)));
      s2(nz2)=0;
      value=s+s1-s2;
      [t3]=gz3term(r1,xpos1,zpos1,ypos1,ny1);
      value=value+t3;
      [t3]=gz3term(r2,xpos2,zpos1,ypos1,ny1);
      value=value-t3;    
      [t3]=gz3term(r3,xpos1,zpos1,ypos2,ny2);
      value=value-t3;
      [t3]=gz3term(r4,xpos2,zpos1,ypos2,ny2);
      value=value+t3;
      [t3]=gz3term(r5,xpos1,zpos2,ypos1,ny1);
      value=value-t3;
      [t3]=gz3term(r6,xpos2,zpos2,ypos1,ny1);
      value=value+t3;
      [t3]=gz3term(r7,xpos1,zpos2,ypos2,ny2);
      value=value+t3;
      [t3]=gz3term(r8,xpos2,zpos2,ypos2,ny2);
      value=value-t3;
      gy=-c.*value;     
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  gx
      s1=ypos1.*log((zpos1+r1).*(zpos2+r6)./((zpos1+r2).*(zpos2+r5)));
      s1(ny1)=0;
      s2=ypos2.*log((zpos1+r3).*(zpos2+r8)./((zpos1+r4).*(zpos2+r7)));
      s2(ny2)=0;
      s=s1-s2;
      s1=zpos1.*log((ypos1+r1).*(ypos2+r4)./((ypos1+r2).*(ypos2+r3)));
      s1(nz1)=0;
      s2=zpos2.*log((ypos1+r5).*(ypos2+r8)./((ypos2+r7).*(ypos1+r6)));
      s2(nz2)=0;
      value=s+s1-s2;
      [t3]=gz3term(r1,ypos1,zpos1,xpos1,nx1);
      value=value+t3;
      [t3]=gz3term(r2,ypos1,zpos1,xpos2,nx2);
      value=value-t3;    
      [t3]=gz3term(r3,ypos2,zpos1,xpos1,nx1);
      value=value-t3;
      [t3]=gz3term(r4,ypos2,zpos1,xpos2,nx2);
      value=value+t3;
      [t3]=gz3term(r5,ypos1,zpos2,xpos1,nx1);
      value=value-t3;
      [t3]=gz3term(r6,ypos1,zpos2,xpos2,nx2);
      value=value+t3;
      [t3]=gz3term(r7,ypos2,zpos2,xpos1,nx1);
      value=value+t3;
      [t3]=gz3term(r8,ypos2,zpos2,xpos2,nx2);
      value=value-t3;
      gx=-c.*value;    
% gravitaional field transformation from prism-based system to station-based Cartesian coordinate system                     
 [temp_Vx temp_Vy temp_Vz]=g_prism2point(gx,gy,gz,lon,lat,prism_lon,prism_lat);
   Vx=sum(temp_Vx(:));
   Vy=sum(temp_Vy(:));
   Vz=sum(temp_Vz(:)); 
end

function [Vxx, Vxy, Vxz, Vyy, Vyz, Vzz]=Prism_gxx(x1,x2,y1,y2,d1,d2,lon,lat,r0,e,resolution_x,resolution_y)
G = 6.67384e-3;  
Pi=3.141592654;
R=6371000;
d2r=Pi/180;
%definitation of prism and prism-based Cartesian coordinate system
[prism_x1 prism_x2 prism_y1 prism_y2 prism_z1 prism_z2 prism_lon prism_lat prism_r,e]=tess2prism(x1,x2,y1,y2,d1,d2,e);

c = G*e;  % multiplication of gravitational constant and density 

%coordinates of calculation point in prism-based Cartesian coordinate system
[x y z]=global2local(lon,lat,r0,prism_lon,prism_lat,prism_r);

    
xpos2=(prism_x2-x);
xpos1=(prism_x1-x);
ypos2=(prism_y2-y);
ypos1=(prism_y1-y);
zpos2=(prism_z1-z);
zpos1=(prism_z2-z);

%detecting irregular points
fx1=abs(xpos1)<=0.05;
nx1=find(fx1);
fx2=abs(xpos2)<=0.05;
nx2=find(fx2);
fy2=abs(ypos2)<=0.05;
ny2=find(fy2);
fy1=abs(ypos1)<=0.05;
ny1=find(fy1);
fz2=abs(zpos2)<=0.05;
nz2=find(fz2);
fz1=abs(zpos1)<=0.05;
nz1=find(fz1);    


r1 = sqrt(xpos1.*xpos1 + ypos1.*ypos1 + (zpos1).*(zpos1)); 
r2 = sqrt(xpos2.*xpos2 + ypos1.*ypos1 + (zpos1).*(zpos1)); 
r3 = sqrt(xpos1.*xpos1 + ypos2.*ypos2 + (zpos1).*(zpos1)); 
r4 = sqrt(xpos2.*xpos2 + ypos2.*ypos2 + (zpos1).*(zpos1)); 
r5 = sqrt(xpos1.*xpos1 + ypos1.*ypos1 + (zpos2).*(zpos2)); 
r6 = sqrt(xpos2.*xpos2 + ypos1.*ypos1 + (zpos2).*(zpos2)); 
r7 = sqrt(xpos1.*xpos1 + ypos2.*ypos2 + (zpos2).*(zpos2)); 
r8 = sqrt(xpos2.*xpos2 + ypos2.*ypos2 + (zpos2).*(zpos2));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Vxx     
 
      res = 0;    
      valuexx =atan((zpos1).*(ypos1)./((xpos1).*r1));
      valuexx(nx1)=0;
      res=res+valuexx;
      valuexx =atan((zpos1).*(ypos1)./((xpos2).*r2));     
      valuexx(nx2)=0;
       res=res-valuexx;
      valuexx =atan((zpos1).*(ypos2)./((xpos1).*r3));
      valuexx(nx1)=0;
      res=res-valuexx;
      valuexx =atan((zpos1).*(ypos2)./((xpos2).*r4));
      valuexx(nx2)=0; 
      res=res+valuexx;
      valuexx =atan((zpos2).*(ypos1)./((xpos1).*r5));
      valuexx(nx1)=0;
      res=res-valuexx;
      valuexx =atan((zpos2).*(ypos1)./((xpos2).*r6));
      valuexx(nx2)=0;
      res=res+valuexx;
      valuexx =atan((zpos2).*(ypos2)./((xpos1).*r7));
      valuexx(nx1)=0;
      res=res+valuexx;
      valuexx =atan((zpos2).*(ypos2)./((xpos2).*r8));
      valuexx(nx2)=0;
      res=res-valuexx;
      gxx = (c.*res)*10^4;	
      

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Vyy     
 
      
      res = 0;    
      valuexx =atan((zpos1).*(xpos1)./((ypos1).*r1));
      valuexx(ny1)=0;
      res=res+valuexx;
      valuexx =atan((zpos1).*(xpos2)./((ypos1).*r2));
      valuexx(ny1)=0;
      res=res-valuexx;
      valuexx =atan((zpos1).*(xpos1)./((ypos2).*r3));
      valuexx(ny2)=0;
      res=res-valuexx;
      valuexx =atan((zpos1).*(xpos2)./((ypos2).*r4));
      valuexx(ny2)=0;
      res=res+valuexx;
      valuexx =atan((zpos2).*(xpos1)./((ypos1).*r5));
      valuexx(ny1)=0;
      res=res-valuexx;
      valuexx =atan((zpos2).*(xpos2)./((ypos1).*r6));
      valuexx(ny1)=0;
      res=res+valuexx;
      valuexx =atan((zpos2).*(xpos1)./((ypos2).*r7));
      valuexx(ny2)=0;
      res=res+valuexx;
      valuexx =atan((zpos2).*(xpos2)./((ypos2).*r8));
      valuexx(ny2)=0;
      res=res-valuexx;
      gyy = (c.*res)*10^4;		
	  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Vzz      
 
      res = 0;    
      valuezz =atan((xpos1).*(ypos1)./((zpos1).*r1));
      valuezz(nz1)=0;
      res=res+valuezz;
      valuezz =atan((xpos2).*(ypos1)./((zpos1).*r2));
      valuezz(nz1)=0;
      res=res-valuezz;
      valuezz =atan((xpos1).*(ypos2)./((zpos1).*r3));
      valuezz(nz1)=0;
      res=res-valuezz;
      valuezz =atan((xpos2).*(ypos2)./((zpos1).*r4));
      valuezz(nz1)=0; 
      res=res+valuezz;
      valuezz =atan((xpos1).*(ypos1)./((zpos2).*r5));
      valuezz(nz2)=0;
      res=res-valuezz;
      valuezz =atan((xpos2).*(ypos1)./((zpos2).*r6));
      valuezz(nz2)=0;
      res=res+valuezz;
      valuezz =atan((xpos1).*(ypos2)./((zpos2).*r7));
      valuezz(nz2)=0;
      res=res+valuezz;
      valuezz =atan((xpos2).*(ypos2)./((zpos2).*r8));
      valuezz(nz2)=0;
      res=res-valuezz;
%        gzz = (-gxx-gyy);
       gzz=(c.*res)*10^4;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Vxy      
   
      s1=(r1+zpos1).*(r4+zpos1).*(r6+zpos2).*(r7+zpos2);     
      s2=(r2+zpos1).*(r3+zpos1).*(r5+zpos2).*(r8+zpos2);
      res=log(s1./s2);
      n=find(abs(s2)<=0.001| abs(s1)<=0.001);
      res(n)=0;
      gxy = -c.*(res)*10^4;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Vxz      
 
      s1=(r1+ypos1).*(r4+ypos2).*(r6+ypos1).*(r7+ypos2);
      s2=(r2+ypos1).*(r3+ypos2).*(r5+ypos1).*(r8+ypos2);
      res=log(s1./s2);
      n=find(abs(s2)<=0.001 | abs(s1)<=0.001);
      res(n)=0; 
      gxz = c.*(res)*10^4;	 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Vyz      
      s1=(r1+xpos1).*(r4+xpos2).*(r6+xpos2).*(r7+xpos1);
      s2=(r2+xpos2).*(r3+xpos1).*(r5+xpos1).*(r8+xpos2);
      res=log(s1./s2);
      n=find(abs(s2)<=0.001 | abs(s1)<=0.001);
      res(n)=0;
      gyz = c.*(res)*10^4;	 
%gravitaional field transformation from prism-based system to station-based Cartesian coordinate system 	  
 [temp_Vxx temp_Vxy temp_Vxz temp_Vyy temp_Vyz temp_Vzz]=ggt_prism2point(gxx,gxy,gxz,gyy,gyz,gzz,lon,lat,prism_lon,prism_lat);   
 Vxx=sum(temp_Vxx(:));
 Vxy=sum(temp_Vxy(:));
 Vxz=sum(temp_Vxz(:));
 Vyy=sum(temp_Vyy(:));
 Vyz=sum(temp_Vyz(:));
 Vzz=sum(temp_Vzz(:));
%  Vzz=-(Vxx+Vyy); 
end

function [prism_x1, prism_x2, prism_y1, prism_y2, prism_z1, prism_z2, prism_lon, prism_lat, prism_r, prism_e]=tess2prism(x1,x2,y1,y2,d1,d2,e)
% transform a tesseroid to a mass-equavilent prism
Pi=3.141592654;
d2r=Pi/180;

r1=d1;
r2=d2;
r0 = 0.5*(r1 + r2);

dx=r0*d2r.*(y2-y1);
dy=r0.*cos(d2r*0.5*(y1+y2))*d2r.*(x2-x1);
prism_x1=-0.5*dx;
prism_x2=-prism_x1;
prism_y1=-0.5*dy;
prism_y2=-prism_y1;
prism_z1=r1;
prism_z1(1:end)=(r2-r1)/2;
prism_z2=(r1-r2)/2;

tess_volume=d2r.*(x2-x1).*(r1.^3-r2.^3).*(sin(d2r*y2)-sin(d2r*y1))/3;
prism_volume=(prism_x2-prism_x1).*(prism_y2-prism_y1).*(prism_z2-prism_z1);

prism_e=e.*tess_volume./prism_volume;
prism_lon=0.5*(x1+x2);
prism_lat=0.5*(y1+y2);
prism_r=r0;
end

function [V, Vx, Vy, Vz, Vxx, Vxy, Vxz, Vyy, Vyz, Vzz]=Tesseroid(lon1,lat1,h1,ref,lon,lat,r0,resolution_x1,resolution_y1,e)
Pi=3.141592654;
d2r=Pi/180;
G = 6.67384e-3;   % Universal Gravitational constant


%% build tesseroid
  zz1=lon1-resolution_x1/2;
  zz2=lon1+resolution_x1/2;
  yy1=lat1-resolution_y1/2;
  yy2=lat1+resolution_y1/2;
  xx1=h1;
  xx2=ref;     

  r1=(xx1+xx2)/2;
 %% variable definition 
 lat=lat*d2r;
 lat1=lat1*d2r;
 coslat=cos(lat);
 coslat1=cos(lat1);
 sinlat=sin(lat);
 sinlat1=sin(lat1);
d_lon=(lon1-lon)*d2r;
cosdlon=cos(d_lon);
sindlon=sin(d_lon);
cosphicosa=coslat1.*cosdlon;
coslatsinlat1=coslat.*sinlat1;
sinphicosa=coslatsinlat1-sinlat.*cosphicosa;
sinphisina=coslat1.*sindlon;
sinlatsinlat1=sinlat.*sinlat1;
cosphi=sinlatsinlat1+coslat.*cosphicosa;
dx=r1.*sinphicosa;
dy=r1.*sinphisina;
dz=r1.*cosphi-r0;
dxx=dx.*dx;
dxy=dx.*dy;
dxz=dx.*dz;
dyy=dy.*dy;
dyz=dy.*dz;
dzz=dz.*dz;
l=sqrt(dxx+dyy+dzz);
xx=xx2-xx1;
yy=(yy2-yy1)*d2r;
zz=(zz2-zz1)*d2r;
c=G*e.*zz.*xx.*yy;
l2=l.*l;
%% Toi
r12=r1.*r1;
toi=r12.*coslat1;

toi1=2*r1.*coslat1;
toi2=-r12.*sinlat1;
toi3=0;

toi12=2*coslat1;
toi22=-toi;
toi32=0;

%% dxi
x1_1=sinphicosa;
x2_1=sinphisina;
x3_1=cosphi;

x1_12=0;
x2_12=0;
x3_12=0;

x1_2=r1.*(coslat.*coslat1+sinlatsinlat1.*cosdlon);
x2_2=-r1.*sinlat1.*sindlon;
x3_2=r1.*(sinlat.*coslat1-coslatsinlat1.*cosdlon);

x1_22=-dx;
x2_22=-dy;
x3_22=-r1.*cosphi;


x1_3=(dy).*sinlat;
x2_3=r1.*cosphicosa;
x3_3=-(dy).*coslat;

x1_32=x2_3.*sinlat;
x2_32=-(dy);
x3_32=-x2_3.*coslat;

%% ml

ml1=r0.*cosphi-r1;
ml2=r0.*x3_2;
ml3=r0.*x3_3;

ml12=-1;
ml22=r0.*x3_22;
ml32=r0.*x3_32;


%% a
a=l2;

a1=dx;
a2=dy;
a3=dz;

l23=3./l2;
a11=l23.*dxx-1;
a12=l23.*dxy;
a13=l23.*dxz;
a22=l23.*dyy-1;
a23=l23.*dyz;
a33=l23.*dzz-1;

%% b
b1=2*ml1;
b2=2*ml2;
b3=2*ml3;

b11=2*(x1_1+l23.*dx.*ml1);
b12=2*(x1_2+l23.*dx.*ml2);
b21=2*(x2_1+l23.*dy.*ml1);
b13=2*(x1_3+l23.*dx.*ml3);
b31=2*(x3_1+l23.*dz.*ml1);
b22=2*(x2_2+l23.*dy.*ml2);
b23=2*(x2_3+l23.*dy.*ml3);
b32=2*(x3_2+l23.*dz.*ml2);
b33=2*(x3_3+l23.*dz.*ml3);

const_1=6./l2;
dxx5=5*dxx;
dxy5=5*dxy;
dxz5=5*dxz;
dyy5=5*dyy;
dyz5=5*dyz;
dzz5=5*dzz;
dx_x11=dx.*x1_1;
dx_x12=dx.*x1_2;
dx_x13=dx.*x1_3;
dx_x21=dx.*x2_1;
dx_x22=dx.*x2_2;
dx_x23=dx.*x2_3;
dy_x11=dy.*x1_1;
dy_x12=dy.*x1_2;
dy_x13=dy.*x1_3;
dz_x11=dz.*x1_1;
dz_x12=dz.*x1_2;
dz_x13=dz.*x1_3;
dy_x21=dy.*x2_1;
dy_x22=dy.*x2_2;
dy_x23=dy.*x2_3;
dz_x21=dz.*x2_1;
dz_x22=dz.*x2_2;
dz_x23=dz.*x2_3;

dx_x31=dx.*x3_1;
dx_x32=dx.*x3_2;
dx_x33=dx.*x3_3;
dy_x31=dy.*x3_1;
dy_x32=dy.*x3_2;
dy_x33=dy.*x3_3;
dz_x31=dz.*x3_1;
dz_x32=dz.*x3_2;
dz_x33=dz.*x3_3;

b111=const_1.*(dx_x11+dx_x11+dxx5.*ml1./l2-ml1);
b112=const_1.*(dx_x12+dx_x12+dxx5.*ml2./l2-ml2);
b113=const_1.*(dx_x13+dx_x13+dxx5.*ml3./l2-ml3);
b121=const_1.*(dx_x21+dy_x11+dxy5.*ml1./l2);
b122=const_1.*(dx_x22+dy_x12+dxy5.*ml2./l2);
b123=const_1.*(dx_x23+dy_x13+dxy5.*ml3./l2);
b131=const_1.*(dx_x31+dz_x11+dxz5.*ml1./l2);
b132=const_1.*(dx_x32+dz_x12+dxz5.*ml2./l2);
b133=const_1.*(dx_x33+dz_x13+dxz5.*ml3./l2);
% b211=const_1.*(dy_x11+dx_x21+dxy5.*ml1./l2);
% b212=const_1.*(dy_x12+dx_x22+dxy5.*ml2./l2);
% b213=const_1.*(dy_x13+dx_x23+dxy5.*ml3./l2);
b221=const_1.*(dy_x21+dy_x21+dyy5.*ml1./l2-ml1);
b222=const_1.*(dy_x22+dy_x22+dyy5.*ml2./l2-ml2);
b223=const_1.*(dy_x23+dy_x23+dyy5.*ml3./l2-ml3);
b231=const_1.*(dy_x31+dz_x21+dyz5.*ml1./l2);
b232=const_1.*(dy_x32+dz_x22+dyz5.*ml2./l2);
b233=const_1.*(dy_x33+dz_x23+dyz5.*ml3./l2);
% b311=const_1.*(dz_x11+dx_x31+dxz5.*ml1./l2);
% b312=const_1.*(dz_x12+dx_x32+dxz5.*ml2./l2);
% b313=const_1.*(dz_x13+dx_x33+dxz5.*ml3./l2);
% b321=const_1.*(dz_x21+dy_x31+dyz5.*ml1./l2);
% b322=const_1.*(dz_x22+dy_x32+dyz5.*ml2./l2);
% b323=const_1.*(dz_x23+dy_x33+dyz5.*ml3./l2);
b331=const_1.*(dz_x31+dz_x31+dzz5.*ml1./l2-ml1);
b332=const_1.*(dz_x32+dz_x32+dzz5.*ml2./l2-ml2);
b333=const_1.*(dz_x33+dz_x33+dzz5.*ml3./l2-ml3);

%% gama
ml1_2=ml1.*ml1;
ml2_2=ml2.*ml2;
ml3_2=ml3.*ml3;
gama1=ml12+l23.*ml1_2;
gama2=ml22+l23.*ml2_2;
gama3=ml32+l23.*ml3_2;


l25=5./l2;
gama11=x1_12+l23.*(2*x1_1.*ml1+dx.*(ml12+l25.*ml1_2));
gama12=x1_22+l23.*(2*x1_2.*ml2+dx.*(ml22+l25.*ml2_2));
gama21=x2_12+l23.*(2*x2_1.*ml1+dy.*(ml12+l25.*ml1_2));
gama13=x1_32+l23.*(2*x1_3.*ml3+dx.*(ml32+l25.*ml3_2));
gama31=x3_12+l23.*(2*x3_1.*ml1+dz.*(ml12+l25.*ml1_2));
gama22=x2_22+l23.*(2*x2_2.*ml2+dy.*(ml22+l25.*ml2_2));
gama23=x2_32+l23.*(2*x2_3.*ml3+dy.*(ml32+l25.*ml3_2));
gama32=x3_22+l23.*(2*x3_2.*ml2+dz.*(ml22+l25.*ml2_2));
gama33=x3_32+l23.*(2*x3_3.*ml3+dz.*(ml32+l25.*ml3_2));


l27=7./l2;

dx_x112=dx.*x1_12;
dx_x122=dx.*x1_22;
dx_x132=dx.*x1_32;
dx_x212=dx.*x2_12;
dx_x222=dx.*x2_22;
dx_x232=dx.*x2_32;
dx_x312=dx.*x3_12;
dx_x322=dx.*x3_22;
dx_x332=dx.*x3_32;

dy_x112=dy.*x1_12;
dy_x122=dy.*x1_22;
dy_x132=dy.*x1_32;
dy_x212=dy.*x2_12;
dy_x222=dy.*x2_22;
dy_x232=dy.*x2_32;
dy_x312=dy.*x3_12;
dy_x322=dy.*x3_22;
dy_x332=dy.*x3_32;

dz_x112=dz.*x1_12;
dz_x122=dz.*x1_22;
dz_x132=dz.*x1_32;
dz_x212=dz.*x2_12;
dz_x222=dz.*x2_22;
dz_x232=dz.*x2_32;
dz_x312=dz.*x3_12;
dz_x322=dz.*x3_22;
dz_x332=dz.*x3_32;


gama111=l23.*(dx_x112+dx_x112+2*x1_1.*x1_1+l25.*(b1.*(dx_x11+dx_x11)+dxx.*(ml12+l27.*(ml1_2)))-(ml12+l25.*(ml1_2)));
gama112=l23.*(dx_x122+dx_x122+2*x1_2.*x1_2+l25.*(b2.*(dx_x12+dx_x12)+dxx.*(ml22+l27.*(ml2_2)))-(ml22+l25.*(ml2_2)));
gama113=l23.*(dx_x132+dx_x132+2*x1_3.*x1_3+l25.*(b3.*(dx_x13+dx_x13)+dxx.*(ml32+l27.*(ml3_2)))-(ml32+l25.*(ml3_2)));
gama121=l23.*(dx_x212+dy_x112+2*x1_1.*x2_1+l25.*(b1.*(dx_x21+dy_x11)+dxy.*(ml12+l27.*(ml1_2))));
gama122=l23.*(dx_x222+dy_x122+2*x1_2.*x2_2+l25.*(b2.*(dx_x22+dy_x12)+dxy.*(ml22+l27.*(ml2_2))));
gama123=l23.*(dx_x232+dy_x132+2*x1_3.*x2_3+l25.*(b3.*(dx_x23+dy_x13)+dxy.*(ml32+l27.*(ml3_2))));
gama131=l23.*(dx_x312+dz_x112+2*x1_1.*x3_1+l25.*(b1.*(dx_x31+dz_x11)+dxz.*(ml12+l27.*(ml1_2))));
gama132=l23.*(dx_x322+dz_x122+2*x1_2.*x3_2+l25.*(b2.*(dx_x32+dz_x12)+dxz.*(ml22+l27.*(ml2_2))));
gama133=l23.*(dx_x332+dz_x132+2*x1_3.*x3_3+l25.*(b3.*(dx_x33+dz_x13)+dxz.*(ml32+l27.*(ml3_2))));
% gama211=l23.*(dy_x112+dx_x212+2*x2_1.*x1_1+l25.*(b1.*(dy_x11+dx_x21)+dxy.*(ml12+l27.*(ml1_2))));
% gama212=l23.*(dy_x122+dx_x222+2*x2_2.*x1_2+l25.*(b2.*(dy_x12+dx_x22)+dxy.*(ml22+l27.*(ml2_2))));
% gama213=l23.*(dy_x132+dx_x232+2*x2_3.*x1_3+l25.*(b3.*(dy_x13+dx_x23)+dxy.*(ml32+l27.*(ml3_2))));
gama221=l23.*(dy_x212+dy_x212+2*x2_1.*x2_1+l25.*(b1.*(dy_x21+dy_x21)+dyy.*(ml12+l27.*(ml1_2)))-(ml12+l25.*(ml1_2)));
gama222=l23.*(dy_x222+dy_x222+2*x2_2.*x2_2+l25.*(b2.*(dy_x22+dy_x22)+dyy.*(ml22+l27.*(ml2_2)))-(ml22+l25.*(ml2_2)));
gama223=l23.*(dy_x232+dy_x232+2*x2_3.*x2_3+l25.*(b3.*(dy_x23+dy_x23)+dyy.*(ml32+l27.*(ml3_2)))-(ml32+l25.*(ml3_2)));
gama231=l23.*(dy_x312+dz_x212+2*x2_1.*x3_1+l25.*(b1.*(dy_x31+dz_x21)+dyz.*(ml12+l27.*(ml1_2))));
gama232=l23.*(dy_x322+dz_x222+2*x2_2.*x3_2+l25.*(b2.*(dy_x32+dz_x22)+dyz.*(ml22+l27.*(ml2_2))));
gama233=l23.*(dy_x332+dz_x232+2*x2_3.*x3_3+l25.*(b3.*(dy_x33+dz_x23)+dyz.*(ml32+l27.*(ml3_2))));
% gama311=l23.*(dz_x112+dx_x312+2*x3_1.*x1_1+l25.*(b1.*(dz_x11+dx_x31)+dxz.*(ml12+l27.*(ml1_2))));
% gama312=l23.*(dz_x122+dx_x322+2*x3_2.*x1_2+l25.*(b2.*(dz_x12+dx_x32)+dxz.*(ml22+l27.*(ml2_2))));
% gama313=l23.*(dz_x132+dx_x332+2*x3_3.*x1_3+l25.*(b3.*(dz_x13+dx_x33)+dxz.*(ml32+l27.*(ml3_2))));
% gama321=l23.*(dz_x212+dy_x312+2*x3_1.*x2_1+l25.*(b1.*(dz_x21+dy_x31)+dyz.*(ml12+l27.*(ml1_2))));
% gama322=l23.*(dz_x222+dy_x322+2*x3_2.*x2_2+l25.*(b2.*(dz_x22+dy_x32)+dyz.*(ml22+l27.*(ml2_2))));
% gama323=l23.*(dz_x232+dy_x332+2*x3_3.*x2_3+l25.*(b3.*(dz_x23+dy_x33)+dyz.*(ml32+l27.*(ml3_2))));
gama331=l23.*(dz_x312+dz_x312+2*x3_1.*x3_1+l25.*(b1.*(dz_x31+dz_x31)+dzz.*(ml12+l27.*(ml1_2)))-(ml12+l25.*(ml1_2)));
gama332=l23.*(dz_x322+dz_x322+2*x3_2.*x3_2+l25.*(b2.*(dz_x32+dz_x32)+dzz.*(ml22+l27.*(ml2_2)))-(ml22+l25.*(ml2_2)));
gama333=l23.*(dz_x332+dz_x332+2*x3_3.*x3_3+l25.*(b3.*(dz_x33+dz_x33)+dzz.*(ml32+l27.*(ml3_2)))-(ml32+l25.*(ml3_2)));

%%   gravitational field
xx_2=(xx).*xx;
yy_2=(yy).*yy;
zz_2=(zz).*zz;
K000=1./l.*toi;
l3=1./l.^3;
g=c.*(K000+1/24.*l3.*(xx_2.*(toi12.*a+toi1.*b1+toi.*gama1)+yy_2.*(toi22.*a+toi2.*b2+toi.*gama2)+zz_2.*(toi32.*a+toi3.*b3+toi.*gama3)));
V=sum(g(:))*1e-5;


Lx=l3.*dx.*toi;
Ly=l3.*dy.*toi;
Lz=l3.*dz.*toi;

gx=c.*(Lx+1/24.*l3.*(xx_2.*(toi12.*a1+toi1.*b11+toi.*gama11)+yy_2.*(toi22.*a1+toi2.*b12+toi.*gama12)+zz_2.*(toi32.*a1+toi3.*b13+toi.*gama13)));
gy=c.*(Ly+1/24.*l3.*(xx_2.*(toi12.*a2+toi1.*b21+toi.*gama21)+yy_2.*(toi22.*a2+toi2.*b22+toi.*gama22)+zz_2.*(toi32.*a2+toi3.*b23+toi.*gama23)));
gz=c.*(Lz+1/24.*l3.*(xx_2.*(toi12.*a3+toi1.*b31+toi.*gama31)+yy_2.*(toi22.*a3+toi2.*b32+toi.*gama32)+zz_2.*(toi32.*a3+toi3.*b33+toi.*gama33)));
Vx=-sum(gx(:));
Vy=-sum(gy(:));
Vz=-sum(gz(:));

Kxx=l3.*(l23.*dxx-1).*toi;
Kxy=l3.*(l23.*dxy).*toi;
Kxz=l3.*(l23.*dxz).*toi;
Kyy=l3.*(l23.*dyy-1).*toi;
Kyz=l3.*(l23.*dyz).*toi;
Kzz=l3.*(l23.*dzz-1).*toi;
Lxx=c.*(Kxx+1/24.*l3.*(xx_2.*(toi12.*a11+toi1.*b111+toi.*gama111)+yy_2.*(toi22.*a11+toi2.*b112+toi.*gama112)+zz_2.*(toi32.*a11+toi3.*b113+toi.*gama113)));
Vxx=sum(Lxx(:))*1e4;
Lxy=c.*(Kxy+1/24.*l3.*(xx_2.*(toi12.*a12+toi1.*b121+toi.*gama121)+yy_2.*(toi22.*a12+toi2.*b122+toi.*gama122)+zz_2.*(toi32.*a12+toi3.*b123+toi.*gama123)));
Vxy=sum(Lxy(:))*1e4;
Lxz=c.*(Kxz+1/24.*l3.*(xx_2.*(toi12.*a13+toi1.*b131+toi.*gama131)+yy_2.*(toi22.*a13+toi2.*b132+toi.*gama132)+zz_2.*(toi32.*a13+toi3.*b133+toi.*gama133)));
Vxz=sum(Lxz(:))*1e4;
Lyy=c.*(Kyy+1/24.*l3.*(xx_2.*(toi12.*a22+toi1.*b221+toi.*gama221)+yy_2.*(toi22.*a22+toi2.*b222+toi.*gama222)+zz_2.*(toi32.*a22+toi3.*b223+toi.*gama223)));
Vyy=sum(Lyy(:))*1e4;
Lyz=c.*(Kyz+1/24.*l3.*(xx_2.*(toi12.*a23+toi1.*b231+toi.*gama231)+yy_2.*(toi22.*a23+toi2.*b232+toi.*gama232)+zz_2.*(toi32.*a23+toi3.*b233+toi.*gama233)));
Vyz=sum(Lyz(:))*1e4;
Lzz=c.*(Kzz+1/24.*l3.*(xx_2.*(toi12.*a33+toi1.*b331+toi.*gama331)+yy_2.*(toi22.*a33+toi2.*b332+toi.*gama332)+zz_2.*(toi32.*a33+toi3.*b333+toi.*gama333)));
Vzz=sum(Lzz(:))*1e4;
% Vzz=-(Vxx+Vyy);

end

function [V]=Tesseroid_g(lon1,lat1,h1,ref,lon,lat,r0,resolution_x1,resolution_y1,e)
%% compute topographic potential of finite element -- tesseroid
Pi=3.141592654;
d2r=Pi/180;
G = 6.67384e-3;   % Universal Gravitational constant


%% build tesseroid
  zz1=lon1-resolution_x1/2;
  zz2=lon1+resolution_x1/2;
  yy1=lat1-resolution_y1/2;
  yy2=lat1+resolution_y1/2;
  xx1=h1;
  xx2=ref; 
 r1=(xx1+xx2)/2;
 %% variable definition 
 lat=lat*d2r;
 lat1=lat1*d2r;
 coslat=cos(lat);
 coslat1=cos(lat1);
 sinlat=sin(lat);
 sinlat1=sin(lat1);
d_lon=(lon1-lon)*d2r;
cosdlon=cos(d_lon);
sindlon=sin(d_lon);
cosphicosa=coslat1.*cosdlon;
coslatsinlat1=coslat.*sinlat1;
sinphicosa=coslatsinlat1-sinlat.*cosphicosa;
sinphisina=coslat1.*sindlon;
cosphi=sinlat.*sinlat1+coslat.*cosphicosa;
dx=r1.*sinphicosa;
dy=r1.*sinphisina;
dz=r1.*cosphi-r0;
dxx=dx.*dx;
dyy=dy.*dy;
dzz=dz.*dz;
l=sqrt(dxx+dyy+dzz);
xx=xx2-xx1;
yy=(yy2-yy1)*d2r;
zz=(zz2-zz1)*d2r;
c=G*e.*zz.*xx.*yy;
l2=l.*l;
%% Toi
r12=r1.*r1;
toi=r12.*coslat1;

toi1=2*r1.*coslat1;
toi2=-r12.*sinlat1;
toi3=0;

toi12=2.*coslat1;
toi22=-toi;
toi32=0;

%% dxi

x3_2=r1.*(sinlat.*coslat1-coslatsinlat1.*cosdlon);
x3_22=-r1.*cosphi;
x2_3=r1.*cosphicosa;

x3_3=-(dy).*coslat;
x3_32=-x2_3.*coslat;
%% ml
ml1=r0.*cosphi-r1;
ml2=r0.*x3_2;
ml3=r0.*x3_3;

ml12=-1;
ml22=r0.*x3_22;
ml32=r0.*x3_32;
%% a
l23=3./l2;

%% b
b1=2*ml1;
b2=2*ml2;
b3=2*ml3;

%% gama
ml1_2=ml1.*ml1;
ml2_2=ml2.*ml2;
ml3_2=ml3.*ml3;
gama1=ml12+l23.*ml1_2;
gama2=ml22+l23.*ml2_2;
gama3=ml32+l23.*ml3_2;

%% topographical potential
xx_2=(xx).*xx;
yy_2=(yy).*yy;
zz_2=(zz).*zz;
K000=1./l.*toi;
l3=1./l.^3;
g=c.*(K000+1/24.*l3.*(xx_2.*(toi12.*l2+toi1.*b1+toi.*gama1)+yy_2.*(toi22.*l2+toi2.*b2+toi.*gama2)+zz_2.*(toi32.*l2+toi3.*b3+toi.*gama3)));
V=sum(g(:))*1e-5;

end

function [V, Vx, Vy, Vz]=Tesseroid_g_gx(lon1,lat1,h1,ref,lon,lat,r0,resolution_x1,resolution_y1,e)
%% compute topographic potential first derivatives of finite element -- tesseroid

Pi=3.141592654;
d2r=Pi/180;
G = 6.67384e-3;   % Universal Gravitational constant


%% build tesseroid
  zz1=lon1-resolution_x1/2;
  zz2=lon1+resolution_x1/2;
  yy1=lat1-resolution_y1/2;
  yy2=lat1+resolution_y1/2;
  xx1=h1;
  xx2=ref;

  r1=(xx1+xx2)/2;
 %% variable definition 
 lat=lat*d2r;
 lat1=lat1*d2r;
 coslat=cos(lat);
 coslat1=cos(lat1);
 sinlat=sin(lat);
 sinlat1=sin(lat1);
d_lon=(lon1-lon)*d2r;
cosdlon=cos(d_lon);
sindlon=sin(d_lon);
cosphicosa=coslat1.*cosdlon;
coslatsinlat1=coslat.*sinlat1;
sinphicosa=coslatsinlat1-sinlat.*cosphicosa;
sinphisina=coslat1.*sindlon;
sinlatsinlat1=sinlat.*sinlat1;
cosphi=sinlatsinlat1+coslat.*cosphicosa;
dx=r1.*sinphicosa;
dy=r1.*sinphisina;
dz=r1.*cosphi-r0;
dxx=dx.*dx;
dyy=dy.*dy;
dzz=dz.*dz;
l=sqrt(dxx+dyy+dzz);
xx=xx2-xx1;
yy=(yy2-yy1)*d2r;
zz=(zz2-zz1)*d2r;
c=G*e.*zz.*xx.*yy;
l2=l.*l;
%% Toi
r12=r1.*r1;
toi=r12.*coslat1;

toi1=2*r1.*coslat1;
toi2=-r12.*sinlat1;
toi3=0;

toi12=2*coslat1;
toi22=-toi;
toi32=0;
%% dxi
x1_1=sinphicosa;
x2_1=sinphisina;
x3_1=cosphi;

x1_12=0;
x2_12=0;
x3_12=0;

x1_2=r1.*(coslat.*coslat1+sinlatsinlat1.*cosdlon);
x2_2=-r1.*sinlat1.*sindlon;
x3_2=r1.*(sinlat.*coslat1-coslatsinlat1.*cosdlon);

x1_22=-dx;
x2_22=-dy;
x3_22=-r1.*cosphi;


x1_3=(dy).*sinlat;
x2_3=r1.*cosphicosa;
x3_3=-(dy).*coslat;

x1_32=x2_3.*sinlat;
x2_32=-(dy);
x3_32=-x2_3.*coslat;

%% ml
ml1=r0.*cosphi-r1;
ml2=r0.*x3_2;
ml3=r0.*x3_3;

ml12=-1;
ml22=r0.*x3_22;
ml32=r0.*x3_32;

%% a
a=l2;
l23=3./l2;

a1=dx;
a2=dy;
a3=dz;

%% b
b1=2*ml1;
b2=2*ml2;
b3=2*ml3;

b11=2*(x1_1+l23.*dx.*ml1);
b12=2*(x1_2+l23.*dx.*ml2);
b21=2*(x2_1+l23.*dy.*ml1);
b13=2*(x1_3+l23.*dx.*ml3);
b31=2*(x3_1+l23.*dz.*ml1);
b22=2*(x2_2+l23.*dy.*ml2);
b23=2*(x2_3+l23.*dy.*ml3);
b32=2*(x3_2+l23.*dz.*ml2);
b33=2*(x3_3+l23.*dz.*ml3);

%% gama
ml1_2=ml1.*ml1;
ml2_2=ml2.*ml2;
ml3_2=ml3.*ml3;

gama1=ml12+l23.*ml1_2;
gama2=ml22+l23.*ml2_2;
gama3=ml32+l23.*ml3_2;

l25=5./l2;
gama11=x1_12+l23.*(2*x1_1.*ml1+dx.*(ml12+l25.*ml1_2));
gama12=x1_22+l23.*(2*x1_2.*ml2+dx.*(ml22+l25.*ml2_2));
gama21=x2_12+l23.*(2*x2_1.*ml1+dy.*(ml12+l25.*ml1_2));
gama13=x1_32+l23.*(2*x1_3.*ml3+dx.*(ml32+l25.*ml3_2));
gama31=x3_12+l23.*(2*x3_1.*ml1+dz.*(ml12+l25.*ml1_2));
gama22=x2_22+l23.*(2*x2_2.*ml2+dy.*(ml22+l25.*ml2_2));
gama23=x2_32+l23.*(2*x2_3.*ml3+dy.*(ml32+l25.*ml3_2));
gama32=x3_22+l23.*(2*x3_2.*ml2+dz.*(ml22+l25.*ml2_2));
gama33=x3_32+l23.*(2*x3_3.*ml3+dz.*(ml32+l25.*ml3_2));

%%  
xx_2=(xx).*xx;
yy_2=(yy).*yy;
zz_2=(zz).*zz;
K000=1./l.*toi;
l3=1./l.^3;

Lx=l3.*dx.*toi;
Ly=l3.*dy.*toi;
Lz=l3.*dz.*toi;

% topographic potential and its first-derivatives of a tesseroid
g=c.*(K000+1/24.*l3.*(xx_2.*(toi12.*a+toi1.*b1+toi.*gama1)+yy_2.*(toi22.*a+toi2.*b2+toi.*gama2)+zz_2.*(toi32.*a+toi3.*b3+toi.*gama3)));
gx=c.*(Lx+1/24.*l3.*(xx_2.*(toi12.*a1+toi1.*b11+toi.*gama11)+yy_2.*(toi22.*a1+toi2.*b12+toi.*gama12)+zz_2.*(toi32.*a1+toi3.*b13+toi.*gama13)));
gy=c.*(Ly+1/24.*l3.*(xx_2.*(toi12.*a2+toi1.*b21+toi.*gama21)+yy_2.*(toi22.*a2+toi2.*b22+toi.*gama22)+zz_2.*(toi32.*a2+toi3.*b23+toi.*gama23)));
gz=c.*(Lz+1/24.*l3.*(xx_2.*(toi12.*a3+toi1.*b31+toi.*gama31)+yy_2.*(toi22.*a3+toi2.*b32+toi.*gama32)+zz_2.*(toi32.*a3+toi3.*b33+toi.*gama33)));
V=sum(g(:))*1e-5;
Vx=-sum(gx(:));
Vy=-sum(gy(:));
Vz=-sum(gz(:));

end

function [Vx Vy Vz]=Tesseroid_gx(lon1,lat1,h1,ref,lon,lat,r0,resolution_x1,resolution_y1,e)
%% compute topographic potential first derivatives of finite element -- tesseroid

Pi=3.141592654;
d2r=Pi/180;
G = 6.67384e-3;   % Universal Gravitational constant

%% build tesseroid
  zz1=lon1-resolution_x1/2;
  zz2=lon1+resolution_x1/2;
  yy1=lat1-resolution_y1/2;
  yy2=lat1+resolution_y1/2;
  xx1=h1;
  xx2=ref;
  r1=(xx1+xx2)/2;
 %% variable definition 
 lat=lat*d2r;
 lat1=lat1*d2r;
 coslat=cos(lat);
 coslat1=cos(lat1);
 sinlat=sin(lat);
 sinlat1=sin(lat1);
d_lon=(lon1-lon)*d2r;
cosdlon=cos(d_lon);
sindlon=sin(d_lon);
cosphicosa=coslat1.*cosdlon;
coslatsinlat1=coslat.*sinlat1;
sinphicosa=coslat.*sinlat1-sinlat.*cosphicosa;
sinphisina=coslat1.*sindlon;
sinlatsinlat1=sinlat.*sinlat1;
cosphi=sinlat.*sinlat1+coslat.*cosphicosa;
dx=r1.*sinphicosa;
dy=r1.*sinphisina;
dz=r1.*cosphi-r0;
l=sqrt(dx.*dx+dy.*dy+dz.*dz);
xx=xx2-xx1;
yy=(yy2-yy1)*d2r;
zz=(zz2-zz1)*d2r;
c=G*e.*zz.*xx.*yy;
l2=l.*l;
%% Toi
r12=r1.*r1;
toi=r12.*coslat1;

toi1=2*r1.*coslat1;
toi2=-r12.*sinlat1;
toi3=0;

toi12=2*coslat1;
toi22=-toi;
toi32=0;
%% dxi
x1_1=sinphicosa;
x2_1=sinphisina;
x3_1=cosphi;

x1_12=0;
x2_12=0;
x3_12=0;

x1_2=r1.*(coslat.*coslat1+sinlatsinlat1.*cosdlon);
x2_2=-r1.*sinlat1.*sindlon;
x3_2=r1.*(sinlat.*coslat1-coslatsinlat1.*cosdlon);

 x1_22=-dx;
 x2_22=-dy;
 x3_22=-r1.*cosphi;


x1_3=dy.*sinlat;
x2_3=r1.*cosphicosa;
x3_3=-dy.*coslat;

x1_32=x2_3.*sinlat;
x2_32=-dy;
x3_32=-x2_3.*coslat;

%% ml
ml1=r0.*cosphi-r1;
ml2=r0.*x3_2;
ml3=r0.*x3_3;

ml12=-1;
ml22=r0.*x3_22;
ml32=r0.*x3_32;


%% a

a1=dx;
a2=dy;
a3=dz;
l23=3./l2;

%% b
b11=2*(x1_1+l23.*dx.*ml1);
b12=2*(x1_2+l23.*dx.*ml2);
b21=2*(x2_1+l23.*dy.*ml1);
b13=2*(x1_3+l23.*dx.*ml3);
b31=2*(x3_1+l23.*dz.*ml1);
b22=2*(x2_2+l23.*dy.*ml2);
b23=2*(x2_3+l23.*dy.*ml3);
b32=2*(x3_2+l23.*dz.*ml2);
b33=2*(x3_3+l23.*dz.*ml3);

%% gama
ml1_2=ml1.*ml1;
ml2_2=ml2.*ml2;
ml3_2=ml3.*ml3;

l25=5./l2;
gama11=x1_12+l23.*(2*x1_1.*ml1+dx.*(ml12+l25.*ml1_2));
gama12=x1_22+l23.*(2*x1_2.*ml2+dx.*(ml22+l25.*ml2_2));
gama21=x2_12+l23.*(2*x2_1.*ml1+dy.*(ml12+l25.*ml1_2));
gama13=x1_32+l23.*(2*x1_3.*ml3+dx.*(ml32+l25.*ml3_2));
gama31=x3_12+l23.*(2*x3_1.*ml1+dz.*(ml12+l25.*ml1_2));
gama22=x2_22+l23.*(2*x2_2.*ml2+dy.*(ml22+l25.*ml2_2));
gama23=x2_32+l23.*(2*x2_3.*ml3+dy.*(ml32+l25.*ml3_2));
gama32=x3_22+l23.*(2*x3_2.*ml2+dz.*(ml22+l25.*ml2_2));
gama33=x3_32+l23.*(2*x3_3.*ml3+dz.*(ml32+l25.*ml3_2));

%%  
xx_2=(xx).*xx;
yy_2=(yy).*yy;
zz_2=(zz).*zz;
l3=1./(l2.*l);

Lx=l3.*dx.*toi;
Ly=l3.*dy.*toi;
Lz=l3.*dz.*toi;
% first-derivatives of potential of a tesseroid
gx=c.*(Lx+1/24*l3.*(xx_2.*(toi12.*a1+toi1.*b11+toi.*gama11)+yy_2.*(toi22.*a1+toi2.*b12+toi.*gama12)+zz_2.*(toi32.*a1+toi3.*b13+toi.*gama13)));
gy=c.*(Ly+1/24*l3.*(xx_2.*(toi12.*a2+toi1.*b21+toi.*gama21)+yy_2.*(toi22.*a2+toi2.*b22+toi.*gama22)+zz_2.*(toi32.*a2+toi3.*b23+toi.*gama23)));
gz=c.*(Lz+1/24*l3.*(xx_2.*(toi12.*a3+toi1.*b31+toi.*gama31)+yy_2.*(toi22.*a3+toi2.*b32+toi.*gama32)+zz_2.*(toi32.*a3+toi3.*b33+toi.*gama33)));
Vx=-sum(gx(:));
Vy=-sum(gy(:));
Vz=-sum(gz(:));

end

function [Vxx, Vxy, Vxz, Vyy, Vyz,Vzz]=Tesseroid_gxx(lon1,lat1,h1,ref,lon,lat,r0,resolution_x1,resolution_y1,e)
%% compute topographic potential second derivatives of finite element -- tesseroid
Pi=3.141592654;
d2r=Pi/180;
G = 6.67384e-3;   % Universal Gravitational constant
%n=find(h1<0);
%e(n)=1.7;

%% build tesseroid
  zz1=lon1-resolution_x1/2;
  zz2=lon1+resolution_x1/2;
  yy1=lat1-resolution_y1/2;
  yy2=lat1+resolution_y1/2;
  xx1=h1;
  xx2=ref;

  r1=(xx1+xx2)/2;
 %% variable definition 
  %% variable definition 
 lat=lat*d2r;
 lat1=lat1*d2r;
 coslat=cos(lat);
 coslat1=cos(lat1);
 sinlat=sin(lat);
 sinlat1=sin(lat1);
d_lon=(lon1-lon)*d2r;
cosdlon=cos(d_lon);
sindlon=sin(d_lon);
cosphicosa=coslat1.*cosdlon;
coslatsinlat1=coslat.*sinlat1;
sinphicosa=coslatsinlat1-sinlat.*cosphicosa;
sinphisina=coslat1.*sindlon;
sinlatsinlat1=sinlat.*sinlat1;
cosphi=sinlatsinlat1+coslat.*cosphicosa;
dx=r1.*sinphicosa;
dy=r1.*sinphisina;
dz=r1.*cosphi-r0;
dxx=dx.*dx;
dxy=dx.*dy;
dxz=dx.*dz;
dyy=dy.*dy;
dyz=dy.*dz;
dzz=dz.*dz;
l=sqrt(dxx+dyy+dzz);
xx=xx2-xx1;
yy=(yy2-yy1)*d2r;
zz=(zz2-zz1)*d2r;
c=G*e.*zz.*xx.*yy;
l2=l.*l;
%% Toi
r12=r1.*r1;
toi=r12.*coslat1;

toi1=2.*r1.*coslat1;
toi2=-r12.*sinlat1;
toi3=0;

toi12=2.*coslat1;
toi22=-toi;
toi32=0;

%% dxi
x1_1=sinphicosa;
x2_1=sinphisina;
x3_1=cosphi;

x1_12=0;
x2_12=0;
x3_12=0;

x1_2=r1.*(coslat.*coslat1+sinlatsinlat1.*cosdlon);
x2_2=-r1.*sinlat1.*sindlon;
x3_2=r1.*(sinlat.*coslat1-coslatsinlat1.*cosdlon);

 x1_22=-dx;
 x2_22=-dy;
 x3_22=-r1.*cosphi;


x1_3=(dy).*sinlat;
x2_3=r1.*cosphicosa;
x3_3=-(dy).*coslat;

x1_32=x2_3.*sinlat;
x2_32=-(dy);
x3_32=-x2_3.*coslat;

%% ml
ml1=r0.*cosphi-r1;
ml2=r0.*x3_2;
ml3=r0.*x3_3;

ml12=-1;
ml22=r0.*x3_22;
ml32=r0.*x3_32;


%% a
l23=3./l2;
a11=l23.*dxx-1;
a12=l23.*dxy;
a13=l23.*dxz;
a22=l23.*dyy-1;
a23=l23.*dyz;
a33=l23.*dzz-1;

%% b
b1=2*ml1;
b2=2*ml2;
b3=2*ml3;

const_1=6./l2;
dxx5=5*dxx;
dxy5=5*dxy;
dxz5=5*dxz;
dyy5=5*dyy;
dyz5=5*dyz;
dzz5=5*dzz;
dx_x11=dx.*x1_1;
dx_x12=dx.*x1_2;
dx_x13=dx.*x1_3;
dx_x21=dx.*x2_1;
dx_x22=dx.*x2_2;
dx_x23=dx.*x2_3;
dy_x11=dy.*x1_1;
dy_x12=dy.*x1_2;
dy_x13=dy.*x1_3;
dz_x11=dz.*x1_1;
dz_x12=dz.*x1_2;
dz_x13=dz.*x1_3;
dy_x21=dy.*x2_1;
dy_x22=dy.*x2_2;
dy_x23=dy.*x2_3;
dz_x21=dz.*x2_1;
dz_x22=dz.*x2_2;
dz_x23=dz.*x2_3;

dx_x31=dx.*x3_1;
dx_x32=dx.*x3_2;
dx_x33=dx.*x3_3;
dy_x31=dy.*x3_1;
dy_x32=dy.*x3_2;
dy_x33=dy.*x3_3;
dz_x31=dz.*x3_1;
dz_x32=dz.*x3_2;
dz_x33=dz.*x3_3;

b111=const_1.*(dx_x11+dx_x11+dxx5.*ml1./l2-ml1);
b112=const_1.*(dx_x12+dx_x12+dxx5.*ml2./l2-ml2);
b113=const_1.*(dx_x13+dx_x13+dxx5.*ml3./l2-ml3);
b121=const_1.*(dx_x21+dy_x11+dxy5.*ml1./l2);
b122=const_1.*(dx_x22+dy_x12+dxy5.*ml2./l2);
b123=const_1.*(dx_x23+dy_x13+dxy5.*ml3./l2);
b131=const_1.*(dx_x31+dz_x11+dxz5.*ml1./l2);
b132=const_1.*(dx_x32+dz_x12+dxz5.*ml2./l2);
b133=const_1.*(dx_x33+dz_x13+dxz5.*ml3./l2);
b221=const_1.*(dy_x21+dy_x21+dyy5.*ml1./l2-ml1);
b222=const_1.*(dy_x22+dy_x22+dyy5.*ml2./l2-ml2);
b223=const_1.*(dy_x23+dy_x23+dyy5.*ml3./l2-ml3);
b231=const_1.*(dy_x31+dz_x21+dyz5.*ml1./l2);
b232=const_1.*(dy_x32+dz_x22+dyz5.*ml2./l2);
b233=const_1.*(dy_x33+dz_x23+dyz5.*ml3./l2);
b331=const_1.*(dz_x31+dz_x31+dzz5.*ml1./l2-ml1);
b332=const_1.*(dz_x32+dz_x32+dzz5.*ml2./l2-ml2);
b333=const_1.*(dz_x33+dz_x33+dzz5.*ml3./l2-ml3);

%% gama
ml1_2=ml1.*ml1;
ml2_2=ml2.*ml2;
ml3_2=ml3.*ml3;
l25=5./l2;
l27=7./l2;

dx_x112=dx.*x1_12;
dx_x122=dx.*x1_22;
dx_x132=dx.*x1_32;
dx_x212=dx.*x2_12;
dx_x222=dx.*x2_22;
dx_x232=dx.*x2_32;
dx_x312=dx.*x3_12;
dx_x322=dx.*x3_22;
dx_x332=dx.*x3_32;

dy_x112=dy.*x1_12;
dy_x122=dy.*x1_22;
dy_x132=dy.*x1_32;
dy_x212=dy.*x2_12;
dy_x222=dy.*x2_22;
dy_x232=dy.*x2_32;
dy_x312=dy.*x3_12;
dy_x322=dy.*x3_22;
dy_x332=dy.*x3_32;

dz_x112=dz.*x1_12;
dz_x122=dz.*x1_22;
dz_x132=dz.*x1_32;
dz_x212=dz.*x2_12;
dz_x222=dz.*x2_22;
dz_x232=dz.*x2_32;
dz_x312=dz.*x3_12;
dz_x322=dz.*x3_22;
dz_x332=dz.*x3_32;


gama111=l23.*(dx_x112+dx_x112+2*x1_1.*x1_1+l25.*(b1.*(dx_x11+dx_x11)+dxx.*(ml12+l27.*(ml1_2)))-(ml12+l25.*(ml1_2)));
gama112=l23.*(dx_x122+dx_x122+2*x1_2.*x1_2+l25.*(b2.*(dx_x12+dx_x12)+dxx.*(ml22+l27.*(ml2_2)))-(ml22+l25.*(ml2_2)));
gama113=l23.*(dx_x132+dx_x132+2*x1_3.*x1_3+l25.*(b3.*(dx_x13+dx_x13)+dxx.*(ml32+l27.*(ml3_2)))-(ml32+l25.*(ml3_2)));
gama121=l23.*(dx_x212+dy_x112+2*x1_1.*x2_1+l25.*(b1.*(dx_x21+dy_x11)+dxy.*(ml12+l27.*(ml1_2))));
gama122=l23.*(dx_x222+dy_x122+2*x1_2.*x2_2+l25.*(b2.*(dx_x22+dy_x12)+dxy.*(ml22+l27.*(ml2_2))));
gama123=l23.*(dx_x232+dy_x132+2*x1_3.*x2_3+l25.*(b3.*(dx_x23+dy_x13)+dxy.*(ml32+l27.*(ml3_2))));
gama131=l23.*(dx_x312+dz_x112+2*x1_1.*x3_1+l25.*(b1.*(dx_x31+dz_x11)+dxz.*(ml12+l27.*(ml1_2))));
gama132=l23.*(dx_x322+dz_x122+2*x1_2.*x3_2+l25.*(b2.*(dx_x32+dz_x12)+dxz.*(ml22+l27.*(ml2_2))));
gama133=l23.*(dx_x332+dz_x132+2*x1_3.*x3_3+l25.*(b3.*(dx_x33+dz_x13)+dxz.*(ml32+l27.*(ml3_2))));
gama221=l23.*(dy_x212+dy_x212+2*x2_1.*x2_1+l25.*(b1.*(dy_x21+dy_x21)+dyy.*(ml12+l27.*(ml1_2)))-(ml12+l25.*(ml1_2)));
gama222=l23.*(dy_x222+dy_x222+2*x2_2.*x2_2+l25.*(b2.*(dy_x22+dy_x22)+dyy.*(ml22+l27.*(ml2_2)))-(ml22+l25.*(ml2_2)));
gama223=l23.*(dy_x232+dy_x232+2*x2_3.*x2_3+l25.*(b3.*(dy_x23+dy_x23)+dyy.*(ml32+l27.*(ml3_2)))-(ml32+l25.*(ml3_2)));
gama231=l23.*(dy_x312+dz_x212+2*x2_1.*x3_1+l25.*(b1.*(dy_x31+dz_x21)+dyz.*(ml12+l27.*(ml1_2))));
gama232=l23.*(dy_x322+dz_x222+2*x2_2.*x3_2+l25.*(b2.*(dy_x32+dz_x22)+dyz.*(ml22+l27.*(ml2_2))));
gama233=l23.*(dy_x332+dz_x232+2*x2_3.*x3_3+l25.*(b3.*(dy_x33+dz_x23)+dyz.*(ml32+l27.*(ml3_2))));
gama331=l23.*(dz_x312+dz_x312+2*x3_1.*x3_1+l25.*(b1.*(dz_x31+dz_x31)+dzz.*(ml12+l27.*(ml1_2)))-(ml12+l25.*(ml1_2)));
gama332=l23.*(dz_x322+dz_x322+2*x3_2.*x3_2+l25.*(b2.*(dz_x32+dz_x32)+dzz.*(ml22+l27.*(ml2_2)))-(ml22+l25.*(ml2_2)));
gama333=l23.*(dz_x332+dz_x332+2*x3_3.*x3_3+l25.*(b3.*(dz_x33+dz_x33)+dzz.*(ml32+l27.*(ml3_2)))-(ml32+l25.*(ml3_2)));

%%  
xx_2=(xx).*xx;
yy_2=(yy).*yy;
zz_2=(zz).*zz;
l3=1./l.^3;
%% second-derivatives of potential of a tesseroid
Kxx=l3.*(l23.*dxx-1).*toi;
Kxy=l3.*(l23.*dxy).*toi;
Kxz=l3.*(l23.*dxz).*toi;
Kyy=l3.*(l23.*dyy-1).*toi;
Kyz=l3.*(l23.*dyz).*toi;
Kzz=l3.*(l23.*dzz-1).*toi;
Lxx=c.*(Kxx+1/24.*l3.*(xx_2.*(toi12.*a11+toi1.*b111+toi.*gama111)+yy_2.*(toi22.*a11+toi2.*b112+toi.*gama112)+zz_2.*(toi32.*a11+toi3.*b113+toi.*gama113)));
Vxx=sum(Lxx)*1e4;
Lxy=c.*(Kxy+1/24.*l3.*(xx_2.*(toi12.*a12+toi1.*b121+toi.*gama121)+yy_2.*(toi22.*a12+toi2.*b122+toi.*gama122)+zz_2.*(toi32.*a12+toi3.*b123+toi.*gama123)));
Vxy=sum(Lxy)*1e4;
Lxz=c.*(Kxz+1/24.*l3.*(xx_2.*(toi12.*a13+toi1.*b131+toi.*gama131)+yy_2.*(toi22.*a13+toi2.*b132+toi.*gama132)+zz_2.*(toi32.*a13+toi3.*b133+toi.*gama133)));
Vxz=sum(Lxz)*1e4;
Lyy=c.*(Kyy+1/24.*l3.*(xx_2.*(toi12.*a22+toi1.*b221+toi.*gama221)+yy_2.*(toi22.*a22+toi2.*b222+toi.*gama222)+zz_2.*(toi32.*a22+toi3.*b223+toi.*gama223)));
Vyy=sum(Lyy)*1e4;
Lyz=c.*(Kyz+1/24.*l3.*(xx_2.*(toi12.*a23+toi1.*b231+toi.*gama231)+yy_2.*(toi22.*a23+toi2.*b232+toi.*gama232)+zz_2.*(toi32.*a23+toi3.*b233+toi.*gama233)));
Vyz=sum(Lyz)*1e4;
Lzz=c.*(Kzz+1/24.*l3.*(xx_2.*(toi12.*a33+toi1.*b331+toi.*gama331)+yy_2.*(toi22.*a33+toi2.*b332+toi.*gama332)+zz_2.*(toi32.*a33+toi3.*b333+toi.*gama333)));
Vzz=sum(Lzz)*1e4;
% Vzz=-(Vxx+Vyy);
end

function [var1, var2, var3, var4, var5, var6, var7, var8, var9, var10] = matrix_preallocating(m,n)
var1=zeros(m,n);
var2=zeros(m,n);
var3=zeros(m,n);
var4=zeros(m,n);
var5=zeros(m,n);
var6=zeros(m,n);
var7=zeros(m,n);
var8=zeros(m,n);
var9=zeros(m,n);
var10=zeros(m,n);
end